package pwc.taxtech.atms.common;

import java.io.File;
import java.util.Properties;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.apache.commons.io.FileUtils;
import org.nutz.lang.Encoding;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/** Web程序启动和结束时输出日志用于问题诊断 */
public class MyServletContextListener implements ServletContextListener {
    private static Logger logger = LoggerFactory.getLogger(MyServletContextListener.class);

    @Override
    public void contextInitialized(ServletContextEvent sce) {
        logger.info("### Starting web application ###");
        if (logger.isDebugEnabled()) {
            ServletContext servletContext = sce.getServletContext();
            Properties sys = System.getProperties();
            logger.debug("Web Container Information:");
            logger.debug(" - Default Charset : {}", Encoding.defaultEncoding());
            logger.debug(" - Current . path  : {}", new File(".").getAbsolutePath());
            logger.debug(" - Temp Directory  : {}", FileUtils.getTempDirectory().getAbsolutePath());
            logger.debug(" - Java Version    : {}", sys.get("java.version"));
            logger.debug(" - File separator  : {}", sys.get("file.separator"));
            logger.debug(" - Timezone        : {}", sys.get("user.timezone"));
            logger.debug(" - OS              : {} {}", sys.get("os.name"), sys.get("os.arch"));
            logger.debug(" - ServerInfo      : {}", servletContext.getServerInfo());
            logger.debug(" - Servlet API     : {}.{}", servletContext.getMajorVersion(),
                    servletContext.getMinorVersion());
            if (servletContext.getMajorVersion() > 2 || servletContext.getMinorVersion() > 4)
                logger.debug(" - ContextPath     : {}", servletContext.getContextPath());
        }
    }

    @Override
    public void contextDestroyed(ServletContextEvent sce) {
        logger.info("### Web application Destroyed ###");
    }
}
