package pwc.taxtech.atms.common.schedule;

import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.quartz.QuartzJobBean;
import pwc.taxtech.atms.analysis.entity.AnalysisMaster;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.EnumTbImportType;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.OrganizationExample;
import pwc.taxtech.atms.service.impl.AnalysisJobServiceImpl;
import pwc.taxtech.atms.service.impl.AnalysisServiceImpl;

import javax.annotation.Resource;
import java.util.List;

public class AnalysisJob extends QuartzJobBean {
    private static final Logger logger = LoggerFactory.getLogger(AnalysisJob.class);

    @Resource
    private OrganizationMapper organizationMapper;

    @Value("${org_sync_url}")
    private String orgSyncUrl;

    @Value("${org_sync_token}")
    private String token;

    @Autowired
    private AnalysisJobServiceImpl analysisJobService;

    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        Integer period = DateUtils.getPeriodNow();
        OrganizationExample e = new OrganizationExample();
        //e.createCriteria().andIsActiveEqualTo(true);
        List<Organization> orgs = organizationMapper.selectByExample(e);

        logger.info(String.format("开始分析%s机构数据",period));
        analysisJobService.analysisMaster(orgs,period, EnumTbImportType.CoverImport.getCode());

        logger.info(String.format("开始分析%s预期返还税数据",period));
        analysisJobService.analysisExpectedTax(orgs,period, EnumTbImportType.CoverImport.getCode());

        logger.info(String.format("开始分析%s费用数据",period));
        analysisJobService.analysisFee(orgs,period, EnumTbImportType.CoverImport.getCode());

        logger.info(String.format("开始分析%s档案管理数据",period));
        analysisJobService.analysisFileManagement(orgs,period, EnumTbImportType.CoverImport.getCode());

        logger.info(String.format("开始分析%s申报表数据",period));
        analysisJobService.analysisSales(orgs,period, EnumTbImportType.CoverImport.getCode());

        logger.info(String.format("开始分析%s返还后税数据",period));
        analysisJobService.analysisTaxReturnEnd(orgs,period, EnumTbImportType.CoverImport.getCode());

        logger.info(String.format("开始分析%s税种税金计算数据",period));
        analysisJobService.analysisTax(orgs,period, EnumTbImportType.CoverImport.getCode());

    }

}
