package pwc.taxtech.atms.common.schedule;

import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.quartz.QuartzJobBean;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.service.impl.LgGlBalanceService;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.List;

public class LgGlBalanceJob extends QuartzJobBean {
    private static final Logger LOGGER = LoggerFactory.getLogger(LgGlBalanceJob.class);
    public static final String ORG_ID = "org_id";
    public static final String PERIOD = "period";

    @Autowired
    private OrganizationMapper orgMapper;
    @Resource
    private LgGlBalanceService lgGlBalanceService;

    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        JobDataMap dataMap = jobExecutionContext.getJobDetail().getJobDataMap();
        String orgId = dataMap.getString(ORG_ID);
        String period = dataMap.getString(PERIOD);
        List<Organization> orgList;
        if (StringUtils.isBlank(orgId)) {
            orgList = orgMapper.selectByExample(null);
        } else {
            orgList = Lists.newArrayList(orgMapper.selectByPrimaryKey(orgId));
        }
        for (Organization organization : orgList) {
            try {
                SimpleDateFormat dateFormat = new SimpleDateFormat(Constant.DateFormat.YEAR_MONTH);
                Calendar calendar = Calendar.getInstance();
                calendar.set(Calendar.MONTH, -1);
                period = StringUtils.isBlank(period) ? dateFormat.format(calendar.getTime()) : period;
                lgGlBalanceService.queryGlBalance(organization, period);
            } catch (Exception e) {
                LOGGER.error("LgGlBalanceJob error. Organization: " + organization.getId(), e);
            }
        }
    }

}
