package pwc.taxtech.atms.common.util;

import com.esotericsoftware.reflectasm.MethodAccess;
import org.apache.commons.lang3.StringUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

@Component
public class BeanUtil {

    private static final String PREFIX_GET = "get";
    private static final String PREFIX_SET = "set";

    /**
     * 拷贝属性
     *
     * @param sourceObject 数据源
     * @param targetObject 目标对象
     * @return T
     */
    public <T> T copyProperties(Object sourceObject, T targetObject) {
        if (sourceObject == null) {
            throw new IllegalArgumentException("sourceObject is null");
        }
        if (targetObject == null) {
            throw new IllegalArgumentException("targetObject is null");
        }

        MethodAccess targetMa = getMethodAccess(targetObject.getClass());
        List<Field> targetFieldList = getFieldList(targetObject.getClass());

        MethodAccess sourceMa = getMethodAccess(sourceObject.getClass());
        List<Field> sourceFieldList = getFieldList(sourceObject.getClass());
        for (Field field : targetFieldList) {
            String name = field.getName();
            if (sourceFieldList.stream().noneMatch(x -> StringUtils.equals(x.getName(), name))) {
                //不包含此属性 跳过
                continue;
            }
            String upperName = upperCaseFirstChar(name);
            try {
                targetMa.invoke(targetObject, PREFIX_SET + upperName,
                        sourceMa.invoke(sourceObject, PREFIX_GET + upperName));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return targetObject;
    }

    @Cacheable(value = "reflectCache", key = "'ma_' + #className.getName()")
    public MethodAccess getMethodAccess(Class className) {
        return MethodAccess.get(className);
    }

    @Cacheable(value = "reflectCache", key = "'fa_' + #className.getName()")
    public List<Field> getFieldList(Class className) {
        List<Field> fields = new ArrayList<>();
        Class nextClass = className;
        while (nextClass != Object.class) {
            Field[] declaredFields = nextClass.getDeclaredFields();
            for (int i = 0, n = declaredFields.length; i < n; i++) {
                Field field = declaredFields[i];
                // 反射所有
                int modifiers = field.getModifiers();
                if (Modifier.isStatic(modifiers)) continue;
//                if (Modifier.isPrivate(modifiers)) continue;
                fields.add(field);
            }
            nextClass = nextClass.getSuperclass();
        }
        return fields;
    }


    /**
     * 首字母大写
     *
     * @param str str
     * @return str
     */
    public String upperCaseFirstChar(String str) {
        char[] ch = str.toCharArray();
        if (ch[0] >= 'a' && ch[0] <= 'z') {
            ch[0] = (char) (ch[0] - 32);
        }
        return new String(ch);
    }




}
