package pwc.taxtech.atms.common.util;

/**
 * @ClassName FileExcelUtil
 * Description TODO
 * @Author pwc kevin
 * @Date 3/31/2019 12:35 PM
 * Version 1.0
 **/

import com.google.common.collect.Lists;
import org.activiti.engine.ProcessEngine;
import org.activiti.engine.ProcessEngineConfiguration;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.streaming.SXSSFSheet;
import org.nutz.http.Http;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import sun.misc.BASE64Encoder;

import javax.servlet.ServletContext;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

public class FileExcelUtil {

    private static final Logger logger = LoggerFactory.getLogger(FileExcelUtil.class);

    /**
     * 编译下载的文件名
     *
     * @param filename
     * @param agent
     * @return
     * @throws IOException
     */
    public static String encodeDownloadFilename(String filename, String agent) throws IOException {
        if (agent.contains("Firefox")) { // 火狐浏览器
            filename = "=?UTF-8?B?"
                    + new BASE64Encoder().encode(filename.getBytes("utf-8"))
                    + "?=";
            filename = filename.replaceAll("\r\n", "");
        } else { // IE及其他浏览器
            filename = URLEncoder.encode(filename, "utf-8");
            filename = filename.replace("+", " ");
        }
        return filename;
    }

    /**
     * 创建文件夹;
     *
     * @param path
     */
    public static File createFile(String path) {
        File file = new File(path);
        //判断文件是否存在;
        if (!file.exists()) {
            //创建文件;
            file.mkdirs();
        }
        return file;
    }

    /**
     * 生成.zip文件;
     *
     * @param path
     * @throws IOException
     */
    public static File createZip(String path) throws IOException {
        ZipOutputStream zipOutputStream = null;
        File file = new File(path + DateUtils.nowDateFormat() + ".zip");
        zipOutputStream = new ZipOutputStream(new BufferedOutputStream(new FileOutputStream(file)));
        File[] files = new File(path).listFiles();
        FileInputStream fileInputStream = null;
        byte[] buf = new byte[1024];
        int len = 0;
        if (files != null && files.length > 0) {
            for (File excelFile : files) {
                String fileName = excelFile.getName();
                fileInputStream = new FileInputStream(excelFile);
                //放入压缩zip包中;
                zipOutputStream.putNextEntry(new ZipEntry(path + "/" + fileName));
                //读取文件;
                while ((len = fileInputStream.read(buf)) > 0) {
                    zipOutputStream.write(buf, 0, len);
                }
                //关闭;
                zipOutputStream.closeEntry();
                if (fileInputStream != null) {
                    fileInputStream.close();
                }
            }
        }

        /*if(zipOutputStream !=null){
            zipOutputStream.close();
        }*/
        return file;
    }


    /**
     * 设置下载excel的响应头信息
     *
     * @param response
     * @param request
     * @param fileName
     * @throws IOException
     */
    public static void setExcelHeadInfo(HttpServletResponse response, HttpServletRequest request, String fileName) {
        try {
            // 获取客户端浏览器的类型
            String agent = request.getHeader("User-Agent");
            // 对文件名重新编码
            String encodingFileName = FileExcelUtil.encodeDownloadFilename(fileName, agent);
            // 告诉客户端允许断点续传多线程连接下载
            response.setHeader("Accept-Ranges", "bytes");
            //文件后缀
            response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
            response.setHeader("Content-Disposition", "attachment; filename=" + encodingFileName);
        } catch (IOException e) {
            logger.error(Thread.currentThread().getStackTrace()[1].getMethodName() + "发生的异常是: ", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * 设置下载zip的响应头信息
     *
     * @param response
     * @param fileName 文件名
     * @param request
     * @throws IOException
     */
    public static void setZipDownLoadHeadInfo(HttpServletResponse response, HttpServletRequest request, String fileName) throws IOException {
        // 获取客户端浏览器的类型
        String agent = request.getHeader("User-Agent");
        response.setContentType("application/octet-stream ");
        // 表示不能用浏览器直接打开
        response.setHeader("Connection", "close");
        // 告诉客户端允许断点续传多线程连接下载
        response.setHeader("Accept-Ranges", "bytes");
        // 对文件名重新编码
        String encodingFileName = FileExcelUtil.encodeDownloadFilename(fileName, agent);
        response.setHeader("Content-Disposition", "attachment; filename=" + encodingFileName);
    }

    /**
     * 下载excel
     *
     * @param request
     * @param response
     * @param fileName
     * @param workbook
     * @time 2018年6月25日11:47:07
     */
    public static void downloadExcel(HttpServletRequest request, HttpServletResponse response, String fileName, Workbook workbook) {
        //一个流两个头
        //设置下载excel的头信息
        FileExcelUtil.setExcelHeadInfo(response, request, fileName);

        // 写出文件
        ServletOutputStream os = null;
        try {
            os = response.getOutputStream();
            workbook.write(os);
        } catch (IOException e) {
            logger.error(Thread.currentThread().getStackTrace()[1].getMethodName() + "发生的异常是: ", e);
            throw new RuntimeException(e);
        } finally {
            try {
                if (os != null) {
                    os.flush();
                    os.close();
                }
                if (workbook != null) {
                    workbook.close();
                }
            } catch (Exception e1) {
                logger.error(Thread.currentThread().getStackTrace()[1].getMethodName() + "发生的异常是: ", e1);
                throw new RuntimeException(e1);
            }
        }
    }

    /**
     * 生成excel到指定路径
     *
     * @param wb
     * @throws Exception
     * @Param path 文件路径，包括zip文件夹路径
     */
    public static File generateExcelToPath(Workbook wb, String fileNamet) throws Exception {
        File file = new File(fileNamet);
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(file);
            wb.write(fos);
            return file;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        } finally {
            if (fos != null) {
                fos.flush();
                fos.close();
            }
            if (wb != null) {
                wb.close();
            }
        }
    }

    /**
     * 将批量文件打包下载成zip
     *
     * @param request
     * @param response
     * @param zipName  下载的zip名
     * @param files    要打包的批量文件
     * @param zipDir   存放zip文件的文件夹路径
     * @throws Exception
     */
    public static synchronized void downloadZip(HttpServletRequest request, HttpServletResponse response, String zipName, List<File> files, String zipDir) throws Exception {
        //ZIPPATH = this.getClass().getResource("/").getPath().substring(1) + "zipDir";
        FileExcelUtil.createFile(zipDir);// 先生成存放zip文件的文件夹
        String zipPath = zipDir + "/" + Math.random() + ".zip";
        File srcfile[] = new File[files.size()];
        File zip = new File(zipPath);
        for (int i = 0; i < files.size(); i++) {
            srcfile[i] = files.get(i);
        }
        FileInputStream inStream = null;
        ServletOutputStream os = null;
        try {
            //设置下载zip的头信息
            FileExcelUtil.setZipDownLoadHeadInfo(response, request, zipName);
            os = response.getOutputStream();
            FileExcelUtil.ZipFiles(srcfile, zip);
            inStream = new FileInputStream(zip);
            byte[] buf = new byte[4096];
            int readLength;
            while (((readLength = inStream.read(buf)) != -1)) {
                os.write(buf, 0, readLength);
            }
        } finally {
            if (inStream != null) {
                inStream.close();
            }
            if (os != null) {
                os.flush();
                os.close();
            }
            deleteDir(zip);
        }
    }

    /**
     * //压缩文件
     *
     * @param srcfile 要压缩的文件数组
     * @param zipfile 生成的zip文件对象
     */
    public static void ZipFiles(java.io.File[] srcfile, File zipfile) throws Exception {
        byte[] buf = new byte[1024];
        FileOutputStream fos = new FileOutputStream(zipfile);
        ZipOutputStream out = new ZipOutputStream(fos);
        for (int i = 0; i < srcfile.length; i++) {
            FileInputStream in = new FileInputStream(srcfile[i]);
            out.putNextEntry(new ZipEntry(srcfile[i].getName()));
            int len;
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
            out.closeEntry();
            in.close();
        }
        out.close();
        fos.flush();
        fos.close();
    }

    /**
     * 删除文件夹及文件夹下所有文件
     *
     * @param dir
     * @return
     */
    public static boolean deleteDir(File dir) {
        if (dir == null || !dir.exists()) {
            return true;
        }
        if (dir.isDirectory()) {
            String[] children = dir.list();
            //递归删除目录中的子目录下
            for (int i = 0; i < children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        // 目录此时为空，可以删除
        return dir.delete();
    }


    /**
     * 删除列
     *
     * @param sheet
     * @param columnToDelete
     * @param cols           制定哪些行不进行列偏移
     */
    public static void deleteColumn(Sheet sheet, int columnToDelete, List<Integer> cols) {
        for (int rId = 0; rId <= sheet.getLastRowNum(); rId++) {
            Row row = sheet.getRow(rId);
            if(row == null)
                continue;
            for (int cID = columnToDelete; cID <= row.getLastCellNum(); cID++) {
                Cell cOld = row.getCell(cID);
                if (cOld != null) {
                    row.removeCell(cOld);
                }
                Cell cNext = row.getCell(cID + 1);
                if (cNext != null) {
                    Cell cNew = row.createCell(cID, cNext.getCellTypeEnum());
                    if (cols.contains(cID))
                        continue;
                    cloneCell(cNew, cNext);
                    //Set the column width only on the first row.
                    //Other wise the second row will overwrite the original column width set previously.
                    if (rId == 0) {
                        sheet.setColumnWidth(cID, sheet.getColumnWidth(cID + 1));
                    }
                }
            }
        }
    }


    /**
     * 右边列左移
     *
     * @param cNew
     * @param cOld
     */
    public static void cloneCell(Cell cNew, Cell cOld) {
        try {
            if(cOld == null || cNew == null)
                return;;
            cNew.setCellComment(cOld.getCellComment());
            cNew.setCellStyle(cOld.getCellStyle());
            try {
                String stringCellValue = cOld.getStringCellValue();
                if ("".equals(stringCellValue))
                    return;
            } catch (Exception e) {
                //do nothing
            }
            if (CellType.BOOLEAN == cNew.getCellTypeEnum() || CellType.BOOLEAN == cOld.getCellTypeEnum()) {
                cNew.setCellValue(cOld.getBooleanCellValue());
            } else if (CellType.NUMERIC == cNew.getCellTypeEnum() || CellType.NUMERIC == cOld.getCellTypeEnum()) {
                cNew.setCellValue(cOld.getNumericCellValue());
            } else if (CellType.STRING == cNew.getCellTypeEnum() || CellType.STRING == cOld.getCellTypeEnum()) {
                cNew.setCellValue(cOld.getStringCellValue());
            } else if (CellType.ERROR == cNew.getCellTypeEnum() || CellType.ERROR == cOld.getCellTypeEnum()) {
                cNew.setCellValue(cOld.getErrorCellValue());
            } else if (CellType.FORMULA == cNew.getCellTypeEnum() || CellType.FORMULA == cOld.getCellTypeEnum()) {
                cNew.setCellValue(cOld.getCellFormula());
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.warn("数据转换异常", e.getMessage());
        }
    }

    /**
     * @param cell 获取值的单元格
     * @param zero 是否将空 或者null转换成  zero
     * @return
     */
    public static Object getCellValue(Cell cell, Boolean zero) {
        String cellValue = null;
        switch (cell.getCellType()) {
            case Cell.CELL_TYPE_STRING://字符串类型
                cellValue = cell.getStringCellValue();
                if (cellValue.trim().equals("") || cellValue.trim().length() <= 0)
                    cellValue = "";
                break;
            case Cell.CELL_TYPE_NUMERIC: //数值类型
                cellValue = String.valueOf(cell.getNumericCellValue());
                break;
            case Cell.CELL_TYPE_FORMULA: //公式
                cell.setCellType(HSSFCell.CELL_TYPE_NUMERIC);
                cellValue = String.valueOf(cell.getNumericCellValue());
                break;
            case Cell.CELL_TYPE_BLANK:
                cellValue = "";
                break;
            case Cell.CELL_TYPE_BOOLEAN:
                break;
            case Cell.CELL_TYPE_ERROR:
                break;
            default:
                break;
        }
        if ("".equals(cellValue) && zero)
            return BigDecimal.ZERO;
        if (zero) {
            try {
                return new BigDecimal(cellValue);
            } catch (Exception e) {
                logger.warn("获取Cell，在值转换成数字的地方出错");
                return BigDecimal.ZERO;
            }
        }
        return cellValue;
    }


}