package pwc.taxtech.atms.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.CitAssetsListDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.CitAssetGroupResult;
import pwc.taxtech.atms.entity.CitAssetsList;
import pwc.taxtech.atms.service.impl.AssetListServiceImpl;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * @Description 资产相关Controller
 * @Author zhikai.z.wei
 */
@RestController
@RequestMapping("/api/v1/asset/")
public class AssetListController {
    private static Logger logger = LoggerFactory.getLogger(AssetListController.class);

    @Autowired
    private AssetListServiceImpl assetListService;

    /**
     *
     * @return
     */
    @RequestMapping(value = "/getAssetListData", method = RequestMethod.GET)
    public @ResponseBody
    ApiResultDto  getAssetListData(){
        logger.info("获取所有资产清单");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData("");
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            apiResultDto.setData("");
            return apiResultDto;
        }
    }

    /**
     * 根据资产类别（固定资产、长期待摊、无形资产）查询出结果
     * @param assetType
     * @return
     */
    @RequestMapping(value = "/getAssetResultList", method = RequestMethod.GET)
    public @ResponseBody ApiResultDto getAssetResultList(Integer assetType, @RequestParam String projectId, @RequestParam Integer taxAccountCompare){
        logger.info("根据资产类别获取资产清单");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(assetListService.getAssetResultList(assetType,projectId,taxAccountCompare));
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }
    /**
     * 根据资产类别（固定资产、长期待摊、无形资产）查询出结果
     * @param citAssetsListDto
     * @return
     */
    @RequestMapping(value = "/getAssetResultListPage", method = RequestMethod.POST)
    public @ResponseBody ApiResultDto getAssetResultListPage(@RequestBody CitAssetsListDto citAssetsListDto){
        logger.info("根据资产类别获取资产清单");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(assetListService.getAssetResultListPage(citAssetsListDto));
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }

    /**
     *
     * @return
     */
    @RequestMapping(value = "/getAssetGroupResultData", method = RequestMethod.GET)
    public @ResponseBody ApiResultDto getAssetGroupResultData(@RequestParam String projectId){
        logger.info("获取资产类别");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(assetListService.getAssetGroupResultData(projectId));
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }
    /**
     * 获取资产一级分类
     * @return
     */
    @RequestMapping(value = "/getAssetGroupListData", method = RequestMethod.GET)
    public @ResponseBody ApiResultDto getAssetGroupListData(){
        logger.info("获取资产一级分类");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(assetListService.getAssetGroupListData());
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }

    /**
     * 获取资产二级分类
     * @return
     */
    @RequestMapping(value = "/getAssetDetailGroupListData", method = RequestMethod.GET)
    public @ResponseBody ApiResultDto getAssetDetailGroupListData(){
        logger.info("根据资产二级分类");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            apiResultDto.setCode(1);
            apiResultDto.setMessage("获取成功");
            apiResultDto.setData(assetListService.getAssetDetailGroupListData());
            return apiResultDto;
        }catch(Exception e){
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("获取失败");
            return apiResultDto;
        }
    }

    /**
     * 当保留差异全选或全反选选项变化时进行批量修改
     * @param assetType
     * @param projectId
     * @param isRetain
     * @return
     */
    @RequestMapping(value="batchUpdateIsRetain",method = RequestMethod.POST)
    public ApiResultDto batchUpdateIsRetain(Integer assetType, @RequestParam String projectId, @RequestParam Integer isRetain){
        assetListService.batchUpdateIsRetain(assetType,projectId, isRetain);
        return null;
    }

    /**
     * 保存资产分类（当对资产类别进行二级分类匹配完成后点击确定按钮所走的方法）
     * @param assetGroupResults
     * @param saveGroupType
     * @param projectId
     * @return
     */
    @RequestMapping(value="saveAssetGroupInfo",method = RequestMethod.POST)
    public ApiResultDto saveAssetGroupInfo(@RequestBody List<CitAssetGroupResult> assetGroupResults,
                                           @RequestParam Integer saveGroupType,
                                           @RequestParam String projectId){

        return assetListService.saveAssetGroupInfo(assetGroupResults, saveGroupType, projectId);
    }

    /**
     * 当资产清单Grid编辑进行保存所走的方法
     * @param citAssetsLists
     * @param projectId
     * @return
     */
    @RequestMapping(value="updateAssetResultList",method = RequestMethod.POST)
    public ApiResultDto updateAssetResultList(@RequestBody List<CitAssetsList> citAssetsLists,
                                           @RequestParam String projectId){
        ApiResultDto apiResultDto = new ApiResultDto();
        assetListService.updateAssetResultList(citAssetsLists,projectId);
        apiResultDto.setCode(1);
        apiResultDto.setMessage("资产修改成功");
        return apiResultDto;
    }

    /**
     * CIT资产文件导入
     * @return
     */
    @RequestMapping(value = "/assetsImport", method = RequestMethod.POST)
    public @ResponseBody
    ApiResultDto assetsImport(@RequestParam MultipartFile file,
                              @RequestParam String filename,
                              @RequestParam String tempFileName,
                              @RequestParam String projectId,
                              @RequestParam Integer importType){
        logger.info("CIT资产导入");
        ApiResultDto apiResultDto = new ApiResultDto();
        try{
            if (file == null || file.getSize() <= 0) {
                apiResultDto.setCode(-1);
                apiResultDto.setMessage("没有获取到文件");
                logger.warn("没有获取到文件");
                return apiResultDto;
            }
            logger.debug("file name: " + file.getOriginalFilename());
            InputStream input = null;
            try {
                input = file.getInputStream();
                //调用资产导入业务逻辑
                OperationResultDto assetsImportResult = assetListService.assetsImport(input, file.getOriginalFilename(), projectId,importType);
                //判断是否导入成功，若成功获取资产类别并返回页面
                if(assetsImportResult.getResult() != null && !assetsImportResult.getResult()){
                    apiResultDto.setData(assetListService.getAssetGroupResultData(projectId));
                }
            } catch (IOException e) {
                apiResultDto.setCode(-2);
                apiResultDto.setMessage("获取文件流失败");
                logger.warn("获取文件流失败");
                return apiResultDto;
            }

            apiResultDto.setCode(1);
            apiResultDto.setMessage("资产导入成功");
            return apiResultDto;
        }catch (Exception e){
            logger.error("资产导入失败，错误信息如下：");
            e.printStackTrace();
            apiResultDto.setCode(0);
            apiResultDto.setMessage("资产导入失败");
            return apiResultDto;
        }
    }
}
