package pwc.taxtech.atms.controller;

import com.alibaba.fastjson.JSONArray;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dpo.PagingDto;
import pwc.taxtech.atms.dto.*;
import pwc.taxtech.atms.dto.dataimport.DataImportAllParam;
import pwc.taxtech.atms.dto.dataimport.DataImportParam;
import pwc.taxtech.atms.dto.dataimport.DataProcessParam;
import pwc.taxtech.atms.dto.input.CamelPagingResultDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceDto;
import pwc.taxtech.atms.dto.vatdto.TrialBalanceParam;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.entity.ProjectExample;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.service.impl.DataImportService;
import pwc.taxtech.atms.service.impl.DidiFileUploadService;
import pwc.taxtech.atms.service.impl.LgGlBalanceService;

import javax.annotation.Resource;
import java.util.List;

@RestController
@RequestMapping("/api/v1/DataImport/")
public class DataImportController extends BaseController {

    @Resource
    private DataImportService dataImportService;
    @Resource
    private LgGlBalanceService lgGlBalanceService;

    @Resource
    private DidiFileUploadService didiFileUploadService;
    @PostMapping("GetBalanceDataForDisplay")
    public PagingResultDto<TrialBalanceDto> getBalanceDataForDisplay(@RequestBody TrialBalanceParam param) {
        PagingResultDto<TrialBalanceDto> resultDto = new PagingResultDto<>();
        List<TrialBalanceDto> list = dataImportService.getBalanceDataForDisplay(param);
        PagingDto pagingDto = new PagingDto();
//        pagingDto.setPageIndex(1);
//        pagingDto.setPageSize(100);
//        pagingDto.setTotalCount(list);
        resultDto.setPageInfo(pagingDto);
        resultDto.setList(list);
        resultDto.setCalculateData(dataImportService.calculateData(list));
        return resultDto;
    }

    @PostMapping("queryGlBalance")
    public ApiResultDto queryGlBalance(@RequestBody TrialBalanceParam param) {
        ApiResultDto resultDto = new ApiResultDto();
        lgGlBalanceService.queryGlBalance(param.getOrgId(), param.getPeriodStart(), param.getPeriodEnd());
        return resultDto;
    }

    @ResponseBody
    @RequestMapping(value = "PLExcelFile", method = RequestMethod.POST)
    public OperationResultDto importPLExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            // 这里可以写aop简化代码，我没时间先不写了:)
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importPLExcelFile(file, orgList,periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importPLExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "BSExcelFile", method = RequestMethod.POST)
    public OperationResultDto importBSExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importBSExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importBSExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "RLITExcelFile", method = RequestMethod.POST)
    public OperationResultDto importRLITExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importRLITExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importRLITExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "ATExcelFile", method = RequestMethod.POST)
    public OperationResultDto importATExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importATExcelFile(file, orgList,periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importATExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }


    @ResponseBody
    @RequestMapping(value = "CFExcelFile", method = RequestMethod.POST)
    public OperationResultDto importCFExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importCFExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importCFExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "CPRExcelFile", method = RequestMethod.POST)
    public OperationResultDto importCPRExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importCPRExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importATExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "IDExcelFile", method = RequestMethod.POST)
    public OperationResultDto importIDExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importIDExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importIDExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "IRExcelFile", method = RequestMethod.POST)
    public OperationResultDto importIRExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importIRExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importIRExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "CILExcelFile", method = RequestMethod.POST)
    public OperationResultDto importCILExcelFile(@RequestParam MultipartFile file,@RequestParam String orgIds,@RequestParam String periodDate,@RequestParam Integer importType) {
        try {
            List<String> orgList = (List<String>)JSONArray.parse(orgIds);
            String valMsg = valParameter(file,orgList,periodDate);
            if(StringUtils.isNotEmpty(valMsg)){
                return OperationResultDto.error(valMsg);
            }
            return dataImportService.importCILExcelFile(file,orgList, periodDate,importType);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("importCILExcelFile error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "displayImportLog", method = RequestMethod.POST)
    public CamelPagingResultDto<DataImportLogDto> displayImportLog(@RequestBody DataImportParam param) {
        return new CamelPagingResultDto<>(dataImportService.displayImportLog(param));
    }

    @ResponseBody
    @RequestMapping(value = "displayImportLogAll", method = RequestMethod.POST)
    public CamelPagingResultDto<DataImportLogDto> displayImportLogAll(@RequestBody DataImportAllParam param) {
        return new CamelPagingResultDto<>(dataImportService.displayImportLogAll(param));
    }

    @ResponseBody
    @RequestMapping(value = "displayProcessLog", method = RequestMethod.POST)
    public CamelPagingResultDto<DataValidateLogDto> displayProcessLog(@RequestBody DataProcessParam param) {
        return new CamelPagingResultDto<>(dataImportService.displayProcessLog(param));
    }

    @ResponseBody
    @RequestMapping(value = "callExtractFinancialData", method = RequestMethod.POST)
    public OperationResultDto callExtractFinancialData(@RequestBody DataExtractParam dataExtractParam) {
        try {
            return dataImportService.callExtractFinancialData(dataExtractParam);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("callExtractFinancialData error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    @ResponseBody
    @RequestMapping(value = "callExtractInvoiceData", method = RequestMethod.POST)
    public OperationResultDto callExtractInvoiceData(@RequestBody DataExtractParam dataExtractParam) {
        try {
            return dataImportService.callExtractInvoiceData(dataExtractParam);
        } catch (ServiceException e) {
            return OperationResultDto.error(e.getMessage());
        } catch (Exception e) {
            logger.error("callExtractInvoiceData error.", e);
            return OperationResultDto.error(ErrorMessage.SystemError);
        }
    }

    private String valParameter(MultipartFile file,List<String> orgList,String periodDate){
        if (null == file) {
            return ErrorMessage.NoFile;
        }
        if(orgList.size()<1){
            return ErrorMessage.DidntSelectedCompany;
        }
        if(StringUtils.isEmpty(periodDate)){
            return ErrorMessage.DidntSelectedPeriod;
        }
        return null;
    }

}
