package pwc.taxtech.atms.controller;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.dto.vatdto.*;
import pwc.taxtech.atms.dto.vatdto.dd.*;
import pwc.taxtech.atms.dto.vatdto.dd.TrialBalanceDto;
import pwc.taxtech.atms.service.impl.DataPreviewSerivceImpl;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.UUID;

import static javax.servlet.http.HttpServletResponse.SC_NO_CONTENT;
import static javax.servlet.http.HttpServletResponse.SC_OK;

@RestController
@RequestMapping("/api/v1/dataPreview/")
public class DataPreviewController extends BaseController {

    @Autowired
    private DataPreviewSerivceImpl dataPreviewSerivceImpl;

    @PostMapping("getTBDataForDisplay")
    public PageInfo<TrialBalanceDto> getTBDataForDisplay(@RequestBody TrialBalanceParam param) {
        logger.debug(String.format("科目余额查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getTBDataForDisplay(param);
    }

    @PostMapping("getPLDataForDisplay")
    public PageInfo<ProfitLossStatementDto> getPLDataForDisplay(@RequestBody ProfitLossStatementParam param) {
        logger.debug(String.format("利润表PRC查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getPLDataForDisplay(param);
    }

    @PostMapping("getJEDataForDisplay")
    public PageInfo<JournalEntryDto> getJEDataForDisplay(@RequestBody JournalEntryParam param) {
        logger.debug(String.format("日记账查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getJEDataForDisplay(param);
    }

    @PostMapping("getCFDataForDisplay")
    public PageInfo<CashFlowDto> getCFDataForDisplay(@RequestBody CashFlowParam param) {
        logger.debug(String.format("现金流量表查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getCFDataForDisplay(param);
    }

    @PostMapping("getBSDataForDisplay")
    public PageInfo<BalanceSheetDto> getBSDataForDisplay(@RequestBody BalanceSheetParam param) {
        logger.debug(String.format("资产负债表PRC查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getBSDataForDisplay(param);
    }

    @PostMapping("getIRDataForDisplay")
    public PageInfo<InvoiceRecordDto> getIRDataForDisplay(@RequestBody InvoiceRecordParam param) {
        logger.debug(String.format("发票记录表查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getIRDataForDisplay(param);
    }

    @PostMapping("getRLITDataForDisplay")
    public PageInfo<RedLetterInfoTableDto> getRLITDataForDisplay(@RequestBody RedLetterInfoTableParam param) {
        logger.debug(String.format("红字信息表查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getRLITDataForDisplay(param);
    }

    @PostMapping("getCPRDataForDisplay")
    public PageInfo<CoupaPurchasingReportDto> getCPRDataForDisplay(@RequestBody CoupaPurchasingReportParam param) {
        logger.debug(String.format("Coupa发票报告表查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getCPRDataForDisplay(param);
    }

    @PostMapping("getCILDataForDisplay")
    public PageInfo<CertifiedInvoicesListDto> getCILDataForDisplay(@RequestBody CertifiedInvoicesListParam param) {
        logger.debug(String.format("已认证发票清单查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getCILDataForDisplay(param);
    }

    @PostMapping("getIDDataForDisplay")
    public PageInfo<InvoiceDataDto> getIDDataForDisplay(@RequestBody InvoiceDataParam param) {
        logger.debug(String.format("发票资料查询 Condition:%s", JSON.toJSONString(param)));
        return dataPreviewSerivceImpl.getIDDataForDisplay(param);
    }

    @RequestMapping(value = "exportCFData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadCFQueryData(@RequestBody CashFlowParam param, HttpServletResponse response) {
        logger.debug("enter downloadCFQueryData");
        String fileName="testFile";
        dataPreviewSerivceImpl.exportCashFlowList(response, param, fileName);
    }

    @RequestMapping(value = "exportTBData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadTBQueryData(@RequestBody TrialBalanceParam paras, HttpServletResponse response) {
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.addHeader("Access-Control-Expose-Headers", "Content-Type,Content-Disposition,x-file-name");
        String fileName = paras.getPeriodStart() + "-" + paras.getPeriodEnd();
        response.setCharacterEncoding("UTF-8");
        OutputStream os = null;
        try {
            response.addHeader("Content-Disposition", "attachment;filename="
                    + UUID.randomUUID() + ".xls");
            response.addHeader("x-file-name", fileName);
            os = response.getOutputStream();
            int count = dataPreviewSerivceImpl.getTBDownloadFilePath(paras, os);

            if (count == 0) {
                response.setStatus(SC_NO_CONTENT);
            } else {
                response.setStatus(SC_OK);
            }
        } catch (IOException e) {
            logger.error(String.format("下载科目余额表-生成文件异常:%s",e.getMessage()));
        }
    }

    @RequestMapping(value = "exportJEData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadJEQueryData(@RequestBody JournalEntryParam paras, HttpServletResponse response) {
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.addHeader("Access-Control-Expose-Headers", "Content-Type,Content-Disposition,x-file-name");
        String fileName = paras.getPeriodStart() + "-" + paras.getPeriodEnd();
        response.setCharacterEncoding("UTF-8");
        OutputStream os = null;
        try {
            response.addHeader("Content-Disposition", "attachment;filename="
                    + UUID.randomUUID() + ".xls");
            response.addHeader("x-file-name", fileName);
            os = response.getOutputStream();
            int count = dataPreviewSerivceImpl.getJEDownloadFilePath(paras, os);

            if (count == 0) {
                response.setStatus(SC_NO_CONTENT);
            } else {
                response.setStatus(SC_OK);
            }
        } catch (IOException e) {
            logger.error(String.format("下载科目余额表-生成文件异常:%s",e.getMessage()));
        }
    }

    @RequestMapping(value = "exportBSData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadBSQueryData(@RequestBody BalanceSheetParam paras, HttpServletResponse response) {
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.addHeader("Access-Control-Expose-Headers", "Content-Type,Content-Disposition,x-file-name");
        String fileName = paras.getPeriodStart() + "-" + paras.getPeriodEnd();
        response.setCharacterEncoding("UTF-8");
        OutputStream os = null;
        try {
            response.addHeader("Content-Disposition", "attachment;filename="
                    + UUID.randomUUID() + ".xls");
            response.addHeader("x-file-name", fileName);
            os = response.getOutputStream();
            int count = dataPreviewSerivceImpl.getBSDownloadFilePath(paras, os);

            if (count == 0) {
                response.setStatus(SC_NO_CONTENT);
            } else {
                response.setStatus(SC_OK);
            }
        } catch (IOException e) {
            logger.error(String.format("下载科目余额表-生成文件异常:%s",e.getMessage()));
        }
    }

    @RequestMapping(value = "exportPLData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadPLQueryData(@RequestBody ProfitLossStatementParam paras, HttpServletResponse response) {
        response.setContentType("application/vnd.ms-excel;charset=utf-8");
        response.addHeader("Access-Control-Expose-Headers", "Content-Type,Content-Disposition,x-file-name");
        String fileName = paras.getPeriodStart() + "-" + paras.getPeriodEnd();
        response.setCharacterEncoding("UTF-8");
        OutputStream os = null;
        try {
            response.addHeader("Content-Disposition", "attachment;filename="
                    + UUID.randomUUID() + ".xls");
            response.addHeader("x-file-name", fileName);
            os = response.getOutputStream();
            int count = dataPreviewSerivceImpl.getPLDownloadFilePath(paras, os);

            if (count == 0) {
                response.setStatus(SC_NO_CONTENT);
            } else {
                response.setStatus(SC_OK);
            }
        } catch (IOException e) {
            logger.error(String.format("下载科目余额表-生成文件异常:%s",e.getMessage()));
        }
    }

    @RequestMapping(value = "exportIRData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadIRQueryData(@RequestBody InvoiceRecordParam param, HttpServletResponse response) {
        logger.debug("enter downloadIRQueryData");
        String fileName="testFile";
        dataPreviewSerivceImpl.exportInvoiceRecordList(response, param, fileName);
    }

    @RequestMapping(value = "exportCILData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadCILQueryData(@RequestBody CertifiedInvoicesListParam param, HttpServletResponse response) {
        logger.debug("enter downloadCILQueryData");
        String fileName="testFile";
        dataPreviewSerivceImpl.exportCILList(response, param, fileName);
    }

    @RequestMapping(value = "exportRLITData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadRLITQueryData(@RequestBody RedLetterInfoTableParam param, HttpServletResponse response) {
        logger.debug("enter downloadRLITQueryData");
        String fileName="testFile";
        dataPreviewSerivceImpl.exportRLITList(response, param, fileName);
    }

    @RequestMapping(value = "exportCPRData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadCPRQueryData(@RequestBody CoupaPurchasingReportParam param, HttpServletResponse response) {
        logger.debug("enter downloadCPRQueryData");
        String fileName="testFile";
        dataPreviewSerivceImpl.exportCPRList(response, param, fileName);
    }

    @RequestMapping(value = "exportIDData/get", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public void downloadIDQueryData(@RequestBody InvoiceDataParam param, HttpServletResponse response) {
        logger.debug("enter downloadIDQueryData");
        String fileName="testFile";
        dataPreviewSerivceImpl.exportIDList(response, param, fileName);
    }

}


