package pwc.taxtech.atms.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import pwc.taxtech.atms.dto.equity.EquityInfoDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.entity.OperationLogBasicData;
import pwc.taxtech.atms.service.impl.EquityServiceImpl;

import java.util.List;

@RestController
@RequestMapping("/api/v1/equity")
public class EquityController {

    @Autowired
    private EquityServiceImpl equityServiceService;

    @RequestMapping(value = "getEquityListByOrgId", method = RequestMethod.GET)
    public @ResponseBody
    List<EquityInfoDto> getEquityListByOrgId(@RequestParam("orgId") String orgId) {
        return equityServiceService.getEquityListByOrgId(orgId);
    }

    @RequestMapping(value = "getEquityListRespByOrgId", method = RequestMethod.GET)
    public @ResponseBody
    ResponseEntity getEquityListRespByOrgId(@RequestParam("orgId") String orgId) {
        return ResponseEntity.ok().body(equityServiceService.getEquityListByOrgId(orgId));
    }

    @RequestMapping(value = "insert", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto<Object> insert(@RequestBody List<EquityInfoDto> equityInfoDtos) {
        return equityServiceService.insertAll(equityInfoDtos);
    }

    @RequestMapping(value = "change", method = RequestMethod.POST, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody
    OperationResultDto<Object> change(@RequestParam("orgName") String orgName, @RequestParam("comment") String comment, @RequestBody List<EquityInfoDto> equityInfoDtos) {
        return equityServiceService.change(orgName, comment, equityInfoDtos);
    }

    @RequestMapping(value = "update", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto<Object> update(@RequestBody EquityInfoDto equityInfoDto) {
        return equityServiceService.update(equityInfoDto);
    }

    @RequestMapping(value = "delete", method = RequestMethod.POST)
    public @ResponseBody
    OperationResultDto<Object> delete(@RequestParam Long id, @RequestBody List<Integer> eNums) {
        return equityServiceService.delete(id, eNums);
    }

    @RequestMapping(value = "getChangeLogByOrgName", method = RequestMethod.GET)
    public @ResponseBody
    List<OperationLogBasicData> getChangeLogByOrgName(@RequestParam("orgName") String orgName) {
        return equityServiceService.getChangeLogByOrgName(orgName);
    }

    @RequestMapping(value = "cancelChange", method = RequestMethod.GET)
    public @ResponseBody
    OperationResultDto<Object> cancelChange(@RequestParam("oldId") Long oldId, @RequestParam("newId") Long newId) {
        return equityServiceService.cancelChange(oldId, newId);
    }

    @RequestMapping(value = "getEquityListById", method = RequestMethod.GET)
    public @ResponseBody
    ResponseEntity getEquityListById(@RequestParam("id") Long id) {
        return ResponseEntity.ok().body(equityServiceService.getEquityListById(id));
    }

    @RequestMapping(value = "getEquityHisListById", method = RequestMethod.GET)
    public @ResponseBody
    ResponseEntity getEquityHisListById(@RequestParam("id") Long id) {
        return ResponseEntity.ok().body(equityServiceService.getEquityHisListById(id));
    }


}
