package pwc.taxtech.atms.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import pwc.taxtech.atms.dto.ReturnData;
import pwc.taxtech.atms.entity.FileTypes;
import pwc.taxtech.atms.service.impl.FileTypesServiceImpl;
import pwc.taxtech.atms.thirdparty.ExcelUtil;

import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Controller
@RequestMapping("/api/v1/fileTypes")
public class FileTypesController {

    @Autowired
    private FileTypesServiceImpl fileTypesService;

    /**
     * 查询档案文件类型列表
     * @return
     */
    @RequestMapping("selectList")
    @ResponseBody
    public ReturnData selectFileTypesList(){
        List<FileTypes> fileTypes = fileTypesService.selectFileTypesList();
        ReturnData returnData = new ReturnData();
        returnData.setItems(fileTypes);
        returnData.setTotalCount(fileTypes.size());
        return returnData;
    }

    /**
     * 查询档案文件类型列表-可用
     * @return
     */
    @RequestMapping("/selectEnableList")
    @ResponseBody
    public ReturnData selectEnableFileTypesList(){
        List<FileTypes> fileTypes = fileTypesService.selectEnableFileTypesList();
        ReturnData returnData = new ReturnData();
        returnData.setItems(fileTypes);
        returnData.setTotalCount(fileTypes.size());
        return returnData;
    }

    /**
     * 查询档案属性和档案类型给前端下拉选择框
     * @return
     */
    @PostMapping("/query4SelectionBox")
    @ResponseBody
    public Map<String,String> query4SelectionBox(){
        List<FileTypes> fileTypes = fileTypesService.query4SelectionBox();
        Map<String,String> result = fileTypes.stream().distinct().collect(Collectors.toMap(FileTypes::getFileType,FileTypes::getFileAttr));
        return result;
    }

    /**
     * 查询档案属性和档案类型给前端下拉选择框(可用)
     * @return
     */
    @PostMapping("/query4SelectionBoxEnable")
    @ResponseBody
    public Map<String,String> query4SelectionBoxEnable(){
        List<FileTypes> fileTypes = fileTypesService.query4SelectionBoxEnable();
        Map<String,String> result = fileTypes.stream().distinct().collect(Collectors.toMap(FileTypes::getFileType,FileTypes::getFileAttr));
        return result;
    }

    /**
     * 添加档案文件类型
     * @param fileTypes
     * @return
     */
    @PostMapping("add")
    @ResponseBody
    public boolean addFileTypes(@RequestBody FileTypes fileTypes){
        return fileTypesService.addFileTypesList(fileTypes);
    }

    /**
     * 删除档案文件类型
     * @param fileTypes
     * @return
     */
    @PostMapping("delete")
    @ResponseBody
    public boolean deleteFileTypes(@RequestBody FileTypes fileTypes){
        return fileTypesService.deleteFileTypes(fileTypes.getId());
    }

    /**
     * 更新档案文件类型
     * @param fileTypes
     * @return
     */
    @PostMapping("edit")
    @ResponseBody
    public boolean editFileTypes(@RequestBody FileTypes fileTypes){
        return fileTypesService.editFilesType(fileTypes);
    }

    /**
     * 导出excel
     * @param response
     */
    @RequestMapping("exportExcel")
    @ResponseBody
    public void exportExcelFile (HttpServletResponse response){
        try {
            Map<String, String> headers = new LinkedHashMap<>();
            headers.put("id", "序号");
            headers.put("file_attr", "档案属性");
            headers.put("file_type", "档案类型");
            headers.put("required_field", "必填字段");
            headers.put("description", "说明");
            headers.put("creator", "办事人");
            headers.put("create_time", "创建时间");
            headers.put("update_time", "更新时间");
            headers.put("status", "状态");
            headers.put("remarks", "备注");
            List<FileTypes> fileTypes = fileTypesService.selectFileTypesList();
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition", "attachment;fileName=" + new String("".getBytes("GB2312"), "ISO-8859-1"));
            OutputStream ouputStream = response.getOutputStream();
            ExcelUtil.exportExcel(headers, fileTypes, ouputStream);
        }catch(Exception e){
            e.printStackTrace();
        }
    }

}
