package pwc.taxtech.atms.controller;

import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.constant.enums.EnumImportType;
import pwc.taxtech.atms.dto.ApiResultDto;
import pwc.taxtech.atms.dto.input.CamelPagingResultDto;
import pwc.taxtech.atms.dto.revenuconf.RevConfAddDto;
import pwc.taxtech.atms.dto.revenuconf.RevenueConfParam;
import pwc.taxtech.atms.dto.revenuconf.RevenueConfResult;
import pwc.taxtech.atms.service.impl.RevenueConfService;
import pwc.taxtech.atms.vat.entity.RevenueConfig;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

@RestController
@RequestMapping(value = "api/v1/revenueConf")
public class RevenueConfController extends BaseController {

    @Resource
    private RevenueConfService revenueConfService;

    @PostMapping("queryPage")
    public CamelPagingResultDto<RevenueConfResult> queryPage(@RequestBody RevenueConfParam param) {
        return new CamelPagingResultDto<>(revenueConfService.queryPage(param));
    }

    @PostMapping("queryAll")
    public List<RevenueConfResult> queryAll(@RequestBody RevenueConfParam param) {
        return revenueConfService.queryAll(param);
    }

    @PostMapping("add")
    public ApiResultDto addConf(@RequestBody RevConfAddDto addDto) {
        return revenueConfService.addConfig(addDto);
    }

    @PostMapping("update")
    public ApiResultDto updateConf(@RequestBody RevenueConfig config) {
        revenueConfService.updateConfig(config);
        return ApiResultDto.success();
    }

    @PostMapping("del")
    public ApiResultDto delConf(@RequestBody List<Long> idList) {
        revenueConfService.delConfig(idList);
        return ApiResultDto.success();
    }

    @PostMapping("upload")
    public ApiResultDto upload(@RequestParam MultipartFile file, @RequestParam Integer type) throws Exception {
        List<String> unSuccessList = revenueConfService.upload(file, type);
        return ApiResultDto.success(unSuccessList);
    }

    @GetMapping("export")
    public void export(HttpServletResponse response) throws Exception {
        OutputStream ouputStream = null;
        try {
            InputStream inputStream = this.getClass().getResourceAsStream("/document/DataImport/" + EnumImportType.RevenueConfig.name() + ".xlsx");
            Workbook tWorkbook = revenueConfService.export(inputStream);
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition", "attachment;fileName=" + EnumImportType.RevenueConfig.name() + ".xlsx");
            ouputStream = response.getOutputStream();
            tWorkbook.write(ouputStream);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if (ouputStream != null) {
                    ouputStream.close();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }
}
