package pwc.taxtech.atms.dto.vatdto;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.ValueEval;

import java.util.ArrayList;
import java.util.List;

import static pwc.taxtech.atms.common.util.FormulaUtil.resolverInteger;
import static pwc.taxtech.atms.common.util.FormulaUtil.resolverString;
import static pwc.taxtech.atms.exception.Exceptions.BAD_BBVO_PARAMS;

public class RSUMIFParasBo {
    private String reportCode;
    private Integer columnIndex;
    private Integer rowIndex;
    private Integer period;
    private Integer year;
    private String formulaExpression;
    private List<PeriodCellDataTemplate> expressionData = new ArrayList<>();

    public static class PeriodCellDataTemplate {
        Integer period;
        Long cellTemplateId;

        @Override
        public String toString() {
            return period + ":" + cellTemplateId;
        }

        public PeriodCellDataTemplate(Integer period, Long cellTemplateId) {
            this.period = period;
            this.cellTemplateId = cellTemplateId;
        }

    }

    public void putPeriodCellTempate(Integer period, Long cellTemplateId) {
        if (period < 1 || period > 12 || cellTemplateId < 0) throw BAD_BBVO_PARAMS;
        expressionData.add(new PeriodCellDataTemplate(period, cellTemplateId));
    }

    public String getExpression(ValueEval[] args, OperationEvaluationContext ec) throws EvaluationException {
        StringBuilder expression = new StringBuilder("");
        begin(expression);
        for (int i = 0; i < args.length; i++) {
            if (i >= args.length-2) {
                concatPara(expression, resolverInteger(args, ec, i));
            } else {
                concatPara(expression, resolverString(args, ec, i));
            }
            if (i == args.length - 1) {
                continue;
            }
            split(expression);
        }
        end(expression);
        return expression.toString();
    }

    public String getQMYEExpression(Integer subjectType, String subjectCode, Integer year, Integer period, OperationEvaluationContext ec) throws EvaluationException {
        StringBuilder expression = new StringBuilder("");
        beginQMYE(expression);
        concatPara(expression, subjectType);
        split(expression);
        concatPara(expression, subjectCode);
        split(expression);
        concatPara(expression, year);
        split(expression);
        concatPara(expression, period);

        end(expression);
        return expression.toString();
    }

    private StringBuilder begin(StringBuilder expression) {
        return expression.append("RSUMIF(");
    }

    private StringBuilder beginQMYE(StringBuilder expression) {
        return expression.append("QMYE(");
    }

    private StringBuilder end(StringBuilder expression) {
        return expression.append(")");
    }

    private StringBuilder split(StringBuilder expression) {
        return expression.append(",");
    }

    private StringBuilder concatPara(StringBuilder expression, String para) {
        return expression.append("\"").append(para).append("\"");
    }

    private StringBuilder concatPara(StringBuilder expression, Integer para) {
        return expression.append(para);
    }

    public RSUMIFParasBo() {
    }

    public RSUMIFParasBo(RSUMIFParasBo otherBo, int period, int curYear) {
        this.reportCode = otherBo.reportCode;
        this.columnIndex = otherBo.getColumnIndex();
        this.rowIndex = otherBo.rowIndex;
        this.period = period;
        this.year = curYear;
        this.formulaExpression = otherBo.formulaExpression;
    }

    public String expression() {
        return formulaExpression;
    }

    public String getReportCode() {
        return this.reportCode;
    }

    public void setReportCode(String reportCode) {
        this.reportCode = reportCode;
    }

    public Integer getColumnIndex() {
        return this.columnIndex;
    }

    public void setColumnIndex(Integer columnIndex) {
        this.columnIndex = columnIndex;
    }

    public Integer getRowIndex() {
        return this.rowIndex;
    }

    public void setRowIndex(Integer rowIndex) {
        this.rowIndex = rowIndex;
    }

    public Integer getPeriod() {
        return this.period;
    }

    public void setPeriod(Integer period) {
        this.period = period;
    }

    public Integer getYear() {
        return this.year;
    }

    public void setYear(Integer year) {
        this.year = year;
    }

    public String getFormulaExpression() {
        return this.formulaExpression;
    }

    public void setFormulaExpression(String formulaExpression) {
        this.formulaExpression = formulaExpression;
    }

    public List<PeriodCellDataTemplate> getExpressionData() {
        return this.expressionData;
    }

    public void setExpressionData(List<PeriodCellDataTemplate> expressionData) {
        this.expressionData = expressionData;
    }
}
