package pwc.taxtech.atms.security.dd;

import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.service.impl.ProjectServiceImpl;

import javax.crypto.Cipher;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;

/**
 * @Auther: Gary J Li
 * @Date: 20/03/2019 20:41
 * @Description:
 */
@Service
public class DtsTokenService {

    private static final Logger logger = LoggerFactory.getLogger(ProjectServiceImpl.class);

    @Value("${dd_pubkey}")
    private String pubKey;

    public static final String KEY_ALGORITHM = "RSA";

    private static final String PUBLIC_KEY = "RSAPublicKey";

    private static final String PRIVATE_KEY = "RSAPrivateKey";

    private static final int KEY_SIZE = 512;

    /**
     * 生成加密token
     * @return
     * @throws
     */
    public String encryptInput(){
        byte[] encodeData;
        try {
            byte[] publicKey = Base64.decodeBase64(pubKey);
            String nonce = generateNonce();
            String sDate = generateDate();
            long rNum = generateRandomNumber();
            String inputStr1 = sDate + "@@" + "DTS" + "@@" + nonce+rNum;
            byte[] data1 = inputStr1.getBytes();
            logger.debug("原文：" + inputStr1);
            encodeData = encryptByPublicKey(data1, publicKey);
            logger.debug("公钥加密后：" + Base64.encodeBase64String(encodeData));
        } catch (Exception ex) {
            throw new ServiceException("cus" + ex);
        }

        return Base64.encodeBase64String(encodeData);
    }



    private String generateNonce() throws Exception {
        String dateTimeString = Long.toString(System.currentTimeMillis());
        byte[] nonceByte = dateTimeString.getBytes();
        return Base64.encodeBase64String(nonceByte);
    }

    private String generateDate() throws Exception {
        DateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
        dateFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
        Date today = Calendar.getInstance().getTime();
        String created = dateFormatter.format(today);

        return created;
    }


    private long generateRandomNumber() throws Exception {
        final double d = Math.random();
        final long num = (int)(d*100000000000000L);
        return num;
    }



    private static byte[] encryptByPublicKey(byte[] data, byte[] key) throws Exception {
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(key);
        KeyFactory keyFactory = KeyFactory.getInstance(KEY_ALGORITHM);
        PublicKey publicKey = keyFactory.generatePublic(x509KeySpec);
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        return cipher.doFinal(data);
    }
}
