package pwc.taxtech.atms.security.vendor;

import com.alibaba.fastjson.JSON;
import com.google.common.collect.ImmutableMap;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.common.util.SignatureUtil;
import pwc.taxtech.atms.dto.ApiResultDto;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.Map;

public class ApiSignatureFilter implements Filter {
    private static final Logger LOGGER = LoggerFactory.getLogger(ApiSignatureFilter.class);
    private static String INVALId_RESULT = JSON.toJSONString(ApiResultDto.fail("invalid api signature"));

    private static Map<String, String> keyMap = new ImmutableMap.Builder<String, String>()
            .put("longi", "eyJhbGciOiJIUzUxMiJ9.eyJzdWIiOiJsb25naSIsImV4cCI6MzI0ODczMDE1NjAsImlhdCI6MTUyOTkyMjM2MSwibmJmIjoxNTI5OTIxNzYxLCJqdGkiOiIyNURFRDEyRi0yRTBBLTRERDUtQjkyOS0xRjlCOTI1QzA2MjciLCJhcHBJZCI6ImxvbmdpIn0.fPddvBGXjViEXNrYA7BesndVjM5eYHA0cX_sKZprHbIasD75Sn8vWNVKb5hMDk3wk3M34k7VgkTFHnpj9BF2uw")
            .build();

    @Override
    public void init(FilterConfig filterConfig) throws ServletException {

    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
        try {
            HttpServletRequest request = (HttpServletRequest) servletRequest;
            String appId = StringUtils.defaultString(request.getParameter(SignatureUtil.SIGN_APP_Id));
            String nonceStr = StringUtils.defaultString(request.getParameter(SignatureUtil.SIGN_NONCE_STR));
            String timestamp = StringUtils.defaultString(request.getParameter(SignatureUtil.SIGN_TIMESTAMP));
            String signature = StringUtils.defaultString(request.getParameter(SignatureUtil.SIGN_SIGNATURE));
            String api = StringUtils.defaultString(request.getParameter(SignatureUtil.SIGN_API));
            if (StringUtils.isAnyBlank(appId, signature, timestamp, nonceStr, api)) {
                writeError(servletResponse, INVALId_RESULT);
                return;
            }
            String key = keyMap.get(appId);
            if (StringUtils.isBlank(key)) {
                writeError(servletResponse, INVALId_RESULT);
                return;
            }
            if (!SignatureUtil.validate(key, api, nonceStr, timestamp, signature)) {
                writeError(servletResponse, INVALId_RESULT);
                return;
            }
            filterChain.doFilter(servletRequest, servletResponse);
        } catch (Exception e) {
            LOGGER.error("invalid api signature: {}", JSON.toJSONString(servletRequest.getParameterMap()), e);
            writeError(servletResponse, INVALId_RESULT);
        }
    }

    @Override
    public void destroy() {

    }

    private void writeError(ServletResponse servletResponse, String msg) throws IOException {
        servletResponse.setCharacterEncoding("UTF-8");
        servletResponse.setContentType("application/json; charset=UTF-8");
        servletResponse.getWriter().write(msg);
    }
}
