package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.message.ErrorMessage;
import pwc.taxtech.atms.constant.enums.FileUploadEnum;
import pwc.taxtech.atms.constant.enums.ReportFileUploadEnum;
import pwc.taxtech.atms.dao.FileTypesMapper;
import pwc.taxtech.atms.dao.OrganizationMapper;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dao.UserMapper;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.dto.reportFileUpload.ReportFileUploadParam;
import pwc.taxtech.atms.dto.reportFileUpload.ReportFileUploadResult;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.vat.dao.ReportFileUploadMapper;
import pwc.taxtech.atms.vat.entity.FileUpload;
import pwc.taxtech.atms.vat.entity.ReportFileUpload;
import pwc.taxtech.atms.vat.entity.ReportFileUploadExample;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class CitReportFileUploadService extends BaseService {

    @Resource
    private ReportFileUploadMapper reportFileUploadMapper;

    @Autowired
    protected AuthUserHelper authUserHelper;

    @Resource
    private UserMapper userMapper;

    @Resource
    private ProjectMapper projectMapper;

    @Resource
    private OrganizationMapper organizationMapper;

    @Resource
    private FileTypesMapper fileTypesMapper;

    @Autowired
    DidiFileUploadService didiFileUploadService;

    @Autowired
    TaxDocumentServiceImpl taxDocumentService;
    public List<ReportFileUploadResult> queryData(ReportFileUploadParam param) {
        ReportFileUploadExample example = new ReportFileUploadExample();
        example.createCriteria().andProjectIdEqualTo(param.getProjectId()).andPeriodEqualTo(param.getPeriod());
        List<ReportFileUpload> dataList = reportFileUploadMapper.selectByExample(example);
        List<String> fileUploadUids = dataList.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList());
        Map<String, String> urlMap = new HashMap<>();
        if (CollectionUtils.isNotEmpty(fileUploadUids)) {
            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
            fileParam.setUuids(fileUploadUids);
            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
            if (CollectionUtils.isNotEmpty(uploadDetail.getList())) {
                urlMap = uploadDetail.getList().stream().collect(Collectors.toMap(DidiFileUploadDetailResult::getUid, didiFileUploadDetailResult -> didiFileUploadDetailResult.getViewHttpUrl()));
            }
        }
        List<String> reportTypeList = dataList.stream()
                .map(o -> o.getReportType()).collect(Collectors.toList());
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_FIRST_QUARTERLY.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_FIRST_QUARTERLY.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_FIRST_QUARTERLY.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_FIRST_QUARTERLY.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_SECOND_QUARTERLY.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_SECOND_QUARTERLY.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_SECOND_QUARTERLY.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_SECOND_QUARTERLY.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_THIRD_QUARTERLY.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_THIRD_QUARTERLY.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_THIRD_QUARTERLY.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_THIRD_QUARTERLY.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_FORTH_QUARTERLY.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_FORTH_QUARTERLY.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_FORTH_QUARTERLY.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_FORTH_QUARTERLY.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_FINAL_SETTLEMENT_DECLARATION_FORM.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_FINAL_SETTLEMENT_DECLARATION_FORM.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_FINAL_SETTLEMENT_DECLARATION_FORM.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_FINAL_SETTLEMENT_DECLARATION_FORM.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_FIRST_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_FIRST_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_FIRST_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_FIRST_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_SECOND_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_SECOND_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_SECOND_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_SECOND_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_THIRD_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_THIRD_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_THIRD_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_THIRD_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_FORTH_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_FORTH_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_FORTH_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_FORTH_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_FINAL_SETTLEMENT_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_FINAL_SETTLEMENT_TAX_STAMP.getCode());
            dataList.add(upload);
        }
        if (!reportTypeList.contains(ReportFileUploadEnum.ReportType.CIT_CORRECT_FINAL_SETTLEMENT_TAX_STAMP.getCode())) {
            ReportFileUpload upload = new ReportFileUpload();
            upload.setReportType(ReportFileUploadEnum.ReportType.CIT_CORRECT_FINAL_SETTLEMENT_TAX_STAMP.getCode());
            dataList.add(upload);
        }

        List<ReportFileUploadResult> resultList = new ArrayList<ReportFileUploadResult>();
        for (ReportFileUpload data : dataList) {
            ReportFileUploadResult result = beanUtil.copyProperties(data, new ReportFileUploadResult());
            result.setFileUrl(urlMap.get(data.getFileUploadId()));
            resultList.add(result);
        }
        return resultList;
    }

    @Transactional
    public void saveData(MultipartFile file, ReportFileUpload data) {
        if (!ReportFileUploadEnum.ReportType.MAPPING.containsKey(data.getReportType())) {
            throw new ServiceException(ErrorMessage.ParamError);
        }
        if (!ReportFileUploadEnum.SuorceType.MAPPING.containsKey(data.getSourceType())) {
            throw new ServiceException(ErrorMessage.ParamError);
        }
        String uid = authUserHelper.getCurrentUserId();
        User user = userMapper.selectByPrimaryKey(uid);
        data.setCreator(user.getUserName());
        data.setUid(CommonUtils.getUUID());
        data.setCreateTime(new Date());
        if(StringUtils.isBlank(data.getReportFileName())){
            data.setReportFileName(file.getOriginalFilename());
        }

        if (StringUtils.isBlank(data.getProjectId()) && StringUtils.isNotBlank(data.getOrgId()) && data.getPeriod() != null) {
            ProjectExample projectExample = new ProjectExample();
            String year = String.valueOf(data.getPeriod()).substring(0,4);
            projectExample.createCriteria().andOrganizationIdEqualTo(data.getOrgId()).andYearEqualTo(Integer.valueOf(year));
            List<Project> projects = projectMapper.selectByExample(projectExample);
            if(CollectionUtils.isNotEmpty(projects)){
                data.setProjectId(projects.get(0).getId());
            }else{
                return;
            }
            if (StringUtils.isBlank(data.getFileUploadId())) {
                FileUpload fileUpload = didiFileUploadService.uploadFile(file, file.getOriginalFilename(), FileUploadEnum.BizSource.REPORT_UPLOAD.name());
                data.setFileUploadId(fileUpload.getUid());
            }
        }else {
            Project project = projectMapper.selectByPrimaryKey(data.getProjectId());
            data.setOrgId(project.getOrganizationId());
            TaxDocument taxDocument = new TaxDocument();
            taxDocument.setCompanyId(project.getOrganizationId());
            Organization organization = organizationMapper.selectByPrimaryKey(project.getOrganizationId());
            taxDocument.setCompanyName(organization.getName());
            taxDocument.setFileUploadId(data.getFileUploadId());
            taxDocument.setEnable("T");
            taxDocument.setOwnTime(data.getPeriod());
            taxDocument.setFileType(data.getReportType());

            FileTypesExample example = new FileTypesExample();
            example.createCriteria().andFileTypeEqualTo(data.getReportType());
            List<FileTypes> dataList = fileTypesMapper.selectByExample(example);
            if(CollectionUtils.isNotEmpty(dataList)){
                taxDocument.setFileAttr(dataList.get(0).getFileAttr());
                taxDocument.setFileTypeId(dataList.get(0).getId());
            }
            //重新命名
            String fileName = taxDocument.getCompanyName()+"_"+taxDocument.getFileType()+"_"+taxDocument.getOwnTime();
            fileName+=file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
            if (StringUtils.isBlank(data.getFileUploadId())) {
                FileUpload fileUpload = didiFileUploadService.uploadFile(file, fileName, FileUploadEnum.BizSource.REPORT_UPLOAD.name());
                data.setFileUploadId(fileUpload.getUid());
                data.setReportFileName(fileName);
            }
            taxDocument.setFileName(fileName);
            taxDocumentService.addTaxDocumentList(file,taxDocument);
        }
        ReportFileUploadExample example = new ReportFileUploadExample();
        example.createCriteria().andProjectIdEqualTo(data.getProjectId()).andPeriodEqualTo(data.getPeriod()).andReportTypeEqualTo(data.getReportType());
        int updateRow = reportFileUploadMapper.updateByExample(data, example);
        if (updateRow == 0) {
            reportFileUploadMapper.insert(data);
        }
    }

}
