package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.constant.enums.FileUploadEnum;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dao.UserMapper;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.dto.reportUpload.ReportUploadParam;
import pwc.taxtech.atms.dto.reportUpload.ReportUploadResult;
import pwc.taxtech.atms.entity.Project;
import pwc.taxtech.atms.entity.User;
import pwc.taxtech.atms.vat.dao.ReportUploadMapper;
import pwc.taxtech.atms.vat.entity.FileUpload;
import pwc.taxtech.atms.vat.entity.ReportUpload;
import pwc.taxtech.atms.vat.entity.ReportUploadExample;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class CitReportUploadService extends BaseService {

    @Resource
    private ReportUploadMapper reportUploadMapper;

    @Autowired
    protected AuthUserHelper authUserHelper;

    @Resource
    private UserMapper userMapper;

    @Resource
    private ProjectMapper projectMapper;


    @Autowired
    DidiFileUploadService didiFileUploadService;

    public PageInfo<ReportUploadResult> queryPage(ReportUploadParam param) {
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        ReportUploadExample example = new ReportUploadExample();
        example.createCriteria().andProjectIdEqualTo(param.getProjectId()).andPeriodEqualTo(param.getPeriod()).andServiceTypeEqualTo(EnumServiceType.CIT.getCode());
        example.setOrderByClause(" create_time DESC ");
        List<ReportUpload> dataList = reportUploadMapper.selectByExample(example);
        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
        fileParam.setUuids(dataList.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList()));
        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
        Map<String,String> urlMap = null;
        if(CollectionUtils.isNotEmpty(uploadDetail.getList())){
            urlMap = uploadDetail.getList().stream().collect(Collectors.toMap(DidiFileUploadDetailResult::getUid, didiFileUploadDetailResult -> didiFileUploadDetailResult.getViewHttpUrl()));
        }
        List<ReportUploadResult> resultList = new ArrayList<ReportUploadResult>();
        for(ReportUpload data:dataList){
            ReportUploadResult result = beanUtil.copyProperties(data, new ReportUploadResult());
            result.setFileUrl(urlMap.get(data.getFileUploadId()));
            resultList.add(result);
        }
        PageInfo<ReportUploadResult> pageInfo = new PageInfo<>(resultList);
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    public void saveData(MultipartFile file, ReportUpload data) {
        String uid = authUserHelper.getCurrentUserId();
        User user = userMapper.selectByPrimaryKey(uid);
        data.setCreator(user.getUserName());
        String fileName = "";
        Project project = projectMapper.selectByPrimaryKey(data.getProjectId());
        data.setOrgId(project.getOrganizationId());
        fileName += project.getName();
        fileName += "-";
        fileName += data.getPeriod();
        fileName += "-V";
        ReportUploadExample example = new ReportUploadExample();
        example.createCriteria().andProjectIdEqualTo(data.getProjectId()).andPeriodEqualTo(data.getPeriod());
        example.setOrderByClause("version desc");
        List<ReportUpload> dataList = reportUploadMapper.selectByExample(example);
        if (CollectionUtils.isNotEmpty(dataList)) {
            fileName += (dataList.get(0).getVersion() + 1);
            data.setVersion(dataList.get(0).getVersion() + 1);
        } else {
            fileName += 1;
            data.setVersion(1);
        }
        fileName += ".xlsx";
        data.setReportName(fileName);
        FileUpload fileUpload = didiFileUploadService.uploadFile(file, fileName, FileUploadEnum.BizSource.REPORT_UPLOAD.name());
        data.setFileUploadId(fileUpload.getUid());
        data.setUid(CommonUtils.getUUID());
        data.setCreateTime(new Date());
        reportUploadMapper.insert(data);
    }

    public void saveData(byte[] bytes, ReportUpload data) {
        String uid = authUserHelper.getCurrentUserId();
        User user = userMapper.selectByPrimaryKey(uid);
        data.setCreator(user.getUserName());
        String fileName = "";
        Project project = projectMapper.selectByPrimaryKey(data.getProjectId());
        data.setOrgId(project.getOrganizationId());
        String period = ""+project.getYear()+(data.getPeriod()>9?data.getPeriod():("0"+data.getPeriod()));
        data.setPeriod(Integer.valueOf(period));
        fileName += project.getName();
        fileName += "-";
        fileName += data.getPeriod();
        fileName += "-V";
        ReportUploadExample example = new ReportUploadExample();
        example.createCriteria().andProjectIdEqualTo(data.getProjectId()).andPeriodEqualTo(data.getPeriod());
        example.setOrderByClause("version desc");
        List<ReportUpload> dataList = reportUploadMapper.selectByExample(example);
        if (CollectionUtils.isNotEmpty(dataList)) {
            fileName += (dataList.get(0).getVersion() + 1);
            data.setVersion(dataList.get(0).getVersion() + 1);
        } else {
            fileName += 1;
            data.setVersion(1);
        }
        fileName += ".xlsx";
        data.setReportName(fileName);
        FileUpload fileUpload = didiFileUploadService.uploadFile(bytes, fileName, FileUploadEnum.BizSource.REPORT_UPLOAD.name());
        data.setFileUploadId(fileUpload.getUid());
        data.setUid(CommonUtils.getUUID());
        data.setCreateTime(new Date());
        data.setServiceType(EnumServiceType.CIT.getCode());
        reportUploadMapper.insert(data);
    }
}
