package pwc.taxtech.atms.service.impl;

import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.dao.FileTypesMapper;
import pwc.taxtech.atms.entity.FileTypes;
import pwc.taxtech.atms.entity.FileTypesExample;
import pwc.taxtech.atms.entity.OperationLogFileType;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 查询
 */
@Slf4j
@Service
public class FileTypesServiceImpl {

    @Resource
    private FileTypesMapper fileTypesMapper;

    @Autowired
    private AuthUserHelper authUserHelper;

    @Autowired
    private OperationLogFileTypeServiceImpl operationLogFileTypeService;

    public List<FileTypes> selectFileTypesList() {
        FileTypesExample example = new FileTypesExample();
        return fileTypesMapper.selectByExample(example);
    }

    public List<FileTypes> selectEnableFileTypesList() {
        FileTypesExample example = new FileTypesExample();
        FileTypesExample.Criteria criteria = example.createCriteria();
        criteria.andStatusEqualTo("1");
        return fileTypesMapper.selectByExample(example);
    }

    /**
     * 查询档案属性和档案类型给前端下拉选择框
     *
     * @return
     */
    public List<FileTypes> query4SelectionBox() {
        return fileTypesMapper.query4SelectionBox();
    }

    @Transactional
    public boolean addFileTypesList(FileTypes fileTypes) {
        try {
            //检测添加类型是否已存在,存在则返回false先(后期改进为多异常)
            checkFileType(fileTypes);
            //对必填字段进行转换成json对象
            requiredFieldFormatToJson(fileTypes);
            //设置当前时间 当前创建人信息
            fileTypes.setCreateTime(new Date());
            fileTypes.setUpdateTime(new Date());
            fileTypes.setCreator(authUserHelper.getCurrentAuditor().get());
            fileTypes.setCreatorId(authUserHelper.getCurrentUserId());
            int num = fileTypesMapper.insert(fileTypes);
            if (num > 0) {
                OperationLogFileType actionEntity = buildOperationLogFileType();
                actionEntity.setOperationAction("新增");
                actionEntity.setId(fileTypes.getId().toString());
                actionEntity.setUpdateState(fileTypes.toString());
                boolean result = operationLogFileTypeService.addFileTypesList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("FileTypesServiceImpl addFileTypesList log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            log.error("FileTypesServiceImpl addFileTypesList error : " + e.getMessage());
            return false;
        }
    }

    /**
     * //检测添加类型是否已存在,存在则返回false先(后期改进为多异常)
     *
     * @param fileTypes
     * @return
     */
    private void checkFileType(FileTypes fileTypes) {
        FileTypesExample example = new FileTypesExample();
        FileTypesExample.Criteria criteria = example.createCriteria();
        if (StringUtils.isNotBlank(fileTypes.getFileType())) {
            criteria.andFileTypeEqualTo(fileTypes.getFileType());
        }
        List<FileTypes> results = fileTypesMapper.selectByExample(example);
        if (results.size() > 0 && !results.stream().map(e->e.getId()).collect(Collectors.toList()).contains(fileTypes.getId())) {
            throw new RuntimeException("filetype已存在: " + fileTypes.getFileType());
        }
    }

    @Transactional
    public boolean deleteFileTypes(Long id) {
        try {
            int num = fileTypesMapper.deleteByPrimaryKey(id);
            if (num > 0) {
                OperationLogFileType actionEntity = buildOperationLogFileType();
                actionEntity.setOperationAction("删除");
                actionEntity.setId(id.toString());
                boolean result = operationLogFileTypeService.addFileTypesList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("FileTypesServiceImpl deleteFileTypes log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            log.error("FileTypesServiceImpl deleteFileTypes error : " + e.getMessage());
            return false;
        }
    }

    @Transactional
    public boolean editFilesType(FileTypes fileTypes) {
        //检测添加类型是否已存在,存在则返回false先(后期改进为多异常)
        checkFileType(fileTypes);
        //对必填字段进行转换成json对象
        requiredFieldFormatToJson(fileTypes);
        fileTypes.setUpdateTime(new Date());
        int num = fileTypesMapper.updateByPrimaryKey(fileTypes);
        if (num > 0) {
            OperationLogFileType actionEntity = buildOperationLogFileType();
            actionEntity.setOperationAction("更新");
            actionEntity.setId(fileTypes.getId().toString());
            //设置更新值
            actionEntity.setUpdateState(fileTypes.toString());
            boolean result = operationLogFileTypeService.addFileTypesList(actionEntity);
            if (result) {
                return true;
            } else {
                throw new RuntimeException("FileTypesServiceImpl editFilesType log error");
            }
        } else {
            return false;
        }
    }

    /**
     * 构建日志对象
     *
     * @return
     */
    private OperationLogFileType buildOperationLogFileType() {
        OperationLogFileType actionEntity = new OperationLogFileType();
        actionEntity.setOperationContent("系统管理");
        actionEntity.setModuleName("档案管理");
        actionEntity.setOperationObject("税务档案类型设置");
        actionEntity.setOperationUser(authUserHelper.getCurrentAuditor().get());
        actionEntity.setIp(authUserHelper.getClientIp());
        actionEntity.setCreateTime(new Date());
        return actionEntity;
    }

    /**
     * 对必填字段进行格式化为JSON 存到数据库中
     *
     * @param fileTypes
     */
    private void requiredFieldFormatToJson(FileTypes fileTypes) {
        if (null != fileTypes.getRequiredField() && fileTypes.getRequiredField().size() > 0) {
            String requiredFieldJson = JSONObject.toJSONString(fileTypes.getRequiredField());
            fileTypes.setRequiredFieldJson(requiredFieldJson);
        }else{
            throw new RuntimeException("必填字段没有填写");
        }
    }

    public List<FileTypes> query4SelectionBoxEnable() {
        return fileTypesMapper.query4SelectionBoxEnable();
    }
}
