package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.POIUtil;
import pwc.taxtech.atms.dpo.OrgCodeIdDto;
import pwc.taxtech.atms.dpo.OrgSelectDto;
import pwc.taxtech.atms.dto.revenuconf.RevTypeAddDto;
import pwc.taxtech.atms.dto.revenuconf.RevenueConfParam;
import pwc.taxtech.atms.dto.revenuconf.RevenueTypeResult;
import pwc.taxtech.atms.vat.dao.RevenueTypeMappingMapper;
import pwc.taxtech.atms.vat.entity.RevenueTypeMapping;
import pwc.taxtech.atms.vat.entity.RevenueTypeMappingExample;

import javax.annotation.Resource;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.constant.Constant.DEFAULT_END_DATE;

@Service
public class RevenueTypeMappingService extends BaseService {

    @Resource
    private RevenueTypeMappingMapper typeMappingMapper;
    @Resource
    private OrganizationServiceImpl organizationService;

    /**
     * 分页查询可查看的配置信息
     *
     * @param param RevenueConfParam
     * @return PageInfo
     */
    public PageInfo<RevenueTypeResult> queryPage(RevenueConfParam param) {
        List<OrgSelectDto> orgDtoList = organizationService.getMyOrgList();
        if (CollectionUtils.isEmpty(orgDtoList)) {
            return new PageInfo<>();
        }
        Page page = PageHelper.startPage(param.getPageInfo().getPageIndex(), param.getPageInfo().getPageSize());
        RevenueTypeMappingExample example = new RevenueTypeMappingExample();
        example.createCriteria().andOrgIdIn(orgDtoList.stream().map(OrgSelectDto::getId).collect(Collectors.toList()));
        example.setOrderByClause("org_id");
        PageInfo<RevenueTypeResult> pageInfo = new PageInfo<>(typeMappingMapper.selectByExample(example).stream()
                .map(o -> beanUtil.copyProperties(o, new RevenueTypeResult())).collect(Collectors.toList()));
        pageInfo.setTotal(page.getTotal());
        return pageInfo;
    }

    /**
     * 新增配置
     *
     * @param mapping RevenueTypeMapping
     */
    public void addConfig(RevenueTypeMapping mapping) {
        //todo 重复校验
        mapping.setId(idService.nextId());
        typeMappingMapper.insertSelective(mapping);
    }

    /**
     * 前台批量新增
     *
     * @param addDto RevTypeAddDto
     */
    public void addConfig(RevTypeAddDto addDto) {
        if (!CollectionUtils.isEmpty(addDto.getOrgList())) {
            addDto.getOrgList().forEach(id -> {
                addDto.setId(idService.nextId());
                addDto.setOrgId(id);
                typeMappingMapper.insertSelective(addDto);
            });
        }
    }

    /**
     * 更新配置
     *
     * @param mapping RevenueConfig
     */
    public void updateConfig(RevenueTypeMapping mapping) {
        //todo 重复校验
        typeMappingMapper.updateByPrimaryKeySelective(mapping);
    }

    public void delConfig(List<Long> idList) {
        if (!CollectionUtils.isEmpty(idList)) {
            idList.forEach(id -> typeMappingMapper.deleteByPrimaryKey(id));
        }
    }

    public void upload(MultipartFile file, Integer type) throws Exception {
        InputStream inputStream = file.getInputStream();
        Workbook workbook = WorkbookFactory.create(inputStream);
        Sheet sheet = workbook.getSheetAt(0);
        List<RevenueTypeMapping> list = new ArrayList<>();
        List<OrgCodeIdDto> orgDtoList = organizationService.getMyOrgCodeList();
        List<String> orgIds = new ArrayList<>();
        for (int r = sheet.getFirstRowNum()+1; r <= sheet.getLastRowNum(); r++) {
            Row row = sheet.getRow(r);
            String orgCode = String.valueOf(new BigDecimal(POIUtil.getCellFormulaString(row.getCell(0))).intValue());
            Optional<OrgCodeIdDto> optional = orgDtoList.stream().filter(o -> StringUtils.equals(o.getCode(), orgCode)).findFirst();
            if (!optional.isPresent()) {
                continue;
            }
            RevenueTypeMapping mapping = new RevenueTypeMapping();
            mapping.setId(idService.nextId());
            mapping.setOrgId(optional.get().getId());
            orgIds.add(optional.get().getId());
            mapping.setOuName(POIUtil.getCellFormulaString(row.getCell(1)));
            mapping.setContent(POIUtil.getCellFormulaString(row.getCell(2)));
            mapping.setTaxRate(new BigDecimal(POIUtil.getCellFormulaString(row.getCell(3))));
            mapping.setRevenueTypeName(POIUtil.getCellFormulaString(row.getCell(4)));
            mapping.setStartDate(POIUtil.getCellFormulaString(row.getCell(5)));
            mapping.setEndDate(StringUtils.defaultString(POIUtil.getCellFormulaString(row.getCell(6)), DEFAULT_END_DATE));
            mapping.setStatus(0);
            mapping.setCreateBy("Admin");
            mapping.setUpdateBy("Admin");
            list.add(mapping);
        }
        if (1 == type) {
            if(CollectionUtils.isNotEmpty(orgIds)){
                RevenueTypeMappingExample delExample = new RevenueTypeMappingExample();
                delExample.createCriteria().andOrgIdIn(orgIds);
                typeMappingMapper.deleteByExample(delExample);
            }
        }
        if (!CollectionUtils.isEmpty(list)) {
            List<List<RevenueTypeMapping>> batchList = CommonUtils.subListWithLen(list, CommonUtils.BATCH_NUM_2000);
            batchList.forEach(l -> typeMappingMapper.batchInsert(l));
        }

    }

}
