package pwc.taxtech.atms.service.impl;

import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.PDFRenderer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.constant.enums.FileUploadEnum;
import pwc.taxtech.atms.constant.enums.ReportFileUploadEnum;
import pwc.taxtech.atms.dao.TaxDocumentMapper;
import pwc.taxtech.atms.dto.TaxDocumentDto;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.entity.OperationLogTaxDocument;
import pwc.taxtech.atms.entity.Organization;
import pwc.taxtech.atms.entity.TaxDocument;
import pwc.taxtech.atms.entity.TaxDocumentExample;
import pwc.taxtech.atms.vat.entity.FileUpload;
import pwc.taxtech.atms.vat.entity.ReportFileUpload;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * 查询
 */
@Slf4j
@Service
public class TaxDocumentServiceImpl {

    @Resource
    private TaxDocumentMapper taxDocumentMapper;

    @Autowired
    private AuthUserHelper authUserHelper;

    @Autowired
    private OperationLogTaxDocServiceImpl operationLogTaxDocService;

    @Autowired
    ReportFileUploadService reportFileUploadService;

    @Autowired
    DidiFileUploadService didiFileUploadService;


    @Autowired
    private OrganizationServiceImpl organizationService;


    public List<TaxDocument> selectTaxDocumentList(TaxDocumentDto taxDocumentDto, List<String> orgIds) {

        List<TaxDocument> dataList = taxDocumentMapper.selectByExample(getExample(taxDocumentDto, orgIds));
        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
        fileParam.setUuids(dataList.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList()));
        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
        Map<String, String> urlMap = null;
        if (CollectionUtils.isNotEmpty(uploadDetail.getList())) {
            urlMap = uploadDetail.getList().stream().collect(Collectors.toMap(DidiFileUploadDetailResult::getUid, didiFileUploadDetailResult -> didiFileUploadDetailResult.getViewHttpUrl()));
        }
        if (urlMap != null) {
            for (TaxDocument data : dataList) {
                data.setFilePositionUrl(urlMap.get(data.getFileUploadId()));
            }
        }


        return dataList;
    }


    /**
     * 根据条件查询
     *
     * @param taxDocumentDto
     * @return
     */
    private TaxDocumentExample getExample(TaxDocumentDto taxDocumentDto, List<String> orgIds) {
        TaxDocumentExample example = new TaxDocumentExample();
        TaxDocumentExample.Criteria criteria = example.createCriteria();
        criteria.andCompanyIdIn(orgIds);
        //档案属性 fileAttr
        if (StringUtils.isNotBlank(taxDocumentDto.getFileAttr())) {
            criteria.andFileAttrEqualTo(taxDocumentDto.getFileAttr());
        }
        //档案类型 fileType
        if (StringUtils.isNotBlank(taxDocumentDto.getFileType())) {
            criteria.andFileTypeEqualTo(taxDocumentDto.getFileType());
        }
        //文件生效起始日期 fileBeginTime
        if (null != taxDocumentDto.getFileBeginTime()) {
            criteria.andFileTimeGreaterThanOrEqualTo(taxDocumentDto.getFileBeginTime());
        }
        //文件生效到期日期 fileEndTTime
        if (null != taxDocumentDto.getFileEndTTime()) {
            criteria.andFileTimeLessThanOrEqualTo(taxDocumentDto.getFileEndTTime());
        }
        //所属開始时间 ownBeginTime
        if (null != taxDocumentDto.getOwnBeginTime()) {
            criteria.andOwnTimeGreaterThanOrEqualTo(taxDocumentDto.getOwnBeginTime());
        }
        //所属結束时间 ownEndTime
        if (null != taxDocumentDto.getOwnEndTime()) {
            criteria.andOwnTimeLessThanOrEqualTo(taxDocumentDto.getOwnEndTime());
        }
        //档案名称 fileName
        if (StringUtils.isNotBlank(taxDocumentDto.getFileName())) {
            criteria.andFileNameLike("%" + taxDocumentDto.getFileName() + "%");
        }
        //业务线 businessLine
        if (StringUtils.isNotBlank(taxDocumentDto.getBusinessLine())) {
            criteria.andBusinessLineEqualTo(taxDocumentDto.getBusinessLine());
        }
        //公司名称 companyName
        if (StringUtils.isNotBlank(taxDocumentDto.getCompanyId())) {
            criteria.andCompanyIdEqualTo(taxDocumentDto.getCompanyId());
        }
        //公司名称 companyName
        if (StringUtils.isNotBlank(taxDocumentDto.getCompanyName())) {
            criteria.andCompanyNameEqualTo(taxDocumentDto.getCompanyName());
        }
        //到期起始日 effectiveBeginTime
        if (null != taxDocumentDto.getEffectiveBeginTime()) {
            criteria.andEffectiveTimeGreaterThanOrEqualTo(taxDocumentDto.getEffectiveBeginTime());
        }
        //到期结束日 effectiveEndTime
        if (null != taxDocumentDto.getEffectiveEndTime()) {
            criteria.andEffectiveTimeLessThanOrEqualTo(taxDocumentDto.getEffectiveEndTime());
        }
        //税种 taxType
        if (StringUtils.isNotBlank(taxDocumentDto.getTaxType())) {
            criteria.andTaxTypeEqualTo(taxDocumentDto.getTaxType());
        }
        //实物索引号 physicalIndexNumber
        if (StringUtils.isNotBlank(taxDocumentDto.getPhysicalIndexNumber())) {
            criteria.andPhysicalIndexNumberLike("%" + taxDocumentDto.getPhysicalIndexNumber() + "%");
        }
        //实物存放地点 storageArea
        if (StringUtils.isNotBlank(taxDocumentDto.getStorageArea())) {
            criteria.andStorageAreaLike("%" + taxDocumentDto.getStorageArea() + "%");
        }
        //实物保管人 keeper
        if (StringUtils.isNotBlank(taxDocumentDto.getKeeper())) {
            criteria.andKeeperLike("%" + taxDocumentDto.getKeeper() + "%");
        }
        //审批状态 auditStatus
        if (null != taxDocumentDto.getAuditStatus()) {
            criteria.andAuditStatusEqualTo(taxDocumentDto.getAuditStatus());
        }
        //上传开始日期 uploadBeginTime
        if (null != taxDocumentDto.getUploadBeginTime()) {
            criteria.andUploadTimeGreaterThanOrEqualTo(taxDocumentDto.getUploadBeginTime());
        }
        //上传结束日期 uploadEndTime
        if (null != taxDocumentDto.getUploadEndTime()) {
            criteria.andUploadTimeLessThanOrEqualTo(taxDocumentDto.getUploadEndTime());
        }
        //创建人 creator
        if (StringUtils.isNotBlank(taxDocumentDto.getCreator())) {
            criteria.andCreatorLike("%" + taxDocumentDto.getCreator() + "%");
        }
        //ids
        if (null != taxDocumentDto.getIds() && taxDocumentDto.getIds().size() > 0) {
            criteria.andIdIn(taxDocumentDto.getIds());
        }
        //设置查询可用的数据
        criteria.andEnableEqualTo("T");
        example.setOrderByClause("create_time DESC");
        return example;
    }

    @Transactional
    public synchronized boolean addTaxDocumentList(MultipartFile file, TaxDocument taxDocument) {
        try {
            //上传文件
            if (StringUtils.isBlank(taxDocument.getFileUploadId())) {

                if (ReportFileUploadEnum.ReportType.MAPPING.containsKey(taxDocument.getFileType())) {
                    //重新命名
                    String fileName = taxDocument.getCompanyName() + "_" + taxDocument.getFileType() + "_" + taxDocument.getOwnTime();
                    fileName += file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
                    FileUpload fileUpload = didiFileUploadService.uploadFile(file, fileName, FileUploadEnum.BizSource.RECORD_UPLOAD.name());
                    taxDocument.setFileUploadId(fileUpload.getUid());
                    taxDocument.setFilePositionUrl(fileUpload.getViewHttpUrl());
                    taxDocument.setFileName(fileName);
                    ReportFileUpload reportFileUpload = new ReportFileUpload();
                    reportFileUpload.setOrgId(taxDocument.getCompanyId());
                    reportFileUpload.setSourceType(ReportFileUploadEnum.SuorceType.RECORD.name());
                    String period = String.valueOf(taxDocument.getOwnTime()).substring(0, 6);
                    reportFileUpload.setPeriod(Integer.valueOf(period));
                    reportFileUpload.setFileUploadId(fileUpload.getUid());
                    reportFileUpload.setReportType(taxDocument.getFileType());
                    reportFileUpload.setReportFileName(fileName);
                    reportFileUploadService.saveData(file, reportFileUpload);
                } else {
                    FileUpload fileUpload = didiFileUploadService.uploadFile(file, file.getOriginalFilename(), FileUploadEnum.BizSource.RECORD_UPLOAD.name());
                    taxDocument.setFileUploadId(fileUpload.getUid());
                    taxDocument.setFilePositionUrl(fileUpload.getViewHttpUrl());
                }
            }
            //设置创建人 创建时间信息 设置年份区分
            taxDocument.setCreateTime(new Date());
            taxDocument.setUpdateTime(new Date());
            taxDocument.setCreator(authUserHelper.getCurrentAuditor().get());
            taxDocument.setCreatorId(authUserHelper.getCurrentUserId());
            taxDocument.setUploadTime(new Date());
            taxDocument.setYearRedundancy(Calendar.getInstance().get(Calendar.YEAR));
            //根据公司Id 设置业务线
            String businessLine = organizationService.queryBusinessByCompanyId(taxDocument.getCompanyId());
            if (StringUtils.isNotBlank(businessLine)) {
                taxDocument.setBusinessLine(businessLine);
            }
            int num = taxDocumentMapper.insert(taxDocument);
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setId(taxDocument.getId().toString());
                actionEntity.setOperationAction("新增");
                actionEntity.setUpdateState(taxDocument.toString());
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl addTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl addTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    public boolean deleteTaxDocument(Long id) {
        try {
            //逻辑删除
            int num = taxDocumentMapper.updateEnableToF(id);
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setOperationAction("删除");
                actionEntity.setId(id.toString());
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl deleteTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl deleteTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @Transactional
    public boolean batchDelete(List<Long> ids) {
        try {
            if (null == ids || ids.size() < 1) {
                return false;
            }
            //逻辑批量删除(批量修改)
            int num = taxDocumentMapper.batchDelete(ids);
            if (num > 0) {
                for (Long id : ids) {
                    OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                    actionEntity.setOperationAction("删除");
                    actionEntity.setId(id.toString());
                    boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                    if (!result) {
                        throw new RuntimeException("TaxDocumentServiceImpl batchDeleteTaxDocument log error");
                    }
                }
            } else {
                return false;
            }
            return true;
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl batchDelete error : " + e.getMessage());
            return false;
        }
    }

    public boolean editFilesType(TaxDocument taxDocument) {
        try {
            taxDocument.setUpdateTime(new Date());
            int num = taxDocumentMapper.updateByPrimaryKey(taxDocument);
            if (num > 0) {
                OperationLogTaxDocument actionEntity = buildOperationLogTaxDocument();
                actionEntity.setOperationAction("修改");
                actionEntity.setId(taxDocument.getId().toString());
                //设置更新值
                actionEntity.setUpdateState(taxDocument.toString());
                boolean result = operationLogTaxDocService.addTaxDocumentList(actionEntity);
                if (result) {
                    return true;
                } else {
                    throw new RuntimeException("TaxDocumentServiceImpl updateTaxDocument log error");
                }
            } else {
                return false;
            }
        } catch (Exception e) {
//            log.error("TaxDocumentServiceImpl updateTaxDocument error : " + e.getMessage());
            return false;
        }
    }

    /**
     * 构建日志对象
     *
     * @return
     */
    private OperationLogTaxDocument buildOperationLogTaxDocument() {
        OperationLogTaxDocument actionEntity = new OperationLogTaxDocument();
        actionEntity.setOperationContent("主页");
        actionEntity.setModuleName("档案管理");
        actionEntity.setOperationObject("税务档案");
        actionEntity.setOperationUser(authUserHelper.getCurrentAuditor().get());
        actionEntity.setIp(authUserHelper.getClientIp());
        actionEntity.setCreateTime(new Date());
        return actionEntity;
    }

    /**
     * 根据档案属性 档案类型 公司名称 档案名称 查询是否已存在数据
     * 不存在则返回 true
     * 已存在则返回 false
     *
     * @param taxDocument
     * @return
     */
    public TaxDocument queryWhetherData(TaxDocument taxDocument) {
        try {
            //简单参数校验
            TaxDocumentExample taxDocumentExample = new TaxDocumentExample();
            TaxDocumentExample.Criteria criteria = taxDocumentExample.createCriteria();
            //档案属性 fileAttr
            if (StringUtils.isNotBlank(taxDocument.getFileAttr())) {
                criteria.andFileAttrEqualTo(taxDocument.getFileAttr());
            }
            //档案类型 fileType
            if (StringUtils.isNotBlank(taxDocument.getFileType())) {
                criteria.andFileTypeEqualTo(taxDocument.getFileType());
            }
            //文件生效日期 fileTime
            if (null != taxDocument.getFileTime()) {
                criteria.andFileTimeEqualTo(taxDocument.getFileTime());
            }
            //所属时间 ownTime
            if (null != taxDocument.getOwnTime()) {
                criteria.andOwnTimeEqualTo(taxDocument.getOwnTime());
            }
            //档案名称 fileName
            if (StringUtils.isNotBlank(taxDocument.getFileName())) {
                criteria.andFileNameEqualTo(taxDocument.getFileName());
            }
            //业务线 businessLine
            if (StringUtils.isNotBlank(taxDocument.getBusinessLine())) {
                criteria.andBusinessLineEqualTo(taxDocument.getBusinessLine());
            }
            //公司名称 companyName
            if (StringUtils.isNotBlank(taxDocument.getCompanyName())) {
                criteria.andCompanyNameEqualTo(taxDocument.getCompanyName());
            }
            //到期日 effectiveTime
            if (null != taxDocument.getEffectiveTime()) {
                criteria.andEffectiveTimeEqualTo(taxDocument.getEffectiveTime());
            }
            //税种 taxType
            if (StringUtils.isNotBlank(taxDocument.getTaxType())) {
                criteria.andTaxTypeEqualTo(taxDocument.getTaxType());
            }
            //实物索引号 physicalIndexNumber
            if (StringUtils.isNotBlank(taxDocument.getPhysicalIndexNumber())) {
                criteria.andPhysicalIndexNumberEqualTo(taxDocument.getPhysicalIndexNumber());
            }
            //实物存放地点 storageArea
            if (StringUtils.isNotBlank(taxDocument.getStorageArea())) {
                criteria.andStorageAreaEqualTo(taxDocument.getStorageArea());
            }
            //实物保管人 keeper
            if (StringUtils.isNotBlank(taxDocument.getKeeper())) {
                criteria.andKeeperEqualTo(taxDocument.getKeeper());
            }
            //审批状态 auditStatus
            if (null != taxDocument.getAuditStatus()) {
                criteria.andAuditStatusEqualTo(taxDocument.getAuditStatus());
            }
            //创建人 creator
            if (StringUtils.isNotBlank(taxDocument.getCreator())) {
                criteria.andCreatorEqualTo(taxDocument.getCreator());
            }
            //设置查询可用的数据
            criteria.andEnableEqualTo("T");
            List<TaxDocument> taxDocuments = taxDocumentMapper.selectByExample(taxDocumentExample);
            if (null != taxDocuments && taxDocuments.size() > 0) {
                return taxDocuments.get(0);
            }
            return new TaxDocument();
        } catch (Exception e) {
            log.error("TaxDocumentServiceImpl queryWhetherData error :" + e.getMessage());
            return new TaxDocument();
        }
    }

    public void previewPDF(HttpServletResponse response ,String path) {
        String imageType = "PNG";
        OutputStream sos = null;
        try {
            sos = response.getOutputStream();
            PDFToImg(sos, path, getPDFNum(path), imageType);
            sos.flush();
        } catch (Exception e) {
            log.error("PDF转换图片异常: " + e.getMessage());
            throw new RuntimeException("PDF转换异常");
        } finally {
            try {
                sos.close();
            } catch (IOException e) {
                log.error("关闭输出流异常: " + e.getMessage());
            }
        }
    }
    public void downloadAllFile(HttpServletResponse response, List<Long> ids) {
        //如果只选择了一个附件,则不打包
        if (null != ids && ids.size() == 1) {
            TaxDocumentExample example = new TaxDocumentExample();
            TaxDocumentExample.Criteria criteria = example.createCriteria();
            criteria.andIdIn(ids);
            TaxDocument taxDocument = taxDocumentMapper.selectByExample(example).get(0);
            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
            fileParam.setUuids(Arrays.asList(taxDocument.getFileUploadId()));
            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
            String urlPath = uploadDetail.getList().get(0).getViewHttpUrl();
            //如果url为null或空字符串而抛出异常
            if (StringUtils.isBlank(urlPath)) {
                throw new RuntimeException("文件url为空,id为:" + taxDocument.getId());
            }
            //文件名称（带后缀） 文件名前+7位uuid截取码以防文件名相同
            String fileName = StringUtils.isBlank(uploadDetail.getList().get(0).getFileName())
                    ? "未知文件(请修改后缀名).xlsx"
                    : uploadDetail.getList().get(0).getFileName();//设置输出流信息
            try {
                response.setContentType("multipart/form-data");
                response.setHeader("Content-Disposition", "attachment;fileName=" + URLEncoder.encode(fileName, "UTF-8"));
            } catch (UnsupportedEncodingException e) {
                throw new RuntimeException("单个下载文件名编码时出现错误.", e);
            }
            //获取需要下载的文件流
            InputStream is = null;
            BufferedInputStream in = null;
            byte[] buffer = new byte[1024];
            int len;
            OutputStream out = null;
            try {
                URL httpurl = new URL(URLDecoder.decode(urlPath, "UTF-8"));
                HttpURLConnection httpConn = (HttpURLConnection) httpurl.openConnection();
                httpConn.setDoOutput(true);// 使用 URL 连接进行输出
                httpConn.setDoInput(true);// 使用 URL 连接进行输入
                httpConn.setUseCaches(false);// 忽略缓存
                httpConn.setRequestMethod("GET");// 设置URL请求方法
                //可设置请求头
                httpConn.setRequestProperty("Content-Type", "application/octet-stream");
                httpConn.setRequestProperty("Connection", "Keep-Alive");// 维持长连接
                httpConn.setRequestProperty("Charset", "UTF-8");
                httpConn.connect();
                if (httpConn.getResponseCode() >= 400) {
                    is = httpConn.getErrorStream();
                } else {
                    is = httpConn.getInputStream();
                }
                in = new BufferedInputStream(is);
                out = response.getOutputStream();
                //文件流循环写入ZipOutputStream
                while ((len = in.read(buffer)) != -1) {
                    out.write(buffer, 0, len);
                }
            } catch (IOException e) {
//                log.error("单个附件下载异常:" + e.getMessage());
            } finally {
                if (null != out) {
                    try {
                        out.close();
                    } catch (IOException e) {
//                        log.error("关闭输出流错误:" + e.getMessage());
                    }
                }
                if (null != is) {
                    try {
                        is.close();
                    } catch (IOException e) {
//                        log.error("关闭输入流错误:" + e.getMessage());
                    }
                }
                if (null != in) {
                    try {
                        in.close();
                    } catch (IOException e) {
//                        log.error("关闭缓存输入流错误:" + e.getMessage());
                    }
                }
            }
            return;
        }
        //如果选择了多个附件
        String downloadName = "多选附件.zip";
        try {
            response.setContentType("multipart/form-data");
            response.setHeader("Content-Disposition", "attachment;fileName=" + URLEncoder.encode(downloadName, "UTF-8"));
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("下载文件名编码时出现错误.", e);
        }
        OutputStream outputStream = null;
        ZipOutputStream zos = null;
        try {
            outputStream = response.getOutputStream();
            zos = new ZipOutputStream(outputStream);
            // 将文件流写入zip中
            downloadTolocal(zos, ids);

        } catch (IOException e) {
//            log.error("downloadAllFile-xxx下载全部附件失败，ids=[{}],错误信息=[{}]", ids, e);
        } finally {
            if (zos != null) {
                try {
                    zos.close();
                } catch (Exception e2) {
//                    log.info("关闭输入流时出现错误", e2);
                }
            }
            if (outputStream != null) {
                try {
                    outputStream.close();
                } catch (Exception e2) {
//                    log.info("关闭输入流时出现错误", e2);
                }
            }

        }

    }

    /**
     * // 将文件流写入zip中
     *
     * @param zos
     * @param ids
     */
    private void downloadTolocal(ZipOutputStream zos, List<Long> ids) {
        TaxDocumentExample example = new TaxDocumentExample();
        TaxDocumentExample.Criteria criteria = example.createCriteria();
        if (null == ids || ids.size() < 1) {
            throw new RuntimeException("传入参数错误:空参数数组");
        }
        criteria.andIdIn(ids);
        //查询出根据条件过滤后需要下载的附件数据信息
        List<TaxDocument> taxDocuments = taxDocumentMapper.selectByExample(example);
        DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
        fileParam.setUuids(taxDocuments.stream()
                .map(o -> o.getFileUploadId()).collect(Collectors.toList()));
        PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);

        //新建一个空集合,记录下载的名字,并用于判断是否重名,重名则修改名字为(n)
        List<String> fileNameList = new ArrayList<>();
        for (DidiFileUploadDetailResult item : uploadDetail.getList()) {
            //文件url
            String urlPath = item.getViewHttpUrl();
//            //如果url为null或空字符串而抛出异常
//            if (StringUtils.isBlank(urlPath)) {
//                throw new RuntimeException("文件url为空,id为:" + item.getId());
//            }
            //检查文件名是否有重复
            int fileNameRepeatTimes = 0;
            if (fileNameList.size() > 0) {
                fileNameRepeatTimes = Collections.frequency(fileNameList, item.getFileName());
            }
            String fileName;
            if (fileNameRepeatTimes > 0) {
                fileName = StringUtils.isBlank(item.getFileName())
                        ? "未知文件(请修改后缀名).xlsx"
                        : "(" + fileNameRepeatTimes + ")" + item.getFileName();
            } else {
                //文件名称（带后缀
                fileName = StringUtils.isBlank(item.getFileName())
                        ? "未知文件(请修改后缀名).xlsx"
                        : item.getFileName();
            }
            fileNameList.add(item.getFileName());
            InputStream is = null;
            BufferedInputStream in = null;
            byte[] buffer = new byte[1024];
            int len;
            //创建zip实体（一个文件对应一个ZipEntry）
            ZipEntry entry = new ZipEntry(fileName);
            try {
                //获取需要下载的文件流
                URL httpurl = new URL(URLDecoder.decode(urlPath, "UTF-8"));
                HttpURLConnection httpConn = (HttpURLConnection) httpurl.openConnection();
                httpConn.setDoOutput(true);// 使用 URL 连接进行输出
                httpConn.setDoInput(true);// 使用 URL 连接进行输入
                httpConn.setUseCaches(false);// 忽略缓存
                httpConn.setRequestMethod("GET");// 设置URL请求方法
                //可设置请求头
                httpConn.setRequestProperty("Content-Type", "application/octet-stream");
                httpConn.setRequestProperty("Connection", "Keep-Alive");// 维持长连接
                httpConn.setRequestProperty("Charset", "UTF-8");
                httpConn.connect();
                if (httpConn.getResponseCode() >= 400) {
                    is = httpConn.getErrorStream();
                } else {
                    is = httpConn.getInputStream();
                }
                in = new BufferedInputStream(is);
                zos.putNextEntry(entry);
                //文件流循环写入ZipOutputStream
                while ((len = in.read(buffer)) != -1) {
                    zos.write(buffer, 0, len);
                }
            } catch (Exception e) {
//                log.info("xxx--下载全部附件--压缩文件出错", e);
            } finally {
                if (entry != null) {
                    try {
                        zos.closeEntry();
                    } catch (Exception e2) {
//                        log.info("xxx下载全部附件--zip实体关闭失败", e2);
                    }
                }
                if (in != null) {
                    try {
                        in.close();
                    } catch (Exception e2) {
//                        log.info("xxx下载全部附件--文件输入流关闭失败", e2);
                    }
                }
                if (is != null) {
                    try {
                        is.close();
                    } catch (Exception e) {
//                        log.info("xxx下载全部附件--输入缓冲流关闭失败", e);
                    }
                }


            }

        }
    }

    public Map<String, Object> multipalInitData(String address) {
        if (StringUtils.isBlank(address)) {
            throw new RuntimeException("地址格式错误");
        }
        Map<String, Object> resultMap = Maps.newHashMap();
        //初始化一个错误文件名列表 和 正确文件名列表
        ArrayList<String> errorFileNameList = Lists.newArrayList();
        ArrayList<String> successFileNameList = Lists.newArrayList();
        ArrayList<String> existedFileNameList = Lists.newArrayList();
        //读取固定文件目录下的所有文件的文件名
        File iniTfile = new File(address);
        if (!iniTfile.isDirectory() || !iniTfile.exists()) {
            throw new RuntimeException("文件夹地址错误");
        }
        List<File> files = Arrays.asList(iniTfile.listFiles());
        if (files.size() < 1) {
            throw new RuntimeException("文件列表为空");
        }
        /*List<String> fileNames = files.stream().filter(e -> e.isFile()).map(File::getName).collect(Collectors.toList());
        if (fileNames.size()<1){
            throw new RuntimeException("文件列表为空");
        }*/
        //公司名简称和公司序号Map(name-id)
        Map<String, String> companyNameAndId;
        try {
            companyNameAndId = organizationService.findAllOrganizations().stream().collect(Collectors.toMap(Organization::getAbbreviation, Organization::getId, (value1, value2) -> {
                return value2;
            }));
        } catch (Exception e) {
            throw new RuntimeException("公司简称和id获取失败!请联系管理员");
        }
        if (companyNameAndId.size() < 1) {
            throw new RuntimeException("公司名称列表无数据!");
        }
        //递归处理
        recursiveSolveFile(errorFileNameList, successFileNameList, existedFileNameList, files, companyNameAndId);
        //返回成功列表和失败列表
        resultMap.put("successFileNameList", successFileNameList);
        resultMap.put("errorFileNameList", errorFileNameList);
        resultMap.put("existedFileNameList", existedFileNameList);
        return resultMap;
    }

    /**
     * 递归循环文件夹 处理
     *
     * @param errorFileNameList
     * @param successFileNameList
     * @param existedFileNameList
     * @param files
     * @param companyNameAndId
     */
    private void recursiveSolveFile(ArrayList<String> errorFileNameList, ArrayList<String> successFileNameList, ArrayList<String> existedFileNameList, List<File> files, Map<String, String> companyNameAndId) {
        files.forEach(file -> {
            if (file.isDirectory()) {
                recursiveSolveFile(errorFileNameList, successFileNameList, existedFileNameList, Arrays.asList(file.listFiles()), companyNameAndId);
                return;
            }
            String fileName = file.getName();//文件名/
            String filePath = file.getPath();//文件路径
            //文件名校验 如果文件名中不包含_则不符合规则,添加到错误列表中
            if (!fileName.contains("_")) {
                errorFileNameList.add(filePath);
                return;//此处相当于continue
            }
            //根据文件名格式分割,以 "_"来分割
            try {
                List<String> params = Arrays.asList(fileName.substring(0, fileName.indexOf(".")).split("_"));
                //根据分割的长度来进行匹配对比 如果长度<3或者>6或公司简称匹配不上则添加至失败列表(目前最少的是3个,若以后有更改请更改此处参数)
                if (params.size() < 3 || params.size() > 6 || !companyNameAndId.containsKey(params.get(0))) {
                    errorFileNameList.add(filePath);
                    return;//此处相当于continue
                }
                //根据档案类型匹配并初始化值
                matchFileTypeToInitData(errorFileNameList, successFileNameList, existedFileNameList, companyNameAndId, file, params, filePath);
            } catch (Exception e) {
                e.printStackTrace();
                errorFileNameList.add(filePath);
                return;//此处相当于continue
            }
        });
    }

    //初始化税种列表(后续可做成从数据库中拿)
    private final static List<String> taxsList = Arrays.asList("纳税申报表", "税票", "企业所得税", "印花税", "附加税", "代扣代缴税", "其它税", "增值税");

    /**
     * 分组 长度为3
     */
    //档案类型分组:公司簡稱_档案类型_文件生效日期
    private final static Map<String, String> groupOfCompanyFileTypeFIleTime = Maps.newHashMap();
    /**
     * 分组 长度为4
     */
    //档案类型分组:公司簡稱_档案类型_所属期间_文件生效日期
    private final static Map<String, String> groupOfCompanyFileTypeOwnTimeFileTime = Maps.newHashMap();
    //档案类型分组:公司簡稱_档案类型_所属期间_到期日
    private final static Map<String, String> groupOfCompanyFileTypeOwnTimeEffectiveTime = Maps.newHashMap();
    //档案类型分组:公司簡稱_档案属性_档案类型_文件生效日期
    private final static Map<String, String> internationalGroupOfCompanyFileAttrFileTypeFileTime = Maps.newHashMap();
    //档案类型分组:公司簡稱_档案属性_档案类型_所属期间(国际档案类型)
    private final static Map<String, String> internationalGroupOfCompanyFileAttrFileTypeOwnTime = Maps.newHashMap();
    /**
     * 分组 长度为5
     */
    //档案类型分组:公司簡稱_档案类型_税种_所属期间_文件生效日期
    private final static Map<String, String> groupOfCompanyFileTypeTaxTypeOwnTimeFileTime = Maps.newHashMap();
    //档案类型分组:公司簡稱_档案属性_档案类型_税种_所属期间(国际档案类型)
    private final static Map<String, String> internationalGroupOfCompanyFileAttrFileTypeTaxTypeOwnTime = Maps.newHashMap();
    /**
     * 分组 长度为6
     */
    //档案类型分组:公司簡稱_档案类型_所属期间_文件生效日期_实物存放地点_实物保管人
    private final static Map<String, String> groupOfCompanyFileTypeOwnTimeFileTimeStorageAreaKeeper = Maps.newHashMap();

    //初始化档案类型-档案属性map列表(后续可做成从数据库中拿)
    static {
        //添加分组
        //公司簡稱_档案类型_文件生效日期
        setGroupOfCompanyFileTypeFIleTime();
        //公司簡稱_档案属性_档案类型_文件生效日期
        setInternationalGroupOfCompanyFileAttrFileTypeFileTime();
        //公司簡稱_档案类型_所属期间_文件生效日期
        setGroupOfCompanyFileTypeOwnTimeFileTime();
        //公司簡稱_档案类型_所属期间_到期日
        setGroupOfCompanyFileTypeOwnTimeEffectiveTime();
        //公司簡稱_档案属性_档案类型_所属期间
        setInternationalGroupOfCompanyFileAttrFileTypeOwnTime();
        //公司簡稱_档案类型_税种_所属期间_文件生效日期
        setGroupOfCompanyFileTypeTaxTypeOwnTimeFileTime();
        //公司簡稱_档案属性_档案类型_税种_所属期间(国际档案类型)
        setInternationalGroupOfCompanyFileAttrFileTypeTaxTypeOwnTime();
        //公司簡稱_档案类型_所属期间_文件生效日期_实物存放地点_实物保管人
        setGroupOfCompanyFileTypeOwnTimeFileTimeStorageAreaKeeper();
    }

    /**
     * 公司簡稱_档案类型_文件生效日期
     */
    private static void setGroupOfCompanyFileTypeFIleTime() {
        groupOfCompanyFileTypeFIleTime.put("设立税务登记资料", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("变更税务登记资料", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("注销税务登记资料", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("扣缴义务人登记资料", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("存款账户账号备案资料", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("财务会计制度备案资料", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("外出经营报验登记", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("税源管理项目资料", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("委托代征登记资料、损毁01 税务登记类", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("发票挂失、损毁、丢失资料", "06 发票管理类");
        groupOfCompanyFileTypeFIleTime.put("发票领购资料", "06 发票管理类");
        groupOfCompanyFileTypeFIleTime.put("发票核销资料", "06 发票管理类");
        groupOfCompanyFileTypeFIleTime.put("开具红字发票资料", "06 发票管理类");
        groupOfCompanyFileTypeFIleTime.put("非居民享受税收协定待遇认定", "02 税务认定类");
        groupOfCompanyFileTypeFIleTime.put("汇总（合并）纳税企业信息备案资料", "01 税务登记类");
        groupOfCompanyFileTypeFIleTime.put("发票票种核定资料", "06 发票管理类");
        groupOfCompanyFileTypeFIleTime.put("发票用票量调整资料", "06 发票管理类");
        groupOfCompanyFileTypeFIleTime.put("增值税税收优惠资料", "03 税收优惠类");
        groupOfCompanyFileTypeFIleTime.put("车辆购置税税收优惠资料", "03 税收优惠类");
        groupOfCompanyFileTypeFIleTime.put("企业所得税税收优惠资料", "03 税收优惠类");
        groupOfCompanyFileTypeFIleTime.put("税收减免审批资料", "03 税收优惠类");
        groupOfCompanyFileTypeFIleTime.put("税收减免备案资料", "03 税收优惠类");
        groupOfCompanyFileTypeFIleTime.put("纳税申报方式认定", "02 税务认定类");
        groupOfCompanyFileTypeFIleTime.put("一般纳税人认定", "02 税务认定类");
        groupOfCompanyFileTypeFIleTime.put("征收方式认定", "02 税务认定类");
        groupOfCompanyFileTypeFIleTime.put("税种核定资料", "04 申报征收类");
    }

    /**
     * 公司簡稱_档案属性_档案类型_文件生效日期
     */
    private static void setInternationalGroupOfCompanyFileAttrFileTypeFileTime() {
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Legal Document - Other", "International Tax - Legal Document");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Ruling Application documents", "International tax - Tax Rulings");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Incentive Approval documents", "International tax - Tax Incentives");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Article of incorporation", "International Tax - Legal Document");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Business registration", "International Tax - Legal Document");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("CIT", "International Tax - Tax Registration");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("VAT", "International Tax - Tax Registration");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Letter from Tax Authority", "International Tax - Tax Inspection");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Reply letter and supporting", "International Tax - Tax Inspection");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Tax Registration - Other", "International Tax - Tax Registration");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Incentive Application documents", "International tax - Tax Incentives");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Ruling Approval documents", "International tax - Tax Rulings");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Tax Incentives - Other", "International tax - Tax Incentives");
        internationalGroupOfCompanyFileAttrFileTypeFileTime.put("Tax Inspection - Other", "International Tax - Tax Inspection");
    }

    /**
     * 公司簡稱_档案类型_所属期间_文件生效日期
     */
    private static void setGroupOfCompanyFileTypeOwnTimeFileTime() {
        groupOfCompanyFileTypeOwnTimeFileTime.put("重点税源申报表", "04 申报征收类");
        groupOfCompanyFileTypeOwnTimeFileTime.put("千户集团申报表", "04 申报征收类");
        groupOfCompanyFileTypeOwnTimeFileTime.put("年度减免税调查表", "04 申报征收类");
        groupOfCompanyFileTypeOwnTimeFileTime.put("全国税收调查表", "04 申报征收类");
        groupOfCompanyFileTypeOwnTimeFileTime.put("重点税源计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeOwnTimeFileTime.put("千户集团计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeOwnTimeFileTime.put("年度减免税调查计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeOwnTimeFileTime.put("全国税收调查计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeOwnTimeFileTime.put("关联业务往来报告表", "04 申报征收类");
    }

    /**
     * 公司簡稱_档案类型_所属期间_到期日
     */
    private static void setGroupOfCompanyFileTypeOwnTimeEffectiveTime() {
        groupOfCompanyFileTypeOwnTimeEffectiveTime.put("审计报告", "05 外部鉴证类");
        groupOfCompanyFileTypeOwnTimeEffectiveTime.put("税审报告", "05 外部鉴证类");
        groupOfCompanyFileTypeOwnTimeEffectiveTime.put("财产损失报告", "05 外部鉴证类");
        groupOfCompanyFileTypeOwnTimeEffectiveTime.put("其他鉴证报告", "05 外部鉴证类");
    }

    /**
     * 公司簡稱_档案类型_所属期间_到期日
     */
    private static void setInternationalGroupOfCompanyFileAttrFileTypeOwnTime() {
        internationalGroupOfCompanyFileAttrFileTypeOwnTime.put("Other tax return and supporting", "International tax - Tax Filling");
        internationalGroupOfCompanyFileAttrFileTypeOwnTime.put("FS and GL", "International tax - Financial statements");
        internationalGroupOfCompanyFileAttrFileTypeOwnTime.put("Audited report", "International tax - Audited report");
    }

    /**
     * 公司簡稱_档案类型_税种_所属期间_文件生效日期
     */
    private static void setGroupOfCompanyFileTypeTaxTypeOwnTimeFileTime() {
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("增值税纳税申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("企业所得税预缴申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("企业所得税汇算清缴申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("财务会计报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("印花税申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("附加税费申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("代扣代缴税申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("其它税（费）申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("增值税缴税银行回单", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("企业所得税缴税银行回单", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("企业所得税汇算清缴缴税银行回单/退税银行回单", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("印花税缴税银行回单", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("附加税费缴税银行回单", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("代扣代缴缴税银行回单", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("其它税（费）缴税银行回单", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("增值税更正申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("企业所得税更正申报表", "04 申报征收类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("增值税及附加税费计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("企业所得税预缴计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("企业所得税汇算清缴计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("印花税计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("代扣代缴税计算底稿", "07 内部文件类");
        groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.put("其它税（费）计算底稿", "07 内部文件类");
    }

    /**
     * 公司簡稱_档案属性_档案类型_税种_所属期间(国际档案类型)
     */
    private static void setInternationalGroupOfCompanyFileAttrFileTypeTaxTypeOwnTime() {
        internationalGroupOfCompanyFileAttrFileTypeTaxTypeOwnTime.put("CIT Return and supporting", "International tax - Tax Filling");
        internationalGroupOfCompanyFileAttrFileTypeTaxTypeOwnTime.put("VAT return and supporting", "International tax - Tax Filling");
    }

    /**
     * 公司簡稱_档案类型_所属期间_文件生效日期_实物存放地点_实物保管人
     */
    private static void setGroupOfCompanyFileTypeOwnTimeFileTimeStorageAreaKeeper() {
        groupOfCompanyFileTypeOwnTimeFileTimeStorageAreaKeeper.put("发票代开资料", "06 发票管理类");
    }

    /**
     * 构建基础对象 设置基础值 公司简称 Id 档案属性 档案类型
     *
     * @return
     */
    private TaxDocument buildBaseTaxDocument(Map<String, String> companyNameAndId, List<String> params, String fileName) {
        TaxDocument taxDocument = new TaxDocument();
        taxDocument.setFileName(fileName);
        taxDocument.setCompanyId(companyNameAndId.get(params.get(0)));
        taxDocument.setCompanyName(params.get(0));
        taxDocument.setCreateTime(new Date());
        taxDocument.setUpdateTime(new Date());
        taxDocument.setCreator("System");
        taxDocument.setCreatorId("1");
        taxDocument.setUploadTime(new Date());
        taxDocument.setYearRedundancy(Calendar.getInstance().get(Calendar.YEAR));
        //根据公司Id 设置业务线
        String businessLine = organizationService.queryBusinessByCompanyId(taxDocument.getCompanyId());
        if (StringUtils.isNotBlank(businessLine)) {
            taxDocument.setBusinessLine(businessLine);
        }
        return taxDocument;
    }

    /**
     * 根据档案类型匹配并初始化值
     *
     * @param errorFileNameList
     * @param successeFileNameList
     * @param companyNameAndId
     * @param params
     */
    private void matchFileTypeToInitData(ArrayList<String> errorFileNameList, ArrayList<String> successeFileNameList, ArrayList<String> existedFileNameList, Map<String, String> companyNameAndId, File file, List<String> params, String filePath) {
        //长度为3(先验证是否国际类型并校验匹配,若是国际类型档案类型索引为2(国际类型第三个为档案类型),否则索引为1为档案类型
        if (params.size() == 3//公司簡稱_档案类型_文件生效日期
                && groupOfCompanyFileTypeFIleTime.keySet().contains(params.get(1))
                && null != DateUtils.stringToDate4yyyyMMdd(params.get(2))){//文件生效日期校验
            //构建基础对象 设置基础值 公司简称 Id 文件名
            TaxDocument taxDocument = buildBaseTaxDocument(companyNameAndId,params,file.getName());
            //设置档案属性 档案类型 文件生效日期
            taxDocument.setFileType(params.get(1));
            taxDocument.setFileAttr(groupOfCompanyFileTypeFIleTime.get(params.get(1)));
            taxDocument.setFileTime(DateUtils.stringToDate4yyyyMMdd(params.get(2)));
            //校验是否已存在
            checkExistedAndInsertOrUpdate(successeFileNameList, existedFileNameList, file, taxDocument,filePath);
        }else if (params.size() == 4 //国际类型 公司簡稱_档案属性_档案类型_文件生效日期
                && (internationalGroupOfCompanyFileAttrFileTypeFileTime.keySet().contains(params.get(2))
                && internationalGroupOfCompanyFileAttrFileTypeFileTime.values().contains(params.get(1)))
                && null != DateUtils.stringToDate4yyyyMMdd(params.get(3))){//文件生效日期校验
            //构建基础对象 设置基础值 公司简称 Id 文件名
            TaxDocument taxDocument = buildBaseTaxDocument(companyNameAndId,params,file.getName());
            //设置档案属性 档案类型
            taxDocument.setFileAttr(params.get(1));
            taxDocument.setFileType(params.get(2));
            //校验是否已存在
            checkExistedAndInsertOrUpdate(successeFileNameList, existedFileNameList, file, taxDocument,filePath);
        }else if (params.size() == 4 //国际类型 公司簡稱_档案属性_档案类型_所属期间
                && (internationalGroupOfCompanyFileAttrFileTypeOwnTime.keySet().contains(params.get(2))
                && internationalGroupOfCompanyFileAttrFileTypeOwnTime.values().contains(params.get(1)))
                && checkOwnTime(params.get(3))){//所属期间校验
            //构建基础对象 设置基础值 公司简称 Id 文件名
            TaxDocument taxDocument = buildBaseTaxDocument(companyNameAndId,params,file.getName());
            //设置档案属性 档案类型 所属期间
            taxDocument.setFileAttr(params.get(1));
            taxDocument.setFileType(params.get(2));
            taxDocument.setOwnTime(Integer.valueOf(params.get(3)));
            //校验是否已存在
            checkExistedAndInsertOrUpdate(successeFileNameList, existedFileNameList, file, taxDocument,filePath);
        }else if (params.size() == 4 //公司簡稱_档案类型_所属期间_文件生效日期
                && groupOfCompanyFileTypeOwnTimeFileTime.keySet().contains(params.get(1))
                && checkOwnTime(params.get(2))//所属期间校验
                && null != DateUtils.stringToDate4yyyyMMdd(params.get(3))){//文件生效日期校验
            //构建基础对象 设置基础值 公司简称 Id 文件名
            TaxDocument taxDocument = buildBaseTaxDocument(companyNameAndId,params,file.getName());
            //设置档案类型 档案属性 所属期间 文件生效日期
            taxDocument.setFileType(params.get(1));
            taxDocument.setFileAttr(groupOfCompanyFileTypeOwnTimeFileTime.get(params.get(1)));
            taxDocument.setOwnTime(Integer.valueOf(params.get(2)));
            taxDocument.setFileTime(DateUtils.stringToDate4yyyyMMdd(params.get(3)));
            //校验是否已存在
            checkExistedAndInsertOrUpdate(successeFileNameList, existedFileNameList, file, taxDocument,filePath);
        }else if (params.size() == 4 //公司簡稱_档案类型_所属期间_到期日
                && groupOfCompanyFileTypeOwnTimeEffectiveTime.keySet().contains(params.get(1))
                && checkOwnTime(params.get(2))//所属期间校验
                && null != DateUtils.stringToDate4yyyyMMdd(params.get(3))){//到期日校验
            //构建基础对象 设置基础值 公司简称 Id 文件名
            TaxDocument taxDocument = buildBaseTaxDocument(companyNameAndId,params,file.getName());
            //设置档案类型 档案属性 所属期间 到期日
            taxDocument.setFileType(params.get(1));
            taxDocument.setFileAttr(groupOfCompanyFileTypeOwnTimeEffectiveTime.get(params.get(1)));
            taxDocument.setOwnTime(Integer.valueOf(params.get(2)));
            taxDocument.setEffectiveTime(DateUtils.stringToDate4yyyyMMdd(params.get(3)));
            //校验是否已存在
            checkExistedAndInsertOrUpdate(successeFileNameList, existedFileNameList, file, taxDocument,filePath);
        }else if (params.size() == 5 //国际类型 公司簡稱_档案属性_档案类型_税种_所属期间
                && (internationalGroupOfCompanyFileAttrFileTypeTaxTypeOwnTime.keySet().contains(params.get(2))
                && internationalGroupOfCompanyFileAttrFileTypeTaxTypeOwnTime.values().contains(params.get(1)))
                && taxsList.contains(params.get(3))//税种校验
                && checkOwnTime(params.get(4))){//所属期间校验
            //构建基础对象 设置基础值 公司简称 Id 文件名
            TaxDocument taxDocument = buildBaseTaxDocument(companyNameAndId,params,file.getName());
            //设置档案属性 档案类型 税种 所属期间
            taxDocument.setFileAttr(params.get(1));
            taxDocument.setFileType(params.get(2));
            taxDocument.setTaxType(params.get(3));
            taxDocument.setOwnTime(Integer.valueOf(params.get(4)));
            //校验是否已存在
            checkExistedAndInsertOrUpdate(successeFileNameList, existedFileNameList, file, taxDocument,filePath);
        }else if (params.size() == 5 //公司簡稱_档案类型_税种_所属期间_文件生效日期
                && groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.keySet().contains(params.get(1))
                && taxsList.contains(params.get(2))//税种校验
                && checkOwnTime(params.get(3))//所属期间校验
                && null != DateUtils.stringToDate4yyyyMMdd(params.get(4))){//文件生效日期校验
            //构建基础对象 设置基础值 公司简称 Id 文件名
            TaxDocument taxDocument = buildBaseTaxDocument(companyNameAndId,params,file.getName());
            //设置档案类型 档案属性 税种 所属期间 文件生效日期
            taxDocument.setFileType(params.get(1));
            taxDocument.setFileAttr(groupOfCompanyFileTypeTaxTypeOwnTimeFileTime.get(params.get(1)));
            taxDocument.setTaxType(params.get(2));
            taxDocument.setOwnTime(Integer.valueOf(params.get(3)));
            taxDocument.setFileTime(DateUtils.stringToDate4yyyyMMdd(params.get(4)));
            //校验是否已存在
            checkExistedAndInsertOrUpdate(successeFileNameList, existedFileNameList, file, taxDocument,filePath);
        }else if (params.size() == 6 //公司簡稱_档案类型_所属期间_文件生效日期_实物存放地点_实物保管人
                && groupOfCompanyFileTypeOwnTimeFileTimeStorageAreaKeeper.keySet().contains(params.get(1))
                && checkOwnTime(params.get(2))//所属期间校验
                && null != DateUtils.stringToDate4yyyyMMdd(params.get(3))){//文件生效日期校验
            //构建基础对象 设置基础值 公司简称 Id 文件名
            TaxDocument taxDocument = buildBaseTaxDocument(companyNameAndId,params,file.getName());
            //设置档案类型 档案属性 所属期间 文件生效日期 实物存放地点 实物保管人
            taxDocument.setFileType(params.get(1));
            taxDocument.setFileAttr(groupOfCompanyFileTypeOwnTimeFileTimeStorageAreaKeeper.get(params.get(1)));
            taxDocument.setOwnTime(Integer.valueOf(params.get(2)));
            taxDocument.setFileTime(DateUtils.stringToDate4yyyyMMdd(params.get(3)));
            taxDocument.setStorageArea(params.get(4));
            taxDocument.setKeeper(params.get(5));
            //校验是否已存在
            checkExistedAndInsertOrUpdate(successeFileNameList, existedFileNameList, file, taxDocument,filePath);
        }else {
            errorFileNameList.add(filePath);
        }
    }

    private void checkExistedAndInsertOrUpdate(ArrayList<String> successeFileNameList, ArrayList<String> existedFileNameList, File file, TaxDocument taxDocument, String filePath) {
        if (null == queryWhetherData(taxDocument).getId()) {
            //TODO 上传文件 并填充文件url 上传人等信息
            FileInputStream fis = null;
            ByteArrayOutputStream bos = null;
            try {
                fis = new FileInputStream(file);
                bos = new ByteArrayOutputStream(1000);
                byte[] b = new byte[1000];
                int n;
                while ((n = fis.read(b)) != -1) {
                    bos.write(b, 0, n);
                }
                FileUpload fileUpload = didiFileUploadService.uploadFile(bos.toByteArray(), file.getName(), FileUploadEnum.BizSource.RECORD_UPLOAD.name());
                taxDocument.setFileUploadId(fileUpload.getUid());
                taxDocument.setFilePositionUrl(fileUpload.getViewHttpUrl());
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                try {
                    fis.close();
                    bos.close();
                } catch (Exception e) {

                }
            }
            //插入数据
            taxDocumentMapper.insert(taxDocument);
            //添加到成功列表中
            successeFileNameList.add(filePath);
        } else {
            //添加到已存在列表中
            existedFileNameList.add(filePath);
            //TODO 或做覆盖操作
        }
    }

    /**
     * 检查所属期间转换是否正常
     *
     * @param s
     * @return
     */
    private boolean checkOwnTime(String s) {
        try {
            return Integer.valueOf(s) == null ? false : true;
        } catch (NumberFormatException e) {
            log.error("所属期间转换异常: " + s);
            return false;
        }
    }

    /**
     * 获取PDF总页数
     * @throws IOException
     */
    public int getPDFNum(String fileUrl) throws IOException {
        PDDocument pdDocument = null;
        int pages = 0;
        try {
            pdDocument = getPDDocument(fileUrl);
            pages = pdDocument.getNumberOfPages();
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(),e);
        } finally {
            if (pdDocument != null) {
                pdDocument.close();
            }
        }
        return pages;
    }


    /**
     * PDF转图片 根据页码一页一页转
     * @throws IOException
     * imgType:转换后的图片类型 jpg,png
     */
    private void PDFToImg(OutputStream sos,String fileUrl,int page,String imgType) throws Exception {
        PDDocument pdDocument = null;
        /* dpi越大转换后越清晰，相对转换速度越慢 */
        int dpi = 100;
        try {
            pdDocument = getPDDocument(fileUrl);
            PDFRenderer renderer = new PDFRenderer(pdDocument);
            int pages = pdDocument.getNumberOfPages();
            if (page <= pages && page > 0) {
                BufferedImage image = renderer.renderImageWithDPI(page-1,dpi);
                ImageIO.write(image, imgType, sos);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(),e);
            throw new RuntimeException("PDF转换异常");
        } finally {
            if (pdDocument != null) {
                pdDocument.close();
            }
        }

    }


    private PDDocument getPDDocument(String fileUrl) throws IOException {
        InputStream inputStream = getInputStreamByUrl(fileUrl);
        return PDDocument.load(inputStream);
    }

    /**
     * 根据url地址 获取输入流
     * @param url
     * @return
     * @throws IOException
     */
    public InputStream getInputStreamByUrl(String url) throws IOException {
        URL httpurl=new URL(URLDecoder.decode(url, "UTF-8"));
        InputStream is;
        HttpURLConnection httpConn=(HttpURLConnection)httpurl.openConnection();
        httpConn.setDoOutput(true);// 使用 URL 连接进行输出
        httpConn.setDoInput(true);// 使用 URL 连接进行输入
        httpConn.setUseCaches(false);// 忽略缓存
        httpConn.setRequestMethod("GET");// 设置URL请求方法
        //可设置请求头
        httpConn.setRequestProperty("Content-Type", "application/octet-stream");
        httpConn.setRequestProperty("Connection", "Keep-Alive");// 维持长连接
        httpConn.setRequestProperty("Charset", "UTF-8");
        httpConn.connect();
        if (httpConn.getResponseCode() >= 400 ) {
            is = httpConn.getErrorStream();
        }
        else{
            is = httpConn.getInputStream();
        }
        return is;
    }

    public String previewPDF2(HttpServletResponse response , String path) {
        InputStream is = null;
        ByteArrayOutputStream os = null;
        String dUrlData;
        //pdf源路径
        byte[] buff = new byte[1024];
        int len;
        try {
            is = getInputStreamByUrl(path);
            os = new ByteArrayOutputStream();
            while ((len = is.read(buff)) != -1) {
                os.write(buff, 0, len);
            }

            os.flush();
            os.toByteArray();
            dUrlData = Base64.getEncoder().encodeToString(os.toByteArray());
        } catch (IOException e) {
            log.error(e.getMessage());
            throw new RuntimeException("pdf转换错误");
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    log.error(e.getMessage());
                    throw new RuntimeException("pdf转换错误");
                }
            }
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e) {
                    log.error(e.getMessage());
                    throw new RuntimeException("pdf转换错误");
                }
            }
        }
        return dUrlData;
    }
}
