package pwc.taxtech.atms.vat.service.impl;
import java.util.Date;

import org.activiti.engine.HistoryService;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.history.HistoricActivityInstance;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.task.Attachment;
import org.activiti.engine.task.Task;
import org.nutz.lang.Strings;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.dao.RoleMapper;
import pwc.taxtech.atms.dao.TemplateMapper;
import pwc.taxtech.atms.dao.UserMapper;
import pwc.taxtech.atms.dao.UserRoleMapper;
import pwc.taxtech.atms.dto.approval.ApprovalDto;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.ApplicationException;
import pwc.taxtech.atms.exception.BadParameterException;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.dao.PeriodApproveMapper;
import pwc.taxtech.atms.vat.dao.PeriodReportMapper;
import pwc.taxtech.atms.vat.dpo.ApprovalTaskInfo;
import pwc.taxtech.atms.vat.dpo.ApprovalTaskNodeInfo;
import pwc.taxtech.atms.vat.entity.PeriodApprove;
import pwc.taxtech.atms.vat.entity.PeriodApproveExample;
import pwc.taxtech.atms.vat.entity.PeriodReport;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class ApprovalService {
    private static Logger logger = LoggerFactory.getLogger(ApprovalService.class);
    @Autowired
    RuntimeService runtimeService;
    @Autowired
    TaskService taskService;
    @Autowired
    HistoryService historyService;
    @Autowired
    PeriodApproveMapper periodApproveMapper;
    @Autowired
    PeriodReportMapper reportMapper;
    @Autowired
    AuthUserHelper authUserHelper;
    @Autowired
    TemplateMapper templateMapper;
    @Autowired
    RepositoryService repositoryService;
    @Autowired
    UserRoleMapper userRoleMapper;
    @Autowired
    RoleMapper roleMapper;
    @Autowired
    UserMapper userMapper;

    @Transactional
    public void startInstanceAndAssignee(ApprovalDto dto) {
       //repositoryService.createDeployment().addClasspathResource("bpmn/approval1.bpmn").deploy();
        PeriodApprove pa = new PeriodApprove();
        startInstance(dto, pa);
        startAssignee(pa);
        //createAttache(pa);
    }


    public List<ApprovalTaskInfo> getTask(Integer year,Integer month) {
        return getApprovalTaskInfos(periodApproveMapper.queryApprovalList(year, month));
    }

    private void startInstance(ApprovalDto dto, PeriodApprove pa) {
        List<PeriodReport> currentReports = reportMapper.selectOrderReportIds(dto.getProjectId(),dto.getPeriod());
        MyAsserts.assertNotEmpty(currentReports, Exceptions.NOT_FOUND_REPORT_EXCEPTION);
        pa.setId(UUID.randomUUID().toString());
        pa.setPeriod(dto.getPeriod());
        pa.setYear(dto.getYear());

        StringBuilder reportIds = new StringBuilder("");
        StringBuilder reportTemplateIds = new StringBuilder("");
        currentReports.forEach(m -> {
            reportIds.append(m.getId()).append(",");
            reportTemplateIds.append(m.getTemplateId()).append(",");
        });

        PeriodApproveExample pae = new PeriodApproveExample();
        PeriodApproveExample.Criteria criteria = pae.createCriteria().andProjectIdEqualTo(dto.getProjectId()).andPeriodEqualTo(dto.getPeriod())
                .andYearEqualTo(dto.getYear()).andReportIdsEqualTo(reportIds.toString())
                .andTemplateIdsEqualTo(reportTemplateIds.toString()).andStatusEqualTo(Constant.APPROVAL_COMMITTED);

        List<PeriodApprove> pas = periodApproveMapper.selectByExample(pae);
        MyAsserts.assertEmpty(pas, Exceptions.REPORT_HAS_COMMIT_EXCEPTION);

        ProcessInstance pi = runtimeService.startProcessInstanceByKey("approvalProcess");

        dto.setInstanceId(pi.getId());
        pa.setInstanceId(pi.getId());
        pa.setReportIds(reportIds.toString());
        pa.setTemplateIds(reportTemplateIds.toString());
        pa.setStatus(Constant.APPROVAL_COMMITTED);
        pa.setProjectId(dto.getProjectId());
        pa.setCreateBy(authUserHelper.getCurrentUserId() == null ? "Admin" : authUserHelper.getCurrentUserId());
        pa.setCreateTime(new Date());
        periodApproveMapper.insert(pa);
    }

    private void startAssignee(PeriodApprove pa) {
        List<Task> tasks = taskService.createTaskQuery().taskName(Constant.ROLE_ACCOUNTANT).processInstanceId(
                pa.getInstanceId()).list();
        if (tasks != null && tasks.size() == 1) {
            Task task = tasks.get(0);
            Map<String, Object> map = new HashMap<>();
            map.put("committed", 0);
            //设置当前操作人Id
            taskService.setAssignee(task.getId(),authUserHelper.getCurrentUserId());
            taskService.complete(task.getId(), map);
        } else {
            logger.warn("task must not null or size eq 1");
        }
    }

    private void createAttache(PeriodApprove pa) {
        List<Task> tasks = taskService.createTaskQuery().taskAssignee(Constant.ASSIGNEE_COMPLIANCE_CHIEF).processInstanceId(
                pa.getInstanceId()).list();
        if (tasks != null && tasks.size() == 1) {
            Task task = tasks.get(0);
            taskService.createAttachment("java.lang.String", task.getId(), task.getProcessInstanceId(),
                    "period_approval_uuid", pa.getId(), "用户id");
        } else {
            logger.warn("task must not null or size eq 1");
        }
    }

    /**
     * 合规负责人审批
     * @param projectId
     * @param period
     * @param decide
     * @param comment
     */
    @Transactional
    public void checkTask1(String projectId,Integer period, String decide,String comment) {
        PeriodApprove approve= periodApproveMapper.getInstanceIdByProjectIdAndPeriod(projectId,period);
        MyAsserts.assertTrue(approve!=null&&approve.getStatus().equals("committed"),Exceptions.NOT_FOUND_INSTANCE_EXCEPTION);
        String instanceId = approve.getInstanceId();
        Map<String, Object> map = new HashMap<>();
        PeriodApprove pa = new PeriodApprove();
        switch (decide) {
            case Constant.APPROVAL_AGREED:
                pa.setStatus(Constant.APPROVAL_AGREED);
                map.put("decide", 1);
                break;
            case Constant.APPROVAL_DISAGREED:
                pa.setStatus(Constant.APPROVAL_DISAGREED);
                map.put("decide", 0);
                break;
            default:
                throw new BadParameterException("not support decide param type");
        }
        List<Task> tasks = taskService.createTaskQuery().taskName(Constant.ROLE_COMPLIANCE_CHIEF).processInstanceId(
                instanceId).list();

        if (tasks != null && tasks.size() == 1) {
            Task task = tasks.get(0);
            List<Attachment> attachments = taskService.getTaskAttachments(task.getId());

            if (attachments != null && attachments.size() == 1) {
                String uuid = attachments.get(0).getDescription();
                taskService.complete(task.getId(), map);
                pa.setId(uuid);
                //pa.setApprovalBy(authUserHelper.getCurrentUserId() == null ? "Admin" : authUserHelper.getCurrentUserId());
                pa.setApprovalTime(new Date());
                pa.setApprovalResualt(comment);
                periodApproveMapper.updateByPrimaryKeySelective(pa);
            } else {
                logger.warn("task must not null or size gt 1");
            }
        } else {
            logger.warn("task must not null or size eq 1");

        }
    }

    /**
     * 税务BP和合规IA审批
     * @param projectId
     * @param period
     * @param decide
     * @param comment
     */
    @Transactional
    public void checkTask(String projectId,Integer period, String decide,String comment,String  roleNull) {
        // 当前用户角色
         List<String> roleNames= getCurrentUserRoles();
        String role="";
        if(roleNames.contains(Constant.ROLE_COMPLIANCE_CHIEF)){
            role=Constant.ROLE_COMPLIANCE_CHIEF;
        }
        if(roleNames.contains(Constant.ROLE_TAX_BP)){
            role=Constant.ROLE_TAX_BP;
        }
        if(roleNames.contains(Constant.ROLE_COMPLIANCE_IA)){
            role=Constant.ROLE_COMPLIANCE_IA;
        }
        if(Strings.isEmpty(role)){
            throw new BadParameterException("Sorry ,you are not authorized to approve the current node");
        }
        PeriodApprove approve= periodApproveMapper.getInstanceIdByProjectIdAndPeriod(projectId,period);
        MyAsserts.assertTrue(approve!=null&&approve.getStatus().equals("committed"),Exceptions.NOT_FOUND_INSTANCE_EXCEPTION);
        String instanceId = approve.getInstanceId();
        Map<String, Object> map = new HashMap<>();
        PeriodApprove pa = new PeriodApprove();
        pa.setId(approve.getId());
        switch (decide) {
            case Constant.APPROVAL_AGREED:
                pa.setStatus(Constant.APPROVAL_AGREED);
                map.put("decide", 1);
                break;
            case Constant.APPROVAL_DISAGREED:
                pa.setStatus(Constant.APPROVAL_DISAGREED);
                map.put("decide", 0);
                break;
            default:
                throw new BadParameterException("not support decide param type");
        }

        List<Task> tasks = taskService.createTaskQuery().taskName(role).processInstanceId(
                instanceId).list();
        if (tasks != null && tasks.size() == 1) {
            Map<String, Object> p = new HashMap<>();
            p.put("description","1111");
            Task task = tasks.get(0);
            //设置操作人id及操作原因
            taskService.setAssignee(task.getId(),authUserHelper.getCurrentUserId()+"<>"+comment);
            taskService.complete(task.getId(), map);
            //只要有驳回都算驳回
            if(Constant.APPROVAL_DISAGREED.equals(decide)){
                pa.setStatus(Constant.APPROVAL_DISAGREED);
            }
            pa.setApprovalBy(authUserHelper.getCurrentUserId() == null ? "Admin" : authUserHelper.getCurrentUserId());
            pa.setApprovalTime(new Date());
            pa.setApprovalResualt(comment);
            periodApproveMapper.updateByPrimaryKeySelective(pa);
        } else {
            logger.warn("task must not null or size eq 1");
            throw new ApplicationException("can't find the status approval task or your role is wrong ");
        }
        //税务BP或者合规IA驳回则返回到上一节点，且删除对方节点
        if(Arrays.asList(Constant.APPROVAL_ROLE2).contains(role)&&Constant.APPROVAL_DISAGREED.equals(decide)){
            List<Task> tasks1=new ArrayList<>();
            if(Constant.ROLE_COMPLIANCE_IA.equals(role)){
                tasks1 = taskService.createTaskQuery().taskName(Constant.ROLE_TAX_BP).processInstanceId(
                        instanceId).list();
                if (tasks1 != null && tasks1.size() == 1) {
                    repositoryService.deleteDeployment(tasks1.get(0).getId());
                }
            }else{
                tasks1 = taskService.createTaskQuery().taskName(Constant.ROLE_COMPLIANCE_IA).processInstanceId(
                        instanceId).list();
                if (tasks1 != null && tasks1.size() == 1) {
                    repositoryService.deleteDeployment(tasks1.get(0).getId());
                }
            }
        }
        currentNode(approve);
    }

    private void currentNode(PeriodApprove pa){
        //历史流程
        List<HistoricActivityInstance> histories=historyService.createHistoricActivityInstanceQuery().orderByHistoricActivityInstanceStartTime().desc()
                .processInstanceId(pa.getInstanceId())
                .list();
        logger.debug("show current node history");
        histories.forEach(a->{
            List<String> assignee=splitAssignee(a.getAssignee());
                     logger.debug("任务ID: "+a.getId()+" 任务名称: "+a.getActivityName()+" 任务创建时间: "+a.getStartTime()
                            +" 任务办理人: "+assignee.get(0)+" 原因： "+assignee.get(1)+" 流程实例ID: "+a.getProcessInstanceId()
                            +" 执行对象ID: "+a.getExecutionId()+" 流程定义ID: "+a.getProcessDefinitionId());
                });
        //当前流程状态
        logger.debug("show current node status");
        List<Task> tasks = taskService.createTaskQuery().processInstanceId(
                pa.getInstanceId()).list();
        tasks.forEach(a->{
            List<String> assignee=splitAssignee(a.getAssignee());
            logger.debug("任务ID: "+a.getId()+" 任务创建时间: "+a.getCreateTime()+" 任务当前角色: "+a.getName()
                    +" 任务待办理人: "+assignee.get(0)+" 原因： "+assignee.get(1)+" 流程实例ID: "+a.getProcessInstanceId()
                    +" 执行对象ID: "+a.getExecutionId()+" 流程定义ID: "+a.getProcessDefinitionId());
        });
        //更新当前流程状态,此流程已完成
        if(histories.size()>0){
            if(histories.get(0).getActivityName().equals("EndEvent")&&tasks.size()==0){
                pa.setStatus(Constant.APPROVAL_AGREED);
                periodApproveMapper.updateByPrimaryKeySelective(pa);
            }
        }
    }
    private List<ApprovalTaskInfo> getApprovalTaskInfos(List<ApprovalTaskInfo> approvalTaskInfos){
        for (ApprovalTaskInfo approvalTaskInfo : approvalTaskInfos) {
            List<ApprovalTaskNodeInfo> approvalTaskNodeInfos=getApprovalTaskNode(approvalTaskInfo.getInstanceId());
            approvalTaskInfo.setItems(approvalTaskNodeInfos);
        }
        return approvalTaskInfos;
    }

    private List<ApprovalTaskNodeInfo> getApprovalTaskNode(String instanceId){
        //历史流程
        List<HistoricActivityInstance> histories=historyService.createHistoricActivityInstanceQuery().orderByHistoricActivityInstanceStartTime().desc()
                .processInstanceId(instanceId)
                .list();
        logger.debug("show current node history");
        histories.forEach(a->{
            List<String> assignee=splitAssignee(a.getAssignee());
            logger.debug("任务ID: "+a.getId()+" 任务名称: "+a.getActivityName()+" 任务创建时间: "+a.getStartTime()
                    +" 任务办理人: "+assignee.get(0)+" 原因： "+assignee.get(1)+" 流程实例ID: "+a.getProcessInstanceId()
                    +" 执行对象ID: "+a.getExecutionId()+" 流程定义ID: "+a.getProcessDefinitionId());
        });
        List<ApprovalTaskNodeInfo> approvalTaskNodeInfos=new ArrayList<>();
        List<HistoricActivityInstance> histories1= histories.stream().filter(a->Arrays.asList(Constant.APPROVAL_ROLEAll).contains(a.getActivityName())).collect(Collectors.toList());
        for(HistoricActivityInstance item:histories1){
            ApprovalTaskNodeInfo approvalTaskNodeInfo=new ApprovalTaskNodeInfo();
            approvalTaskNodeInfo.setStartTime(item.getStartTime());
            approvalTaskNodeInfo.setRole(item.getActivityName());
            List<String> assignee=splitAssignee(item.getAssignee());
            approvalTaskNodeInfo.setUserId(assignee.get(0));
            approvalTaskNodeInfo.setComment(assignee.get(1));
            approvalTaskNodeInfo.setUserName(getUserName(assignee.get(0)));
            approvalTaskNodeInfos.add(approvalTaskNodeInfo);
        }
        return  approvalTaskNodeInfos;
    }
    private String getUserName(String id){
        User user=userMapper.selectByPrimaryKey(id);
        return  user==null?"":user.getUserName();
    }

    public Template getTemplateInfo(Long templateId) {
       return templateMapper.selectByPrimaryKey(templateId);
    }

    public String getApprovalStatus(String projectId, Integer period) {
        return periodApproveMapper.getStatusByProjectIdAndPeriod(projectId,period);
    }

    //分割操作人及原因
    private List<String> splitAssignee(String assignee){
        if(assignee==null){
            return Arrays.asList("","");
        }
        List<String> bb=Arrays.asList(assignee.split("<>"));
        List<String> aa=new ArrayList<>(bb);
        if(aa.size()==0){
            aa.add("");
            aa.add("");
        }
        if(aa.size()==1){
            aa.add("");
        }
        return  aa;
    }

    /**
     * 根据projectid获取当前用户在该机构的角色
     * @return
     */
    private List<String> getCurrentUserRoles(){
        UserRoleExample userRoleExample=new UserRoleExample();
        userRoleExample.createCriteria().andUserIdEqualTo(authUserHelper.getCurrentUserId());
        List<UserRole> userRols=userRoleMapper.selectByExample(userRoleExample);
        List<String> roleNames=new ArrayList<>();
        if(userRols.size()>0){
            List<String> roleIds=userRols.stream().map(UserRole::getRoleId).collect(Collectors.toList()) ;
            RoleExample roleExample=new RoleExample();
            roleExample.createCriteria().andIdIn(roleIds);
            List<Role> roles=roleMapper.selectByExample(roleExample);
            if (roles.size() > 0) {
                roleNames=roles.stream().map(Role::getName).collect(Collectors.toList());
            }
        }
        return  roleNames;
    }

}
