package pwc.taxtech.atms.vat.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;
import pwc.taxtech.atms.MyVatMapper;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.dao.CitTbamMapper;
import pwc.taxtech.atms.dao.FormulaAdminMapper;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dao.ProjectServiceTypeMapper;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dto.TableRule;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.service.impl.CitCommonUtil;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.impl.report.functions.FormulaContext;

import java.math.BigDecimal;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static pwc.taxtech.atms.constant.Constant.FIRST_OR_DEFAULT;

@Component
public class FormulaAgent {
    @Autowired
    private FormulaAdminMapper adminMp;
    @Autowired
    private PeriodReportMapper periodReportMapper;
    @Autowired
    private CellDataMapper cellDataMapper;
    @Autowired
    private PeriodCellDataMapper periodCellDataMapper;
    @Autowired
    private ProjectServiceTypeMapper projectServiceTypeMapper;
    @Autowired
    private PeriodDataSourceMapper periodDataSourceMapper;
    @Autowired
    private CitTbamMapper citTbamMapper;
    @Autowired
    private ProjectMapper projectMapper;

    public List<CellTemplatePerGroupDto> getCellTemplateGroupDto(Long templateGroupId, String projectId, String code, Integer rowIndex, Integer columnIndex, Integer period) {
        return adminMp.getCellTemplatePerGroupDto(templateGroupId, code, rowIndex, columnIndex, projectId, period);
    }

    /**
     * 根据列index及row名称查询项目报表模板
     *
     * @param templateGroupId
     * @param projectId
     * @param code
     * @param rowName
     * @param columnIndex
     * @param period
     * @return
     */
    public List<CellTemplatePerGroupDto> getCellTemplateByNameAndIndex(Long templateGroupId, String projectId, String code, Integer rowColumnIndex, String rowName, Integer columnIndex, Integer period) {
        return adminMp.getCellTemplateByNameAndIndex(templateGroupId, code, rowColumnIndex, rowName, columnIndex, projectId, period);
    }

    /**
     * 根据收入类型配置及列index查询报表模板
     *
     * @param templateGroupId
     * @param code
     * @param rowColumnIndex
     * @param taxRate
     * @param revenueType
     * @param taxType
     * @param columnIndex
     * @param projectId
     * @param period
     * @param orgId
     * @param startDate
     * @param endDate
     * @return
     */
    public List<CellTemplatePerGroupDto> getCellTemplateByTypeAndIndex(Long templateGroupId,
                                                                       String projectId,
                                                                       String code,
                                                                       Integer rowColumnIndex,
                                                                       BigDecimal taxRate,
                                                                       Integer revenueType,
                                                                       Integer taxType,
                                                                       Integer columnIndex,
                                                                       Integer period,
                                                                       String orgId,
                                                                       String startDate,
                                                                       String endDate) {
        return adminMp.getCellTemplateByTypeAndIndex(templateGroupId, projectId, code, rowColumnIndex, taxRate, revenueType, taxType, columnIndex, period, orgId, startDate, endDate);
    }

    public Project getFixedProject(String projectId, Integer year) {
        return adminMp.getFixedProject(projectId, year);
    }

    public List<PeriodDataSource> queryManualDataSource(Long templateId, String projectId, Integer period) {
        PeriodDataSourceExample example = new PeriodDataSourceExample();
        example.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period)
                .andCellTemplateIdEqualTo(templateId).andTypeEqualTo(10);
        return periodDataSourceMapper.selectByExample(example);

    }

    /**
     * 查询所有的数据源，来用作判断当前单元格是只有手工数据源还是还有别的数据源 create by zhikai.z.wei at 20190411 20:44
     * @param templateId
     * @param projectId
     * @param period
     * @return
     */
    public List<PeriodDataSource> queryNoManualDataSource(Long templateId, String projectId, Integer period) {
        PeriodDataSourceExample example = new PeriodDataSourceExample();
        example.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period)
                .andCellTemplateIdEqualTo(templateId);
        return periodDataSourceMapper.selectByExample(example);

    }

    private PeriodReport getReportByTemplate(String templateId, Integer periodId, String projectId) {
        PeriodReportExample example = new PeriodReportExample();
        example.createCriteria().andProjectIdEqualTo(projectId).andTemplateIdEqualTo(Long.valueOf(templateId))
                .andPeriodEqualTo(periodId);
        List<PeriodReport> reports = periodReportMapper.selectByExample(example);

        if (reports != null && !reports.isEmpty()) return reports.get(FIRST_OR_DEFAULT);
        return null;
    }

    private PeriodCellData getCellDataListByTemplate(String templateId, Long reportId) {
        PeriodCellDataExample dataExample = new PeriodCellDataExample();
        dataExample.createCriteria().andCellTemplateIdEqualTo(Long.valueOf(templateId)).andReportIdEqualTo(reportId);
        List<PeriodCellData> cellDataList = periodCellDataMapper.selectByExample(dataExample);

        if (cellDataList != null && !cellDataList.isEmpty()) return cellDataList.get(FIRST_OR_DEFAULT);
        return null;
    }

    public String getPastProjectId(int year, String orgId) {
        return adminMp.getPastProjectId(year, orgId);
    }

    public PeriodCellData getCellData(String templateId, String cellId, int periodId, String projectId) {
        PeriodReport report = getReportByTemplate(templateId, periodId, projectId);
        MyAsserts.assertNotNull(report, Exceptions.BB_REPORT_NULL);
        PeriodCellData cellData = getCellDataListByTemplate(cellId, report.getId());
        MyAsserts.assertNotNull(cellData, Exceptions.BB_CELL_DATA_NULL);
        return cellData;
    }

    public List<CellTemplatePerGroupDto> getCellDataByPos(Long templateId, Integer period, String cellRow, String cellCol, String projectId) {
        return adminMp.getCellDataByPos(templateId, period, cellRow, cellCol, projectId);
    }

    public List<CellTemplatePerGroupDto> getTableData(String tableName, String getField, String selectFilter, String year, String selectPeriod) {
        return adminMp.getTableData(tableName, getField, selectFilter, year, selectPeriod);
    }


    public static String buildSql(String tableName, String getField, String[] filters, String[] filterValues, Integer period, Integer year, FormulaContext formulaContext) {
/*

        if (split[0].indexOf(">=") != -1) {
            filterValue = " > " + split[1] + "  or  " + filter + " = " + split[1];
        } else if (split[0].indexOf("<=") != -1) {
            filterValue = " < " + split[1] + "  or  " + filter + " = " + split[1];
        } else {
            filterValue = split[0] + " = " + split[1] + "";
        }
*/

        if (tableName.substring(0, 3).equals("cit")) {
            try {
                return _buildSql(getField, tableName, filters, filterValues, period, formulaContext, year, true, false);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        try {
            return _buildSql(getField, tableName, filters, filterValues, period, formulaContext, year, false, false);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    public static String _buildSql(String getField, String tableName, String[] filters, String[] filterValues, Integer period, FormulaContext formulaContext, Integer year, boolean bool, boolean getSql) throws Exception {
        StringBuilder sql = new StringBuilder();
        //统一判断年度
        year = formulaContext.getYear() + year;
        if (getSql) {
            tableName = tableName.toLowerCase();
            sql.append("select  *  from ").append(tableName).append(" where 1=1  and ");
            for (int i = 0; i < filters.length; i++){
                sql.append(filters[i]);
                sql.append(filterValues[i]);
                if(i < filters.length-1){
                    sql.append(" and ");
                }
            }
            //注意：如果此处不是return，请另新建一个变量再返回，因StringBuilder操作的是同一个对象
            sql.append(" and project_id = '" ).append(formulaContext.getProjectId()).append("'");
            return sql.toString();
        } else {
            sql.append("select  sum(" + getField + ")  as ").append(getField).append(" from " ).append(tableName).
                    append(" where 1=1  and ");
            for (int i = 0; i < filters.length; i++){
                sql.append(filters[i]);
                sql.append(filterValues[i]);
                if(i < filters.length-1){
                    sql.append(" and ");
                }
            }
//            sql = "select  sum(" + getField + ")  as " + getField + " from " + tableName + " where 1=1  and " + filter + filterValue;
        }
        String _p = insertZero(formulaContext.getPeriod() - Math.abs(period));
        String _period = insertZero(formulaContext.getPeriod());
        String per = insertZero((formulaContext.getPeriod() - 1));
        if (!bool) {
            if (period == 0) {
                sql.append(" and tms_period  = " ).append( Integer.parseInt(year + _period));
//                sql += " and tms_period  = " + Integer.parseInt(year + _period);
            } else if (period == -99) {
                sql.append(" and tms_period  between  ").append(Integer.parseInt(year + "01")).append("  and ").append(Integer.parseInt(year + per));
//                sql += " and tms_period  between  " + Integer.parseInt(year + "01") + "  and " + Integer.parseInt(year + per);
            } else if (period == 99) {
                sql.append(" and tms_period  between  ").append(Integer.parseInt(year + "01")).append("  and ").append(Integer.parseInt(year + _period));
//                sql += " and tms_period  between " + Integer.parseInt(year + "01") + " and " + Integer.parseInt(year + _period);
            } else {
                if (period > 0){
                    throw new Exception("参数不正确");
                }
                sql.append(" and tms_period   between ").append(Integer.parseInt(year + _p)).append("  and ").append(Integer.parseInt(year + per));
//                sql += " and tms_period   between " + Integer.parseInt(year + _p) + " and " + Integer.parseInt(year + per);
            }
        }
        if (bool){
            sql.append(" and period = ").append(year);
//            sql += " and period = " + year;
        }
        sql.append(" and project_id = '" ).append(formulaContext.getProjectId()).append("'");
//        sql += " and project_id = '" + formulaContext.getProjectId() + "'";
        return sql.toString();
    }

    public static String insertZero(Integer itr) {
        String str = null;
        if (itr < 10) {
            str = "0" + itr;
        } else {
            str = "" + itr;
        }
        return str;
    }


    @Autowired
    JdbcTemplate jdbcTemplate;

    public static void main(String[] args) {
        FormulaAgent formulaAgent = new FormulaAgent();
        FormulaContext formulaContext = new FormulaContext();
        formulaContext.setPeriod(3);
        String[] filtes = new String[]{"attribute"};
        String[] filteValues = new String[]{"= '销售费用-财产损耗、盘亏及毁损损失'"};
        formulaAgent.getTableDataByName("CIT_TBAM", "ending_balance", filtes, filteValues, 0, 0, null);
    }

    public BigDecimal getTableDataByName(String tableName, String getField, String[] filters, String[] filterValues, Integer period, Integer year, FormulaContext formulaContext) {

        Map<String, Object> stringObjectMap = new HashMap<>();
        try {
            stringObjectMap = jdbcTemplate.queryForMap(buildSql(TableRule.map.get(tableName), getField, filters, filterValues, period, year, formulaContext));
            if (stringObjectMap.get(getField) != null) {
                return new BigDecimal(stringObjectMap.get(getField).toString());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return new BigDecimal(0);
        }
       /* for (int i = 0, j = stringObjectMap.size(); i < j; i++) {
            if (i == 1) {
                if(stringObjectMap.get(i).get(StringUtil.underline2Camel(getField)) == null){
                    rev = 0.00;
                }else{
                    rev = Long.parseLong(String.valueOf(stringObjectMap.get(i).get(StringUtil.underline2Camel(getField))));
                }

            } else {
                if(stringObjectMap.get(i).get(StringUtil.underline2Camel(getField)) == null){
                    rev= rev+ 0.00;
                }else{
                    rev = rev + Long.parseLong(String.valueOf(stringObjectMap.get(i).get(StringUtil.underline2Camel(getField))));
                }
            }
        }
*/
        return new BigDecimal(0);
    }

    /**
     * 获取期末余额
     *
     * @param subjectType
     * @param subjectCode
     * @param year
     * @param period
     * @param formulaContext
     * @return
     * @author zhikai.z.wei
     */
    public BigDecimal getQMYE(Integer subjectType, String subjectCode, Integer year, Integer period, FormulaContext formulaContext) {
        Project project = projectMapper.selectByPrimaryKey(formulaContext.getProjectId());
        CitTbamExample citTbamExample = new CitTbamExample();
        citTbamExample.createCriteria().andAccountCodeEqualTo(subjectCode).andPeriodEqualTo(year + project.getYear());
        List<CitTbam> citTbams = citTbamMapper.selectByExample(citTbamExample);
        if (citTbams == null || citTbams.isEmpty()) {
            return new BigDecimal(0.0);
        }
        return citTbams.get(0).getEndingBalance();
    }

    /**
     * 本年底的Q列，预提
     * @param projectId
     * @return
     */
    public BigDecimal getAdvance(String projectId){
        return adminMp.getAdvance(projectId);
    }

    /**
     * 下年初的S列，已批准标准发票金额
     * @param projectId
     * @return
     */
    public BigDecimal getApprovedStandardInvoiceAmount(String projectId){
        return adminMp.getApprovedStandardInvoiceAmount(projectId);
    }
}
