package pwc.taxtech.atms.vat.service.impl;

import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.apache.poi.ss.formula.udf.AggregatingUDFFinder;
import org.apache.poi.ss.formula.udf.DefaultUDFFinder;
import org.apache.poi.ss.formula.udf.UDFFinder;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFFormulaEvaluator;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import pwc.taxtech.atms.common.POIUtil;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.CellDataSourceType;
import pwc.taxtech.atms.dao.ProjectMapper;
import pwc.taxtech.atms.dpo.AnalysisSalesValueDto;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileIUploadParam;
import pwc.taxtech.atms.dto.didiFileUpload.DidiFileUploadDetailResult;
import pwc.taxtech.atms.dto.vatdto.CellCalcInfoDto;
import pwc.taxtech.atms.dto.vatdto.PeriodResources;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.service.impl.DidiFileUploadService;
import pwc.taxtech.atms.service.impl.DistributedIdService;
import pwc.taxtech.atms.service.impl.HttpFileService;
import pwc.taxtech.atms.vat.dao.*;
import pwc.taxtech.atms.vat.dpo.PeriodCellTemplateConfigExtendDto;
import pwc.taxtech.atms.vat.entity.*;
import pwc.taxtech.atms.vat.service.impl.report.functions.*;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static pwc.taxtech.atms.common.util.SpringContextUtil.reportMapper;
import static pwc.taxtech.atms.dto.vatdto.WrapPeriodJobDto.*;

@Component
public class ReportGeneratorImpl {
    private static final Logger logger = LoggerFactory.getLogger(ReportGeneratorImpl.class);
    @Resource
    private ProjectMapper projectMapper;
    @Autowired
    private HttpFileService httpFileService;
    @Autowired
    private FormulaAgent formulaAgent;
    @Resource
    private PeriodCellDataMapper periodCellDataMapper;
    @Resource
    private PeriodFormulaBlockMapper periodFormulaBlockMapper;
    @Resource
    private RevenueConfigMapper revenueConfigMapper;
    @Resource
    private PeriodCellTemplateConfigMapper periodCellTemplateConfigMapper;
    @Resource
    private PeriodCellTemplateMapper periodCellTemplateMapper;
    @Resource
    private PeriodTemplateMapper periodTemplateMapper;
    @Autowired
    private DistributedIdService distributedIdService;
    @Resource
    private PeriodJobMapper periodJobMapper;
    @Autowired
    private DidiFileUploadService didiFileUploadService;


    public FormulaContext initContext(PeriodResources resources, Integer period) {
        return FormulaContext.extractContextFromProject(resources.getProject()).fixedFormula(period, resources.getTemplateGroupId(),
                formulaAgent);
    }

    public PeriodResources getPeriodResources(String projectId, Integer period, List<Long> templateIds) {
        PeriodResources periodResources = new PeriodResources();
        periodResources.putAllTemplate(queryPeriodTemplates(projectId, period, templateIds));
        periodResources.putAllCellTemplate(queryPeriodCellTemplates(projectId, period, periodResources.getTemolateIds()));
        periodResources.putAllCellTemplateConfig(queryPeriodCellTemplateConfigs(projectId, period, periodResources.getTemolateIds()));
        periodResources.setProject(projectMapper.selectByPrimaryKey(projectId));
        return periodResources;
    }

    public void updateWorkbookCaclsValueToDb(String projectId, Integer period, Workbook workbook, PeriodResources resources,
                                             Boolean isMergeMunual, PeriodJob job) {
        for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
            Sheet sheet = workbook.getSheetAt(i);
            String code = sheet.getSheetName();
            logger.info("-------------------------------------Begin Job [{}]------------------------------------------------------", code);
            setStatus(job, STATUS_END);
            setStatus(job, code, STATUS_BEGIN);
            job.setCurrentStep(code);
            periodJobMapper.updateByPrimaryKey(job);

            Optional<PeriodTemplate> periodTemplate = resources.getPeriodTemplates().stream()
                    .filter(a -> a.getCode().equals(code))
                    .findFirst();
            Long templateId;
            if (periodTemplate.isPresent()) {
                templateId = periodTemplate.get().getTemplateId();
            } else {
                templateId = 0L;
            }

            if (templateId > 0) {
                PeriodReport report = new PeriodReport();
                Long reportId = distributedIdService.nextId();
                report.setId(reportId);
                report.setTemplateId(templateId);
                report.setPeriod(period);
                report.setProjectId(projectId);
                report.setCreateBy("Admin");
                report.setCreateTime(new Date());
                report.setUpdateBy("Admin");
                report.setUpdateTime(new Date());
                report.setProjectId(projectId);
                reportMapper.insertSelective(report);

                List<PeriodCellTemplateConfig> periodCellTemplateConfigs = resources.getPeriodCellTemplateConfigs().stream()
                        .filter(a -> a.getReportTemplateId().equals(templateId) && a.getDataSourceType().equals(CellDataSourceType.Formula.getCode()))
                        .collect(Collectors.toList());

                //update formulablock table reportid field
                List<Long> cellTemplateConfigIds = periodCellTemplateConfigs.stream()
                        .map(PeriodCellTemplateConfig::getCellTemplateId)
                        .collect(Collectors.toList());
                if (cellTemplateConfigIds.size() > 0) {
                    periodFormulaBlockMapper.updateReportId(reportId, cellTemplateConfigIds, period, projectId);
                }

                for (PeriodCellTemplateConfig periodCellTemplateConfig : periodCellTemplateConfigs) {

                    PeriodFormulaBlockExample periodFormulaBlockExample2 = new PeriodFormulaBlockExample();
                    periodFormulaBlockExample2.createCriteria()
                            .andProjectIdEqualTo(projectId)
                            .andCellTemplateIdEqualTo(periodCellTemplateConfig.getCellTemplateId())
                            .andReportIdEqualTo(reportId)
                            .andPeriodEqualTo(period);
                    List<PeriodFormulaBlock> periodFormulaBlocks = periodFormulaBlockMapper.selectByExample(periodFormulaBlockExample2);

                    //TODO：如果formula 为 ND(100) +ND(22) ,需要使用正则表达式拆分出自定义公式，然后根据自定义公式取formulablock 的数据进行替换
//                    String regex = "[A-Z]*\\([\\-A-Za-z0-9\\\"\\,\\.\\:\\u4e00-\\u9fa5\\%]*\\)";
                    String regex = "[A-Z]*\\([^(^)]*\\)";
                    Pattern p = Pattern.compile(regex);
                    String sourceFormula = StringUtils.isNotBlank(periodCellTemplateConfig.getKeyValueParsedFormula()) ?
                            periodCellTemplateConfig.getKeyValueParsedFormula() :
                            periodCellTemplateConfig.getFormula();
                    String resultFormula = StringUtils.isNotBlank(periodCellTemplateConfig.getKeyValueParsedFormula()) ?
                            periodCellTemplateConfig.getKeyValueParsedFormula() :
                            periodCellTemplateConfig.getFormula();
                    Matcher m = p.matcher(sourceFormula);
                    Boolean isFind = false;
                    while (m.find()) {
                        isFind = true;
                        //如果有些公式无法用正则匹配，可以做特殊处理
                        String findStr = m.group();
                        Optional<PeriodFormulaBlock> formulaBlock = periodFormulaBlocks.stream()
                                .filter(a -> a.getFormulaExpression().equals(findStr))
                                .findFirst();
                        if (formulaBlock.isPresent()) {
                            resultFormula = resultFormula.replace(findStr, formulaBlock.get().getData());
                        }
                    }

                    //如果有正则匹配就进行更新公式解析
                    if (isFind) {
                        periodCellTemplateConfig.setParsedFormula(StringUtils.isNotBlank(resultFormula) ? resultFormula : null);
                        periodCellTemplateConfigMapper.updateByPrimaryKeySelective(periodCellTemplateConfig);
                    }

                    Optional<PeriodCellTemplate> tempPeriodCellTemplate = resources.getPeriodCellTemplates().stream()
                            .filter(a -> a.getCellTemplateId().equals(periodCellTemplateConfig.getCellTemplateId()))
                            .findFirst();
                    if (tempPeriodCellTemplate.isPresent()) {
                        PeriodCellData cellData = new PeriodCellData();
                        Long cellDataId = distributedIdService.nextId();
                        cellData.setId(cellDataId);
                        cellData.setReportId(reportId);
                        cellData.setCellTemplateId(tempPeriodCellTemplate.get().getCellTemplateId());
                        String data;
                        if (sheet.getRow(tempPeriodCellTemplate.get().getRowIndex()) != null
                                && sheet.getRow(tempPeriodCellTemplate.get().getRowIndex())
                                .getCell(tempPeriodCellTemplate.get().getColumnIndex()) != null) {
                            Cell cell = sheet.getRow(tempPeriodCellTemplate.get().getRowIndex())
                                    .getCell(tempPeriodCellTemplate.get().getColumnIndex());

                            data = ((XSSFCell) cell).getRawValue();
                            if (data != null && data.equals("#VALUE!")) {
                                FormulaEvaluator formulaEvaluator = new XSSFFormulaEvaluator((XSSFWorkbook) workbook);
                                try {
                                    data = formulaEvaluator.evaluate(cell).getStringValue();
                                } catch (Exception e) {
                                    logger.error(e.getStackTrace().toString());
                                    data = "0.0";
                                }
                            }
                        } else {
                            data = "0.0";
                        }

                        if (StringUtils.isNotBlank(data)) {
                            Pattern pattern = Pattern.compile("[0-9.]*");
                            Matcher isNum = pattern.matcher(data);
                            if (isNum.matches()) {
                                cellData.setData(new BigDecimal(data).toString());
                            } else {
                                cellData.setData(data);
                            }
                        } else {
                            cellData.setData(data);
                        }
                        if (StringUtils.isBlank(resultFormula)) {
                            resultFormula = " ";
                        }
                        cellData.setFormulaExp(resultFormula);
                        cellData.setCreateBy("Admin");
                        cellData.setCreateTime(new Date());
                        cellData.setUpdateBy("Admin");
                        cellData.setUpdateTime(new Date());
                        cellData.setProjectId(projectId);
                        cellData.setPeriod(period);


                        //after insert celldata, insert the celldatasource for link celldata and datasource
                        PeriodDataSourceExample dataSourceExample = new PeriodDataSourceExample();
                        dataSourceExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(projectId)
                                .andCellTemplateIdEqualTo(tempPeriodCellTemplate.get().getCellTemplateId()).andTypeNotEqualTo(10);
                        List<PeriodDataSource> dataSourceList = SpringContextUtil.periodDataSourceMapper.selectByExample(dataSourceExample);

                        for (int ii = 0; ii < dataSourceList.size(); ii++) {
                            PeriodDataSource dataSource = dataSourceList.get(ii);
                            PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                            cellDataSource.setId(distributedIdService.nextId());
                            cellDataSource.setCellTemplateId(tempPeriodCellTemplate.get().getCellTemplateId());
                            cellDataSource.setCellDataId(cellDataId);
                            cellDataSource.setDataSourceId(dataSource.getId());
                            cellDataSource.setCreateTime(new Date());
                            cellDataSource.setUpdateTime(new Date());
                            cellDataSource.setPeriod(period);
                            cellDataSource.setProjectId(projectId);

                            SpringContextUtil.periodCellDataSourceMapper.insertSelective(cellDataSource);
                        }

                        periodCellDataMapper.insertSelective(cellData);
                    }
                }
                if (isMergeMunual) {
                    List<PeriodCellTemplateConfig> keyInCellTemplateConfigs = resources.getPeriodCellTemplateConfigs().stream()
                            .filter(a -> a.getReportTemplateId().equals(templateId) && a.getDataSourceType().equals(CellDataSourceType.KeyIn.getCode()))
                            .collect(Collectors.toList());
                    for (PeriodCellTemplateConfig keyInCellTemplateConfig : keyInCellTemplateConfigs) {
                        PeriodDataSourceExample dataSourceExample = new PeriodDataSourceExample();
                        dataSourceExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(projectId)
                                .andCellTemplateIdEqualTo(keyInCellTemplateConfig.getCellTemplateId()).andTypeEqualTo(10);
                        List<PeriodDataSource> dataSourceList = SpringContextUtil.periodDataSourceMapper.selectByExample(dataSourceExample);
                        if (!dataSourceList.isEmpty() && dataSourceList.size() == 1) {
                            PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
                            cellDataExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(projectId)
                                    .andCellTemplateIdEqualTo(keyInCellTemplateConfig.getCellTemplateId());
                            List<PeriodCellData> cellDataList = periodCellDataMapper.selectByExample(cellDataExample);
                            if (cellDataList.size() == 1) {
                                PeriodCellData cellData = cellDataList.get(0);
                                PeriodDataSource dataSource = dataSourceList.get(0);
                                if (StringUtils.isEmpty(cellData.getData().trim()))
                                    cellData.setData("0.0");
                                if (StringUtils.isEmpty(cellData.getFormulaExp().trim()))
                                    cellData.setFormulaExp("0.0");

                                PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                                cellDataSource.setId(distributedIdService.nextId());
                                cellDataSource.setCellTemplateId(keyInCellTemplateConfig.getCellTemplateId());
                                cellDataSource.setCellDataId(cellData.getId());
                                cellDataSource.setDataSourceId(dataSource.getId());
                                cellDataSource.setCreateTime(new Date());
                                cellDataSource.setUpdateTime(new Date());
                                cellDataSource.setPeriod(period);
                                cellDataSource.setProjectId(projectId);
                                SpringContextUtil.periodCellDataSourceMapper.insertSelective(cellDataSource);
                                periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                            } else if (cellDataList.isEmpty()) {
                                PeriodCellData cellData = new PeriodCellData();
                                Long cellDataId = distributedIdService.nextId();
                                cellData.setId(cellDataId);
                                cellData.setReportId(reportId);
                                cellData.setCellTemplateId(keyInCellTemplateConfig.getCellTemplateId());
                                cellData.setCreateBy("Admin");
                                cellData.setCreateTime(new Date());
                                cellData.setUpdateBy("Admin");
                                cellData.setUpdateTime(new Date());
                                cellData.setProjectId(projectId);
                                cellData.setPeriod(period);

                                PeriodDataSource dataSource = dataSourceList.get(0);
                                cellData.setData("0.0");
                                cellData.setFormulaExp("0.0");

                                PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                                cellDataSource.setId(distributedIdService.nextId());
                                cellDataSource.setCellTemplateId(keyInCellTemplateConfig.getCellTemplateId());
                                cellDataSource.setCellDataId(cellData.getId());
                                cellDataSource.setDataSourceId(dataSource.getId());
                                cellDataSource.setCreateTime(new Date());
                                cellDataSource.setUpdateTime(new Date());
                                cellDataSource.setPeriod(period);
                                cellDataSource.setProjectId(projectId);
                                SpringContextUtil.periodCellDataSourceMapper.insertSelective(cellDataSource);
                                periodCellDataMapper.insertSelective(cellData);
                            } else {
                                logger.warn("should not be !!!");
                            }
                        }

                    }
                }
            }
            logger.info("-------------------------------------End Job [{}]------------------------------------------------------", code);
        }
    }

    /**
     * create by zhikai.z.wei 更改正则表达式，匹配E13-D13这种公式，要不然此单元格设置为允许手工数据后页面不能计算出实际值
     * @param projectId
     * @param period
     * @param workbook
     * @param resources
     * @param isMergeMunual
     * @param job
     */
    public void citUpdateWorkbookCaclsValueToDb(String projectId, Integer period, Workbook workbook, PeriodResources resources,
                                             Boolean isMergeMunual, PeriodJob job) {
        for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
            Sheet sheet = workbook.getSheetAt(i);
            String code = sheet.getSheetName();
            logger.info("-------------------------------------Begin Job [{}]------------------------------------------------------", code);
            setStatus(job, STATUS_END);
            setStatus(job, code, STATUS_BEGIN);
            job.setCurrentStep(code);
            periodJobMapper.updateByPrimaryKey(job);

            Optional<PeriodTemplate> periodTemplate = resources.getPeriodTemplates().stream()
                    .filter(a -> a.getCode().equals(code))
                    .findFirst();
            Long templateId;
            if (periodTemplate.isPresent()) {
                templateId = periodTemplate.get().getTemplateId();
            } else {
                templateId = 0L;
            }

            if (templateId > 0) {
                PeriodReport report = new PeriodReport();
                Long reportId = distributedIdService.nextId();
                report.setId(reportId);
                report.setTemplateId(templateId);
                report.setPeriod(period);
                report.setProjectId(projectId);
                report.setCreateBy("Admin");
                report.setCreateTime(new Date());
                report.setUpdateBy("Admin");
                report.setUpdateTime(new Date());
                report.setProjectId(projectId);
                reportMapper.insertSelective(report);

                List<PeriodCellTemplateConfig> periodCellTemplateConfigs = resources.getPeriodCellTemplateConfigs().stream()
                        .filter(a -> a.getReportTemplateId().equals(templateId) && a.getDataSourceType().equals(CellDataSourceType.Formula.getCode()))
                        .collect(Collectors.toList());

                //update formulablock table reportid field
                List<Long> cellTemplateConfigIds = periodCellTemplateConfigs.stream()
                        .map(PeriodCellTemplateConfig::getCellTemplateId)
                        .collect(Collectors.toList());
                if (cellTemplateConfigIds.size() > 0) {
                    periodFormulaBlockMapper.updateReportId(reportId, cellTemplateConfigIds, period, projectId);
                }

                for (PeriodCellTemplateConfig periodCellTemplateConfig : periodCellTemplateConfigs) {

                    PeriodFormulaBlockExample periodFormulaBlockExample2 = new PeriodFormulaBlockExample();
                    periodFormulaBlockExample2.createCriteria()
                            .andProjectIdEqualTo(projectId)
                            .andCellTemplateIdEqualTo(periodCellTemplateConfig.getCellTemplateId())
                            .andReportIdEqualTo(reportId)
                            .andPeriodEqualTo(period);
                    List<PeriodFormulaBlock> periodFormulaBlocks = periodFormulaBlockMapper.selectByExample(periodFormulaBlockExample2);

                    //TODO：如果formula 为 ND(100) +ND(22) ,需要使用正则表达式拆分出自定义公式，然后根据自定义公式取formulablock 的数据进行替换
//                    String regex = "[A-Z]*\\([\\-A-Za-z0-9\\\"\\,\\.\\:\\u4e00-\\u9fa5\\%]*\\)";
                    String regex = "([A-Z]*\\([^(^)]*\\))|(([A-Z]*[0-9]*((\\+|\\-|\\*|\\/))*)*)";
                    Pattern p = Pattern.compile(regex);
                    String sourceFormula = StringUtils.isNotBlank(periodCellTemplateConfig.getKeyValueParsedFormula()) ?
                            periodCellTemplateConfig.getKeyValueParsedFormula() :
                            periodCellTemplateConfig.getFormula();
                    String resultFormula = StringUtils.isNotBlank(periodCellTemplateConfig.getKeyValueParsedFormula()) ?
                            periodCellTemplateConfig.getKeyValueParsedFormula() :
                            periodCellTemplateConfig.getFormula();
                    Matcher m = p.matcher(sourceFormula);
                    Boolean isFind = false;
                    while (m.find()) {
                        isFind = true;
                        //如果有些公式无法用正则匹配，可以做特殊处理
                        String findStr = m.group();
                        Optional<PeriodFormulaBlock> formulaBlock = periodFormulaBlocks.stream()
                                .filter(a -> a.getFormulaExpression().equals(findStr))
                                .findFirst();
                        if (formulaBlock.isPresent()) {
                            resultFormula = resultFormula.replace(findStr, formulaBlock.get().getData());
                        }
                    }

                    //如果有正则匹配就进行更新公式解析
                    if (isFind) {
                        periodCellTemplateConfig.setParsedFormula(StringUtils.isNotBlank(resultFormula) ? resultFormula : null);
                        periodCellTemplateConfigMapper.updateByPrimaryKeySelective(periodCellTemplateConfig);
                    }

                    Optional<PeriodCellTemplate> tempPeriodCellTemplate = resources.getPeriodCellTemplates().stream()
                            .filter(a -> a.getCellTemplateId().equals(periodCellTemplateConfig.getCellTemplateId()))
                            .findFirst();
                    if (tempPeriodCellTemplate.isPresent()) {
                        PeriodCellData cellData = new PeriodCellData();
                        Long cellDataId = distributedIdService.nextId();
                        cellData.setId(cellDataId);
                        cellData.setReportId(reportId);
                        cellData.setCellTemplateId(tempPeriodCellTemplate.get().getCellTemplateId());
                        String data;
                        if (sheet.getRow(tempPeriodCellTemplate.get().getRowIndex()) != null
                                && sheet.getRow(tempPeriodCellTemplate.get().getRowIndex())
                                .getCell(tempPeriodCellTemplate.get().getColumnIndex()) != null) {
                            Cell cell = sheet.getRow(tempPeriodCellTemplate.get().getRowIndex())
                                    .getCell(tempPeriodCellTemplate.get().getColumnIndex());

                            data = ((XSSFCell) cell).getRawValue();
                            if (data != null && data.equals("#VALUE!")) {
                                FormulaEvaluator formulaEvaluator = new XSSFFormulaEvaluator((XSSFWorkbook) workbook);
                                try {
                                    data = formulaEvaluator.evaluate(cell).getStringValue();
                                } catch (Exception e) {
                                    logger.error(e.getStackTrace().toString());
                                    data = "0.0";
                                }
                            }
                        } else {
                            data = "0.0";
                        }

                        if (StringUtils.isNotBlank(data)) {
                            Pattern pattern = Pattern.compile("[0-9.]*");
                            Matcher isNum = pattern.matcher(data);
                            if (isNum.matches()) {
                                cellData.setData(new BigDecimal(data).toString());
                            } else {
                                cellData.setData(data);
                            }
                        } else {
                            cellData.setData(data);
                        }
                        if (StringUtils.isBlank(resultFormula)) {
                            resultFormula = " ";
                        }
                        cellData.setFormulaExp(resultFormula);
                        cellData.setCreateBy("Admin");
                        cellData.setCreateTime(new Date());
                        cellData.setUpdateBy("Admin");
                        cellData.setUpdateTime(new Date());
                        cellData.setProjectId(projectId);
                        cellData.setPeriod(period);


                        //after insert celldata, insert the celldatasource for link celldata and datasource
                        PeriodDataSourceExample dataSourceExample = new PeriodDataSourceExample();
                        dataSourceExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(projectId)
                                .andCellTemplateIdEqualTo(tempPeriodCellTemplate.get().getCellTemplateId()).andTypeNotEqualTo(10);
                        List<PeriodDataSource> dataSourceList = SpringContextUtil.periodDataSourceMapper.selectByExample(dataSourceExample);

                        for (int ii = 0; ii < dataSourceList.size(); ii++) {
                            PeriodDataSource dataSource = dataSourceList.get(ii);
                            PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                            cellDataSource.setId(distributedIdService.nextId());
                            cellDataSource.setCellTemplateId(tempPeriodCellTemplate.get().getCellTemplateId());
                            cellDataSource.setCellDataId(cellDataId);
                            cellDataSource.setDataSourceId(dataSource.getId());
                            cellDataSource.setCreateTime(new Date());
                            cellDataSource.setUpdateTime(new Date());
                            cellDataSource.setPeriod(period);
                            cellDataSource.setProjectId(projectId);

                            SpringContextUtil.periodCellDataSourceMapper.insertSelective(cellDataSource);
                        }

                        periodCellDataMapper.insertSelective(cellData);
                    }
                }
                if (isMergeMunual) {
                    List<PeriodCellTemplateConfig> keyInCellTemplateConfigs = resources.getPeriodCellTemplateConfigs().stream()
                            .filter(a -> a.getReportTemplateId().equals(templateId) && a.getDataSourceType().equals(CellDataSourceType.KeyIn.getCode()))
                            .collect(Collectors.toList());
                    for (PeriodCellTemplateConfig keyInCellTemplateConfig : keyInCellTemplateConfigs) {
                        PeriodDataSourceExample dataSourceExample = new PeriodDataSourceExample();
                        dataSourceExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(projectId)
                                .andCellTemplateIdEqualTo(keyInCellTemplateConfig.getCellTemplateId()).andTypeEqualTo(10);
                        List<PeriodDataSource> dataSourceList = SpringContextUtil.periodDataSourceMapper.selectByExample(dataSourceExample);
                        if (!dataSourceList.isEmpty() && dataSourceList.size() == 1) {
                            PeriodCellDataExample cellDataExample = new PeriodCellDataExample();
                            cellDataExample.createCriteria().andPeriodEqualTo(period).andProjectIdEqualTo(projectId)
                                    .andCellTemplateIdEqualTo(keyInCellTemplateConfig.getCellTemplateId());
                            List<PeriodCellData> cellDataList = periodCellDataMapper.selectByExample(cellDataExample);
                            if (cellDataList.size() == 1) {
                                PeriodCellData cellData = cellDataList.get(0);
                                PeriodDataSource dataSource = dataSourceList.get(0);
                                if (StringUtils.isEmpty(cellData.getData().trim()))
                                    cellData.setData("0.0");
                                if (StringUtils.isEmpty(cellData.getFormulaExp().trim()))
                                    cellData.setFormulaExp("0.0");

                                PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                                cellDataSource.setId(distributedIdService.nextId());
                                cellDataSource.setCellTemplateId(keyInCellTemplateConfig.getCellTemplateId());
                                cellDataSource.setCellDataId(cellData.getId());
                                cellDataSource.setDataSourceId(dataSource.getId());
                                cellDataSource.setCreateTime(new Date());
                                cellDataSource.setUpdateTime(new Date());
                                cellDataSource.setPeriod(period);
                                cellDataSource.setProjectId(projectId);
                                SpringContextUtil.periodCellDataSourceMapper.insertSelective(cellDataSource);
                                periodCellDataMapper.updateByPrimaryKeySelective(cellData);
                            } else if (cellDataList.isEmpty()) {
                                PeriodCellData cellData = new PeriodCellData();
                                Long cellDataId = distributedIdService.nextId();
                                cellData.setId(cellDataId);
                                cellData.setReportId(reportId);
                                cellData.setCellTemplateId(keyInCellTemplateConfig.getCellTemplateId());
                                cellData.setCreateBy("Admin");
                                cellData.setCreateTime(new Date());
                                cellData.setUpdateBy("Admin");
                                cellData.setUpdateTime(new Date());
                                cellData.setProjectId(projectId);
                                cellData.setPeriod(period);

                                PeriodDataSource dataSource = dataSourceList.get(0);
                                cellData.setData("0.0");
                                cellData.setFormulaExp("0.0");

                                PeriodCellDataSource cellDataSource = new PeriodCellDataSource();
                                cellDataSource.setId(distributedIdService.nextId());
                                cellDataSource.setCellTemplateId(keyInCellTemplateConfig.getCellTemplateId());
                                cellDataSource.setCellDataId(cellData.getId());
                                cellDataSource.setDataSourceId(dataSource.getId());
                                cellDataSource.setCreateTime(new Date());
                                cellDataSource.setUpdateTime(new Date());
                                cellDataSource.setPeriod(period);
                                cellDataSource.setProjectId(projectId);
                                SpringContextUtil.periodCellDataSourceMapper.insertSelective(cellDataSource);
                                periodCellDataMapper.insertSelective(cellData);
                            } else {
                                logger.warn("should not be !!!");
                            }
                        }

                    }
                }
            }
            logger.info("-------------------------------------End Job [{}]------------------------------------------------------", code);
        }
    }

    private List<PeriodReport> createReportsByTemplates(Workbook
                                                                workbook, List<PeriodTemplate> periodTemplateList, String projectId, Integer period) {
        List<PeriodReport> reports = new ArrayList<>();
        for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
            Sheet sheet = workbook.getSheetAt(i);
            String code = sheet.getSheetName();
            Optional<PeriodTemplate> periodTemplate = periodTemplateList.stream()
                    .filter(a -> a.getCode().equals(code))
                    .findFirst();
            Long templateId;
            if (periodTemplate.isPresent()) {
                templateId = periodTemplate.get().getTemplateId();
            } else {
                templateId = 0L;
            }

            if (templateId > 0) {
                PeriodReport report = new PeriodReport();
                Long reportId = distributedIdService.nextId();
                report.setId(reportId);
                report.setTemplateId(templateId);
                report.setPeriod(period);
                report.setProjectId(projectId);
                report.setCreateBy("Admin");
                report.setCreateTime(new Date());
                report.setUpdateBy("Admin");
                report.setUpdateTime(new Date());
                report.setProjectId(projectId);
                reportMapper.insertSelective(report);
                reports.add(report);
            }
        }

        return reports;
    }

    public void setConfigAndDataToWorkBook(Workbook workbook, PeriodResources resources) {

        for (int i = 0; i < workbook.getNumberOfSheets(); i++) {
            Sheet sheet = workbook.getSheetAt(i);
            String code = sheet.getSheetName();
            Optional<PeriodTemplate> periodTemplate = resources.getPeriodTemplates().stream()
                    .filter(a -> a.getCode().equals(code))
                    .findFirst();
            Long templateId;
            if (periodTemplate.isPresent()) {
                templateId = periodTemplate.get().getTemplateId();
            } else {
                templateId = 0L;
            }

            if (templateId > 0) {
                //get cell template and cell template config with templateId
                Map<PeriodCellTemplate, PeriodCellTemplateConfig> cellTemplatePeriodCellTemplateConfigMap = new HashMap<>();
                List<PeriodCellTemplate> tempPeriodCellTemplateList = resources.getPeriodCellTemplates().stream()
                        .filter(a -> a.getReportTemplateId().equals(templateId))
                        .collect(Collectors.toList());

                tempPeriodCellTemplateList.forEach(a -> {
                    Optional<PeriodCellTemplateConfig> tempCellTemplateConfig = resources.getPeriodCellTemplateConfigs().stream()
                            .filter(item -> item.getCellTemplateId().equals(a.getCellTemplateId())
                                    && item.getDataSourceType().equals(CellDataSourceType.Formula.getCode()))
                            .findFirst();
                    tempCellTemplateConfig.ifPresent(periodCellTemplateConfig ->
                            cellTemplatePeriodCellTemplateConfigMap.put(a, periodCellTemplateConfig));
                });

                cellTemplatePeriodCellTemplateConfigMap.forEach((k, v) -> {
                    Row row = sheet.getRow(k.getRowIndex());
                    if (null == row) {
                        sheet.createRow(k.getRowIndex());
                    }
                    row = sheet.getRow(k.getRowIndex());

                    Cell cell = row.getCell(k.getColumnIndex());
                    if (null == cell) {
                        row.createCell(k.getColumnIndex());
                    }
                    cell = row.getCell(k.getColumnIndex());

                    //todo:后面单独处理kv的公式
                    if (StringUtils.isNotBlank(v.getFormula()) && !v.getFormula().contains("@")) {
                        cell.setCellFormula(v.getFormula());
                        logger.debug("formula:" + v.getFormula());
                        //kv 公式处理
                    } else if (v.getFormula().contains("@")) {
                        if (StringUtils.isNotBlank(v.getKeyValueParsedFormula())) {
                            cell.setCellFormula(v.getKeyValueParsedFormula());
                            logger.debug("formula:" + v.getKeyValueParsedFormula());
                        }
                    }
                });
            }
        }
    }

    private List<PeriodTemplate> queryPeriodTemplates(String projectId, Integer period, List<Long> templateIds) {
        PeriodTemplateExample periodTemplateExample = new PeriodTemplateExample();
        periodTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andPeriodEqualTo(period).andTemplateIdIn(templateIds);
        return periodTemplateMapper.selectByExample(periodTemplateExample);
    }

    private List<PeriodCellTemplate> queryPeriodCellTemplates(String projectId, Integer
            period, List<Long> periodTemplateIdList) {
        PeriodCellTemplateExample periodCellTemplateExample = new PeriodCellTemplateExample();
        if (periodTemplateIdList.isEmpty()) periodTemplateIdList.add(Long.MAX_VALUE);
        periodCellTemplateExample.createCriteria().andProjectIdEqualTo(projectId).andReportTemplateIdIn(periodTemplateIdList).andPeriodEqualTo(period);
        return periodCellTemplateMapper.selectByExample(periodCellTemplateExample);
    }

    private List<PeriodCellTemplateConfig> queryPeriodCellTemplateConfigs(String projectId, Integer
            period, List<Long> periodTemplateIdList) {
        PeriodCellTemplateConfigExample periodCellTemplateConfigExample = new PeriodCellTemplateConfigExample();
        periodCellTemplateConfigExample.createCriteria().andProjectIdEqualTo(projectId).andReportTemplateIdIn(periodTemplateIdList).andPeriodEqualTo(period);
        return periodCellTemplateConfigMapper.selectByExample(periodCellTemplateConfigExample);
    }

    private String getTemplatePath(String path) {
        String result;
        String PREFIX_VALUE = "~";
        if (path.startsWith(PREFIX_VALUE)) {
            result = path.substring(1, path.length());
        } else {
            result = path;
        }
        return result;
    }

    /**
     * 根据传入的模板code和模板路径创建workbook
     *
     * @param templates 模板code和模板路径 键值对
     * @return 工作簿workbook
     */
    public Workbook createWorkBookByPeriodTemplate(List<PeriodTemplate> templates, PeriodJob periodJob) {
        Workbook workbook = new XSSFWorkbook();
        try {
            String filePath = this.getClass().getResource("").toURI().getPath();
            String tempPath = filePath.substring(0, filePath.indexOf("classes") + "\\classes".length());
            templates.forEach(a -> {
                Workbook tWorkbook = null;
                File file = null;
                if (a.getIsSystemType()) {

                    file = new File(tempPath + a.getPath());
                    try {
                        tWorkbook = WorkbookFactory.create(file);
                    } catch (IOException e) {
                        e.printStackTrace();
                    } catch (InvalidFormatException e) {
                        e.printStackTrace();
                    }
                } else {
                    InputStream is = null;
                    try {
                        String path = "";
                        if (a.getPath().indexOf("/") > 0) {
                            path = a.getPath();
                        } else {
                            DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
                            fileParam.setUuids(Arrays.asList(a.getPath()));
                            PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
                            Map<String, String> urlMap = null;
                            if (CollectionUtils.isNotEmpty(uploadDetail.getList())) {
                                path = uploadDetail.getList().get(0).getViewHttpUrl();
                            }
                        }
                        is = httpFileService.getUserTemplate(path);
                        tWorkbook = WorkbookFactory.create(is);
                    } catch (Exception e) {
                        e.printStackTrace();
                    } finally {
                        if (is != null) {
                            try {
                                is.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                }

                POIUtil.cloneSheet(tWorkbook.getSheetAt(0), workbook.createSheet(a.getCode()));

            });
            return workbook;
        } catch (Exception e) {
            logger.error("createWorkBookByPeriodTemplate error.", e);
            setStatus(periodJob, STATUS_ERROR);
            periodJob.setErrorMsg("error for gen excel , redo it later!!1");
            periodJobMapper.updateByPrimaryKey(periodJob);
            throw Exceptions.SERVER_ERROR_EXCEPTION;
        }
    }
    public Workbook createWorkBookByPeriodTemplate(List<PeriodTemplate> templates) {
        {
            Workbook workbook = new XSSFWorkbook();
            try {
                String filePath = this.getClass().getResource("").toURI().getPath();
                String tempPath = filePath.substring(0, filePath.indexOf("classes") + "\\classes".length());
                templates.forEach(a -> {
                    Workbook tWorkbook = null;
                    File file = null;
                    if (a.getIsSystemType()) {

                        file = new File(tempPath + a.getPath());
                        try {
                            tWorkbook = WorkbookFactory.create(file);
                        } catch (IOException e) {
                            e.printStackTrace();
                        } catch (InvalidFormatException e) {
                            e.printStackTrace();
                        }
                    } else {
                        InputStream is = null;
                        try {
                            String path = "";
                            if (a.getPath().indexOf("/") > 0) {
                                path = a.getPath();
                            } else {
                                DidiFileIUploadParam fileParam = new DidiFileIUploadParam();
                                fileParam.setUuids(Arrays.asList(a.getPath()));
                                PageInfo<DidiFileUploadDetailResult> uploadDetail = didiFileUploadService.queryPage(fileParam);
                                Map<String, String> urlMap = null;
                                if (CollectionUtils.isNotEmpty(uploadDetail.getList())) {
                                    path = uploadDetail.getList().get(0).getViewHttpUrl();
                                }
                            }
                            is = httpFileService.getUserTemplate(path);
                            tWorkbook = WorkbookFactory.create(is);
                        } catch (Exception e) {
                            e.printStackTrace();
                        } finally {
                            if (is != null) {
                                try {
                                    is.close();
                                } catch (IOException e) {
                                    e.printStackTrace();
                                }
                            }
                        }
                    }
                    //处理workbook
                    List<AnalysisSalesValueDto> dataList = periodCellDataMapper.selectReportData(a.getTemplateId(),a.getProjectId(),a.getPeriod());
                    for(AnalysisSalesValueDto cellData:dataList){
                        Sheet sheet = tWorkbook.getSheetAt(0);
                        sheet.getRow(cellData.getRowIndex()).getCell(cellData.getColumnIndex()).setCellValue(cellData.getData());
                    }
                    POIUtil.cloneSheetAndStyle(tWorkbook.getSheetAt(0), workbook.createSheet(a.getCode()),workbook);

                });
                return workbook;
            } catch (Exception e) {
                logger.error("createWorkBookByPeriodTemplate error.", e);
                throw Exceptions.SERVER_ERROR_EXCEPTION;
            }
        }
    }

    /**
     * 注册所有的自定义方法到工作簿
     *
     * @param workbook 工作簿
     */
    public void addFunctionsAndContext(Workbook workbook, String[] functions, FormulaContext formulaContext, Boolean isValidate) {
        FreeRefFunction[] functionImpls;
        if (!isValidate) {
            functionImpls = new FreeRefFunction[]{new SGSR(formulaContext), new FSJZ(formulaContext), new ND(formulaContext),
                    new BB(formulaContext), new XXFP(formulaContext), new GZSD(formulaContext), new PC(formulaContext)
                    , new JXFPMX(formulaContext), new JXFP(formulaContext), new PSUM(formulaContext), new DFFS(formulaContext),
                    new JFFS(formulaContext), new WPSR(formulaContext), new WPNAME(formulaContext), new WPTYPE(formulaContext),
                    new SUM2(formulaContext), new RSUMIF(formulaContext), new SUM(formulaContext), new KPSR(formulaContext),new TBM(formulaContext)};
        } else {
            functionImpls = new FreeRefFunction[]{new SGSR(formulaContext), new FSJZ(formulaContext), new ND(formulaContext),
                    new pwc.taxtech.atms.vat.service.impl.report.functions.validation.BB(formulaContext), new XXFP(formulaContext), new GZSD(formulaContext), new PC(formulaContext)
                    , new JXFPMX(formulaContext), new JXFP(formulaContext), new PSUM(formulaContext), new DFFS(formulaContext),
                    new JFFS(formulaContext), new WPSR(formulaContext), new WPNAME(formulaContext), new WPTYPE(formulaContext),
                    new SUM2(formulaContext), new RSUMIF(formulaContext), new SUM(formulaContext), new KPSR(formulaContext),new TBM(formulaContext)};
        }
        UDFFinder udfs = new DefaultUDFFinder(functions, functionImpls);
        UDFFinder udfToolpack = new AggregatingUDFFinder(udfs);
        workbook.addToolPack(udfToolpack);
    }

    /**
     * 为CIT复制的一份注册自定义公式，和上面的addFunctionsAndContext方法基本相同
     * CIT 注册所有的自定义方法到工作簿
     *
     * @param workbook 工作簿
     */
    public void addCitFunctionsAndContext(Workbook workbook, String[] functions, FormulaContext formulaContext, Boolean isValidate) {
        //**************************** 为CIT复制的一份注册自定义公式 ****************************
        FreeRefFunction[] functionImpls;
        if (!isValidate) {
            functionImpls = new FreeRefFunction[]{new SGSR(formulaContext), new FSJZ(formulaContext), new ND(formulaContext),
                    new BB(formulaContext), new XXFP(formulaContext), new GZSD(formulaContext), new PC(formulaContext)
                    , new JXFPMX(formulaContext), new JXFP(formulaContext), new PSUM(formulaContext), new DFFS(formulaContext),
                    new JFFS(formulaContext), new WPSR(formulaContext), new WPNAME(formulaContext), new WPTYPE(formulaContext),
                    new SUM2(formulaContext), new RSUMIF(formulaContext), new QMYE(formulaContext), new ZC(formulaContext)};    //为CIT复制的一份注册自定义公式
        } else {
            functionImpls = new FreeRefFunction[]{new SGSR(formulaContext), new FSJZ(formulaContext), new ND(formulaContext),
                    new pwc.taxtech.atms.vat.service.impl.report.functions.validation.BB(formulaContext), new XXFP(formulaContext), new GZSD(formulaContext), new PC(formulaContext)
                    , new JXFPMX(formulaContext), new JXFP(formulaContext), new PSUM(formulaContext), new DFFS(formulaContext),
                    new JFFS(formulaContext), new WPSR(formulaContext), new WPNAME(formulaContext), new WPTYPE(formulaContext),
                    new SUM2(formulaContext), new RSUMIF(formulaContext), new QMYE(formulaContext), new ZC(formulaContext)};    //为CIT复制的一份注册自定义公式
        }

        //**************************** 为CIT复制的一份注册自定义公式 ****************************
        UDFFinder udfs = new DefaultUDFFinder(functions, functionImpls);
        UDFFinder udfToolpack = new AggregatingUDFFinder(udfs);
        workbook.addToolPack(udfToolpack);
    }

    public List<CellCalcInfoDto> getCellCalcInfo(List<Long> templateIdList, Integer periodParam, String
            projectId) {
        //already finished by 20180711
        int period = 0;
        if (periodParam != null) {
            period = periodParam;
        }

        List<PeriodCellTemplateConfigExtendDto> periodCellTemplateConfigExtendDtos =
                periodCellTemplateConfigMapper.getPeriodCellTemplateConfigExtendDtos(templateIdList, period, projectId);

        List<CellCalcInfoDto> cellCalcInfoDtos = new ArrayList<>();
        Map<String, List<PeriodCellTemplateConfigExtendDto>> myStream = new HashMap<>();
        for (PeriodCellTemplateConfigExtendDto pctce : periodCellTemplateConfigExtendDtos) {
            String key = pctce.getRowIndex() + "_" + pctce.getColumnIndex() + "_" + pctce.getReportTemplateId();
            if (!myStream.containsKey(key)) myStream.put(key, new ArrayList<>());
            myStream.get(key).add(pctce);
        }

        myStream.keySet().forEach(m -> {
            PeriodCellTemplateConfigExtendDto first = myStream.get(m).get(0);
            CellCalcInfoDto cellCalcInfoDto = new CellCalcInfoDto();
            cellCalcInfoDto.setColumnIndex(first.getColumnIndex());
            cellCalcInfoDto.setColumnName(first.getColumnName());
            cellCalcInfoDto.setRowIndex(first.getRowIndex());
            cellCalcInfoDto.setRowName(first.getRowName());
            cellCalcInfoDto.setFormula(convertListToString(myStream.get(m).stream()
                    .map(PeriodCellTemplateConfig::getFormula).collect(Collectors.toList())));
            cellCalcInfoDto.setParsedFormula(convertListToString(myStream.get(m).stream()
                    .map(PeriodCellTemplateConfig::getParsedFormula).collect(Collectors.toList())));
            //todo: add validation at here later
            cellCalcInfoDto.setCellTemplateId(first.getCellTemplateId() + "");
            cellCalcInfoDto.setReportTemplateId(first.getReportTemplateId());
            cellCalcInfoDto.setDataType(first.getDataType());
            cellCalcInfoDto.setIsReadOnly(first.getReadOnly());
            cellCalcInfoDto.setComment(first.getComment());
            cellCalcInfoDto.setConfigList(myStream.get(m).stream()
                    .map(PeriodCellTemplateConfigExtendDto::getPeriodCellTemplateConfig).collect(Collectors.toList()));
            cellCalcInfoDtos.add(cellCalcInfoDto);
        });
        return cellCalcInfoDtos;
    }

    private String convertListToString(List<String> list) {
        StringBuilder stringBuilder = new StringBuilder();
        list.forEach(s -> {
            if (StringUtils.isNotBlank(s)) {
                stringBuilder.append(s);
                stringBuilder.append("+");
            }
        });
        if (stringBuilder.length() > 0) {
            stringBuilder.deleteCharAt(stringBuilder.length() - 1);
        }
        return stringBuilder.toString();
    }


}
