package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.Constant;
import pwc.taxtech.atms.constant.enums.InvoiceRecordEnum;
import pwc.taxtech.atms.constant.enums.EnumOperationType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.KeyValueConfigResultType;
import pwc.taxtech.atms.dto.vatdto.OutputInvoiceDataSourceDto;
import pwc.taxtech.atms.vat.entity.*;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 根据收入类型配置查询开票收入开票金额总计
 */
public class KPSR extends FunctionBase implements FreeRefFunction {
    public KPSR(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 3) {
            return null;
        }

        String revenueTypeName = getStringParam(args[0], ec);//收入类型名称
        Integer billType = getIntParam(args[1], ec);//发票类型
        Integer amountType = getIntParam(args[2], ec);//金额类型
        String formulaExpression = "KPSR(\"" + revenueTypeName + "\"," + billType + "," + amountType + ")";
        logger.debug(formulaExpression);
        List<OutputInvoiceDataSourceDto> dataSource = new ArrayList<>();
        double result = assembleData(revenueTypeName, dataSource, billType, amountType, ec);
        Long dataSoureId = saveDataSource(ec, Collections.singletonList(dataSource), FormulaDataSourceDetailType.OutputInvoiceDataSourceDto, new BigDecimal(result), formulaContext.getPeriod(),
                formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
        saveFormulaBlock(formulaContext.getPeriod(), ec, formulaExpression, new BigDecimal(result), dataSoureId, formulaContext.getProjectId());
        return new NumberEval(result);

    }

    private double assembleData(String revenueTypeName, List<OutputInvoiceDataSourceDto> contain, Integer billType, Integer amountType, OperationEvaluationContext ec) {

        String queryDate = formulaContext.getYear() + (formulaContext.getPeriod() < 10 ? ("0" + formulaContext.getPeriod()) : (formulaContext.getPeriod() + ""));
//        RevenueTypeMappingExample typeMappingExample = new RevenueTypeMappingExample();
//        typeMappingExample.createCriteria().andOrgIdEqualTo(formulaContext.getOrganizationId())
//                .andRevenueTypeNameEqualTo(revenueTypeName).andStartDateLessThanOrEqualTo(queryDate)
//                .andEndDateGreaterThanOrEqualTo(queryDate);
//        List<RevenueTypeMapping> typeMappingList = SpringContextUtil.revenueTypeMappingMapper.selectByExample(typeMappingExample);
//        if (CollectionUtils.isEmpty(typeMappingList)) {
//            return 0.0;
//        }
//        List<String> revenueTypes = typeMappingList.stream()
//                .map(o -> o.getRevenueTypeName()).collect(Collectors.toList());
        RevenueConfigExample configExample = new RevenueConfigExample();
        configExample.createCriteria().andOrgIdEqualTo(formulaContext.getOrganizationId()).andStartDateLessThanOrEqualTo(queryDate)
                .andEndDateGreaterThanOrEqualTo(queryDate).andNameEqualTo(revenueTypeName);
        List<RevenueConfig> configDatas = SpringContextUtil.revenueConfigMapper.selectByExample(configExample);
        if (CollectionUtils.isEmpty(configDatas)) {
            return 0.0;
        }
        List<Long> revenueTypeIds = configDatas.stream()
                .map(o -> o.getId()).collect(Collectors.toList());
        List<InvoiceRecord> billDetails = new ArrayList<>();

        InvoiceRecordExample invoiceRecordExample = new InvoiceRecordExample();
        invoiceRecordExample.createCriteria().andRevenueCofIdIn(revenueTypeIds)
                .andProjectIdEqualTo(formulaContext.getProjectId())
                .andPeriodEqualTo(Integer.valueOf(queryDate))
                .andInvoiceTypeEqualTo(InvoiceRecordEnum.InvoiceType.MAPPING.get(billType));
        List<InvoiceRecord> billDetails1 = SpringContextUtil.invoiceRecordMapper.selectByExample(invoiceRecordExample);
        if(CollectionUtils.isNotEmpty(billDetails1)){
            Iterator<InvoiceRecord> iterator = billDetails1.iterator();
            while (iterator.hasNext()){
                if(iterator.next().getModifyRevenueCofId()!=null){
                    iterator.remove();
                }
            }
            billDetails.addAll(billDetails1);
        }
        invoiceRecordExample = new InvoiceRecordExample();
        invoiceRecordExample.createCriteria().andModifyRevenueCofIdIn(revenueTypeIds)
                .andProjectIdEqualTo(formulaContext.getProjectId())
                .andPeriodEqualTo(Integer.valueOf(queryDate))
                .andInvoiceTypeEqualTo(InvoiceRecordEnum.InvoiceType.MAPPING.get(billType));
        List<InvoiceRecord> billDetails2 = SpringContextUtil.invoiceRecordMapper.selectByExample(invoiceRecordExample);
        if(CollectionUtils.isNotEmpty(billDetails2)){
            billDetails.addAll(billDetails2);
        }
        for (InvoiceRecord invoiceRecord : billDetails) {
            OutputInvoiceDataSourceDto outputInvoiceDataSourceDto = new OutputInvoiceDataSourceDto();
            outputInvoiceDataSourceDto.setAmount(FormulaHelper.roundValue(invoiceRecord.getInvoiceAmount(),
                    KeyValueConfigResultType.Accounting, null, formulaContext));
            outputInvoiceDataSourceDto.setResultType(KeyValueConfigResultType.Accounting.getCode());
            outputInvoiceDataSourceDto.setTaxAmount(FormulaHelper.roundValue(invoiceRecord.getTaxAmount(),
                    KeyValueConfigResultType.Accounting, null, formulaContext));
            outputInvoiceDataSourceDto.setTaxRate(invoiceRecord.getTaxRate());
            outputInvoiceDataSourceDto.setBuyerName(invoiceRecord.getCustomerCompanyName());
            outputInvoiceDataSourceDto.setInvoiceCode(invoiceRecord.getInvoiceCode());
            outputInvoiceDataSourceDto.setInvoiceNumber(invoiceRecord.getInvoiceNum());
            outputInvoiceDataSourceDto.setInvoiceDate(invoiceRecord.getBillingDate());
            outputInvoiceDataSourceDto.setInvoiceType(billType);
            outputInvoiceDataSourceDto.setPeriod(formulaContext.getPeriod());
            outputInvoiceDataSourceDto.setName(Constant.DataSourceName.OutputInvoiceDataSource);
            outputInvoiceDataSourceDto.setOperationType(EnumOperationType.Single.getCode());
            contain.add(outputInvoiceDataSourceDto);
        }
        if (InvoiceRecordEnum.AmountType.INVOICE_AMOUNT.getCode().equals(amountType)) {
            return billDetails.stream().mapToDouble(a -> a.getInvoiceAmount().doubleValue()).sum();
        } else if (InvoiceRecordEnum.AmountType.TAX_AMOUNT.getCode().equals(amountType)) {
            return billDetails.stream().mapToDouble(a -> a.getTaxAmount().doubleValue()).sum();
        }
        return 0;
    }


}
