package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.common.util.SpringContextUtil;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.dto.vatdto.RSUMIFParasBo;
import pwc.taxtech.atms.vat.entity.PeriodFormulaBlock;
import pwc.taxtech.atms.vat.service.impl.FormulaAgent;

import java.math.BigDecimal;
import java.util.*;


/// <summary>
/// 名称：ND
/// 功能：获取某一年度值
/// 备注：只有一个参数： “0”表示当年，“1”表示下一年度，“-1”表示上一年度，“-2”前两年 如当前年度为2017=ND(0),  2016=ND(-1),   2015=ND(-2)
/// </summary>
/// <param name="v">本年度的相对值</param>
/// <returns></returns>
public class ND extends FunctionBase implements FreeRefFunction {

    public ND(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        List<Object> dataSource = new ArrayList<>();
        Calendar calendar = Calendar.getInstance();
        Date creatTime = calendar.getTime();
        int value = getIntParam(args[0], ec);
        int currentYear = calendar.get(Calendar.YEAR);
        Long cellTemplateId = getCellTemplateId(formulaContext.getPeriod(), ec);
        String formulaExpression = "ND(" + value + ")";
        value = value + currentYear;
        logger.debug(formulaExpression);
     /*   PeriodFormulaBlock periodFormulaBlock = new PeriodFormulaBlock();
        periodFormulaBlock.setId(SpringContextUtil.distributedIdService.nextId());
        periodFormulaBlock.setPeriod(formulaContext.getPeriod());
        periodFormulaBlock.setReportId(0L);
        periodFormulaBlock.setCellTemplateId(cellTemplateId);
        periodFormulaBlock.setFormulaExpression(formulaExpression);
        periodFormulaBlock.setData(Integer.toString(value));
        periodFormulaBlock.setCreateBy("Admin");
        periodFormulaBlock.setCreateTime(creatTime);
        periodFormulaBlock.setUpdateTime(creatTime);
        periodFormulaBlock.setUpdateBy("Admin");
        SpringContextUtil.formulaBlockMapper.insertSelective(periodFormulaBlock);*/

        Long dataSourceId = saveDataSource(ec, Collections.singletonList(dataSource),
                FormulaDataSourceDetailType.AssetDetailDataSourceDto,
                new BigDecimal(value), formulaContext.getPeriod(), formulaContext.getReportTemplateGroupId(), formulaContext.getProjectId());
        saveFormulaBlock(0, ec, formulaExpression,  new BigDecimal(value), dataSourceId, formulaContext.getProjectId());
        return new NumberEval(value);
    }
}