package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceType;
import pwc.taxtech.atms.dto.TableRule;
import pwc.taxtech.atms.dto.vatdto.RSUMIFParasBo;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.ReportCellTableSUMIFDataSourceDto;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.service.impl.FormulaAgent;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static pwc.taxtech.atms.common.util.FormulaUtil.resolverInteger;
import static pwc.taxtech.atms.common.util.FormulaUtil.resolverString;

/**
 * @ClassName RSUMIF
 * Description TODO
 * @Author pwc kevin
 * @Date 3/11/2019 2:52 PM
 * Version 1.0
 **/
public class RSUMIF extends FunctionBase implements FreeRefFunction {

    private String tableName;
    Map<String, String> _map = TableRule.map;

    public RSUMIF(FormulaContext formulaContext) {
        super(formulaContext);
    }

    //进行参数验证
    public void parameterCheck(ValueEval[] args, OperationEvaluationContext ec) {
        if (args.length < 6 && (args.length & 1) == 1) {
            //参数小于6且如果是奇数的话当做异常处理， 最后俩参数可以取默认值
            throw Exceptions.parameterError;
        }
        try {
            tableName = resolverString(args, ec, 0);
            if (!_map.containsKey(tableName)) {
                throw Exceptions.parameterError;
            }
        } catch (EvaluationException e) {
            e.printStackTrace();
        }

    }

    String getField;
    Integer period;
    Integer year;

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        List<Object> dataSource = new ArrayList<>();
        BigDecimal cellValue = BigDecimal.ZERO;
        String filter = null;
        String filterValue = null;
        //减4的原因是去掉前面两个参数和后面两个参数，剩下的参数才是条件列与条件值的总和，再除以2就是条件列的的数量
        String[] filters = new String[(args.length - 4)/2];
        String[] filterValues = new String[(args.length - 4)/2];
        try {
            parameterCheck(args, ec);
            Map map = new HashMap<String, String>();
            ReportCellTableSUMIFDataSourceDto reportCellTableSUMIFDataSourceDto = new ReportCellTableSUMIFDataSourceDto();
            reportCellTableSUMIFDataSourceDto.insertData(ec, formulaContext);

            dataSource.add(reportCellTableSUMIFDataSourceDto);
            tableName = resolverString(args, ec, 0);
            getField = resolverString(args, ec, 1);
            //获取条件列及条件值

            for (int i = 2; i < args.length - 2; i++) {
                filters[(i - 2)/2] = resolverString(args, ec, i);
                filterValues[(i - 2)/2] = resolverString(args, ec, ++i);
            }
            resolverFilterAndValues(args);
//            filter = resolverString(args, ec, 2);
////            filterValue = resolverString(args, ec, 3);
            year = resolverInteger(args, ec, args.length - 2);//会计年度
            period = Integer.parseInt(resolverString(args, ec, args.length - 1));//会计期间
            cellValue = agent.getTableDataByName(tableName, getField, filters, filterValues, period, year, formulaContext);
            return new NumberEval(cellValue.doubleValue());
        } catch (EvaluationException e) {
            e.printStackTrace();
            return new NumberEval(0.00);
        } finally {
            boolean boo;

            Long dataSourceId = null;
            try {
                if (TableRule.map.get(tableName).substring(0, 3).equals("cit")) {
                    boo = true;
                } else {
                    boo = false;
                }
                dataSourceId = saveDataRSUMIFSource(ec, dataSource, FormulaDataSourceDetailType.ReportCellTableSUMIFDataSourceDto,
                        cellValue, formulaContext.getPeriod(),
                        formulaContext.getReportTemplateGroupId(), ec.getColumnIndex(), ec.getRowIndex(),
                        formulaContext.getProjectId(), selectShow(tableName), FormulaAgent._buildSql(getField, tableName, filters, filterValues, period, formulaContext, year, boo, true));
                saveFormulaBlock(formulaContext.getPeriod(), ec,
                        new RSUMIFParasBo().getExpression(args, ec), cellValue, dataSourceId, formulaContext.getProjectId());
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    //解决不同表结构关联不同的展示方式
    public FormulaDataSourceType selectShow(String tableName) {
        if ("CIT_TBAM".equals(tableName)) {
            return FormulaDataSourceType.CIT_TBAM;
        } else {
            return FormulaDataSourceType.Other;
        }
    }

    public static String getSql() {
        return "";
    }

    public void resolverFilterAndValues(ValueEval[] args) {

    }


}
