package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.commons.collections.CollectionUtils;
import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.constant.enums.FormulaDataSourceDetailType;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dto.vatdto.ReportCellDataSourceDto;
import pwc.taxtech.atms.dto.vatdto.WPTYPEParasBo;
import pwc.taxtech.atms.exception.FormulaException;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

public class WPTYPE extends FunctionBase implements FreeRefFunction {
    static final Logger LOGGER = LoggerFactory.getLogger(WPTYPE.class);
    final static ValueEval defaultEval = new StringEval("0");
    final static BigDecimal defaultBigDecimal = new BigDecimal("0");

    public WPTYPE(FormulaContext context) {
        super(context);
    }

    @Override
    public ValueEval evaluate(ValueEval[] args, OperationEvaluationContext ec) {
        try {
            return wrapExceptionEval(args, ec);
        } catch (Exception e) {
            if (e instanceof FormulaException)
                LOGGER.warn("Formula Exception || {}", e.getMessage());

            e.printStackTrace();
            return defaultEval;
        }
    }

    public ValueEval wrapExceptionEval(ValueEval[] args, OperationEvaluationContext ec) throws Exception {
        List<Object> ds = new ArrayList<>();
        return new NumberEval(bb(new WPTYPEParasBo(args, ec), ec, ds).doubleValue());
    }

    public BigDecimal bb(WPTYPEParasBo bo, OperationEvaluationContext ec, List<Object> dataSource) throws Exception {
        List<CellTemplatePerGroupDto> cellTemplateDataList = new ArrayList<CellTemplatePerGroupDto>();
        BigDecimal cellValue = BigDecimal.ZERO;
        try {
            String queryDate = formulaContext.getYear() + "-" + (formulaContext.getPeriod() < 10 ? "0" + formulaContext.getPeriod() : formulaContext.getPeriod());
            //根据type数据行，再根据参数列+数据行定位sheet单元格
            cellTemplateDataList = agent.getCellTemplateByTypeAndIndex(formulaContext.getReportTemplateGroupId(),
                    formulaContext.getProjectId(), bo.getReportCode(), bo.getRowColumnIndex() - 1, bo.getTaxRate(), bo.getRevenueType(), bo.getTaxType(), bo.getColumnIndex() - 1,
                    formulaContext.getPeriod(), formulaContext.getOrganizationId(), queryDate, queryDate);
//            if(CollectionUtils.isEmpty(cellTemplateDataList)){
//                cellTemplateDataList = agent.getCellTemplateGroupDto(formulaContext.getReportTemplateGroupId(),
//                        formulaContext.getProjectId(), bo.getReportCode(), ec.getRowIndex(), ec.getColumnIndex(), formulaContext.getPeriod());
//            }
//            MyAsserts.assertNotEmpty(cellTemplateDataList, Exceptions.BB_CELL_TEMP_NULL);
            if(CollectionUtils.isEmpty(cellTemplateDataList)){
                return cellValue;
            }
            for (CellTemplatePerGroupDto cellTemplateData : cellTemplateDataList) {
                int index = ec.getWorkbook().getSheetIndex(bo.getReportCode());
                BigDecimal curCellVal = getCellValue(index, ec, formulaContext, agent, cellTemplateData.getRowIndex(), bo.getColumnIndex() - 1,
                        Long.parseLong(cellTemplateData.getCellTemplateId()));
                cellValue = cellValue.add(curCellVal);
                ReportCellDataSourceDto nullCellDto = new ReportCellDataSourceDto();
                nullCellDto.extractFromGroup(bo, formulaContext.getPeriod(), formulaContext.getYear(), cellTemplateData);
                nullCellDto.setAmount(curCellVal);
                dataSource.add(nullCellDto);
            }
            return cellValue;
        } catch (Exception e) {
            logger.warn("[BB_Exception] some error {}", bo.toString());
            throw e;
        } finally {
            LOGGER.warn("[BB_Exception] error for bb cacls for {}", bo.toString());
            Long dataSourceId = saveDataSource(ec, dataSource, FormulaDataSourceDetailType.ReportCellDataSourceDto,
                        cellValue, formulaContext.getPeriod(),
                        formulaContext.getReportTemplateGroupId(), ec.getColumnIndex(), ec.getRowIndex(),
                        formulaContext.getProjectId());
            saveFormulaBlock(formulaContext.getPeriod(), ec,
                        bo.getFormulaExpression(), cellValue, dataSourceId, formulaContext.getProjectId());


        }
    }

}
