package pwc.taxtech.atms.vat.service.impl.report.functions;

import org.apache.poi.ss.formula.OperationEvaluationContext;
import org.apache.poi.ss.formula.eval.EvaluationException;
import org.apache.poi.ss.formula.eval.NumberEval;
import org.apache.poi.ss.formula.eval.StringEval;
import org.apache.poi.ss.formula.eval.ValueEval;
import org.apache.poi.ss.formula.functions.FreeRefFunction;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.exception.FormulaException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static pwc.taxtech.atms.common.util.FormulaUtil.resolverInteger;
import static pwc.taxtech.atms.common.util.FormulaUtil.resolverString;

/**
 * @ClassName _RSUMIF
 * Description TODO 公式还需要校验
 * @Author pwc kevin
 * @Date 3/8/2019 5:21 PM
 * Version 1.0
 **/
public class _RSUMIF extends FunctionBase implements FreeRefFunction {

    final static ValueEval defaultEval = new StringEval("0");
    static final Logger LOGGER = LoggerFactory.getLogger(_RSUMIF.class);
    String tableName;
    String getField;//取值列字段
    Map<String, String> fileterMap = new HashMap<String, String>();
    String year;
    int period;
    String selectPeriod = "";
    String selectFilter = "";
    Long result;
    int argsLength;
    int selectCount;

    public _RSUMIF(FormulaContext formulaContext) {
        super(formulaContext);
    }

    @Override
    public ValueEval evaluate(ValueEval[] valueEvals, OperationEvaluationContext operationEvaluationContext) {
        try {
            return wrapExceptionEval(valueEvals, operationEvaluationContext);
        } catch (Exception e) {
            if (e instanceof FormulaException)
                LOGGER.warn("Formula Exception || {}", e.getMessage());
            e.printStackTrace();
            return defaultEval;
        }
    }

    public ValueEval wrapExceptionEval(ValueEval[] args, OperationEvaluationContext ec) throws Exception {
        List<Object> ds = new ArrayList<>();
        parameterCheck(args);
        setWhere(args, ec);
        List<CellTemplatePerGroupDto> tableData = null;
        try {
            tableData = agent.getTableData(tableName, getField, selectFilter, year, selectPeriod);
        } catch (Exception e) {
            e.printStackTrace();
            throw new Exception("公式参数配置错误");
        }
        for (CellTemplatePerGroupDto cellTemplatePerGroupDto : tableData) {
            result = result + Long.parseLong(cellTemplatePerGroupDto.getData());
        }
        return new NumberEval(result);
    }


    //进行参数验证
    public void parameterCheck(ValueEval[] args) {
        argsLength = args.length;
        if (argsLength < 4) {//参数小于3当做异常处理， 最后俩参数可以取默认值
            throw Exceptions.parameterError;
        }
    }

    private void setWhere(ValueEval[] args, OperationEvaluationContext ec) {
        try {
            selectCount = (argsLength - 4) / 2;
            tableName = resolverString(args, ec, 0);
            getField = resolverString(args, ec, 1);
            period = resolverInteger(args, ec, argsLength - 1);//会计期间
            year = resolverString(args, ec, argsLength - 2);//会计年度
            if (selectCount % 2 != 0)
                throw Exceptions.parameterError;
            for (int i = 3; i < selectCount * 2 + 3; i++) {
                if (i % 2 != 0) {
                    fileterMap.put(resolverString(args, ec, i), resolverString(args, ec, i + 1));
                }
            }
            if (period == 0) {
                selectPeriod = "t.period = " + formulaContext.getPeriod();
            } else if (period == -99) {
                selectPeriod = "t.period between  '0'  and " + (formulaContext.getPeriod() - 1);
            } else if (period == 99) {
                selectPeriod = "t.period between '0' and " + (formulaContext.getPeriod() - 1);
            } else {
                selectPeriod = "t.period between " + (formulaContext.getPeriod() - period) + " and " + (formulaContext.getPeriod() - 1);
            }

            for (Map.Entry<String, String> entry : fileterMap.entrySet()) {
                if (selectFilter.equals("")) {
                    selectFilter = "t." + entry.getKey() + entry.getValue();
                } else {
                    selectFilter = selectFilter + " and " + "t." + entry.getKey() + entry.getValue();
                }
            }
        } catch (EvaluationException e) {
            e.printStackTrace();
        }
    }

}
