package pwc.taxtech.atms.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.assertj.core.util.Lists;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import pwc.taxtech.atms.CommonIT;
import pwc.taxtech.atms.common.AuthUserHelper;
import pwc.taxtech.atms.common.util.BaseThreadFactory;
import pwc.taxtech.atms.common.util.DateUtils;
import pwc.taxtech.atms.common.util.HttpUtil;
import pwc.taxtech.atms.constant.EbsExtractTypeConstant;
import pwc.taxtech.atms.constant.enums.EnumApiCodeMsg;
import pwc.taxtech.atms.constant.enums.EnumEbsExtractType;
import pwc.taxtech.atms.constant.enums.EnumImportType;
import pwc.taxtech.atms.dao.DataImportLogMapper;
import pwc.taxtech.atms.dto.DataExtractParam;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.ebsdto.*;
import pwc.taxtech.atms.dto.organization.DDSyncOrgInfo;
import pwc.taxtech.atms.dto.organization.OrgSyncData;
import pwc.taxtech.atms.entity.*;
import pwc.taxtech.atms.exception.ServiceException;
import pwc.taxtech.atms.security.dd.DtsTokenService;
import pwc.taxtech.atms.service.EbsApiService;

import javax.annotation.Resource;
import javax.crypto.Cipher;
import java.math.BigDecimal;
import java.security.KeyFactory;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;

public class EbsApiServiceImplTest extends CommonIT {

    private static final Logger logger = LoggerFactory.getLogger(EbsApiServiceImplTest.class);

    @Resource
    private EbsApiService ebsApiService;
    @Resource
    private DistributedIdService idService;
    @Autowired
    private DataImportService dataImportService;
    @Value("${org_sync_url}")
    private String orgSyncUrl;

    @Value("${org_sync_token}")
    private String token;

    @Resource
    private DtsTokenService dtsTokenService;

    private static String ebsCallUrl = "http://172.20.201.201:8020/ebs-proxy-test/dts";

    private static int corePoolSize = 15;

    private static int maxPoolSize = 15;

    private static int qSize = 5000;

    private static BaseThreadFactory factory = new BaseThreadFactory("callEbs");

    private static ExecutorService executorService = new ThreadPoolExecutor(corePoolSize, maxPoolSize,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(qSize),factory,new ThreadPoolExecutor.AbortPolicy());


    @Test
    public void testExtractTB(){
        try{
            String response = HttpUtil.post("http://172.20.201.201:8020/ebs-proxy-test/dts/glMonthlyBal?ledgerId=2021&companyCode=120200&period=2018-11",
                    "",
                    "application/json;charset=utf-8", "UTF-8", 10000, 10000);
            String ledgerId = "2021";
            String code = "120200";
            String period = "2018-11";
            String url = ebsCallUrl + "/glMonthlyBal?ledgerId="+ledgerId+"&companyCode="+ code + "&period=" + period;
            System.out.println(url);
            response = HttpUtil.post(url,"","application/json;charset=utf-8", "UTF-8", 10000, 10000);
            DataImportLog log = new DataImportLog();
            log.setTaxpayerIdNum("123");
            log.setOrganizationId(ledgerId);
            log.setCompanyCode(code);
            log.setCompanyName(ledgerId);
            log.setPeriodYear(2018);
            log.setPeriodMonth(11);
            log.setType(EnumImportType.ExtractFinancialData.getCode());
            log.setFileType("caicai");
            log.setPeriodStatus("EBS抽取");
            log.setOperator("admin");
            System.out.println(response);
            if (StringUtils.isNotEmpty(response)) {
                EbsCallResp resp = JSON.parseObject(response, EbsCallResp.class);
                long taskId = Long.valueOf(resp.getTaskId());
                if (taskId == -1) {
                    log.setId(idService.nextId());
                    log.setImportResult(false);
                    log.setErrorMsg(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), resp.getTaskDesc()));
                } else {
                    log.setId(taskId);
                    log.setImportResult(true);
                    log.setErrorMsg(String.format(EnumApiCodeMsg.CALLSUCCESS.getMsg(), resp.getTaskDesc()));
                }
                int i = dataImportLogMapper.insertSelective(log);
                System.out.println(i);
            }
            System.out.println(response);
        }catch (Exception e){
            e.printStackTrace();
        }
    }

    @Test
    public void testExtract(){
        DataExtractParam param = new DataExtractParam();
        List<String> orgs = Lists.newArrayList();
        orgs.add("5bbd739c-1a13-4b0f-aba6-32ba41e4de69");
        List<Integer> dataTypes = Lists.newArrayList();
        dataTypes.add(100);
        dataTypes.add(101);
        dataTypes.add(102);
        dataTypes.add(103);
        dataTypes.add(104);
        dataTypes.add(105);
        dataTypes.add(106);
        dataTypes.add(108);
        param.setOrgIds(orgs);
        param.setDataTypes(dataTypes);
        param.setPeriod("2019-03");
        callExtractFinancialData(param);
    }

    public OperationResultDto callExtractFinancialData(DataExtractParam param) {
        OrganizationExample example = new OrganizationExample();
        example.createCriteria().andIdIn(param.getOrgIds());
        List<Organization> orgs = organizationMapper.selectByExample(example);
        Integer periodInt = DateUtils.strToPeriod(param.getPeriod());
        String period = param.getPeriod();
        // 这两个参数未来可能有用
        String effectiveDateFrom = param.getEffectiveDateFrom();
        String effectiveDateTo = param.getEffectiveDateTo();
        List<Future> list = Lists.newArrayList();
        param.getDataTypes().forEach(type -> {
            orgs.forEach(o -> {
                try {
                    Callable callEbs = new CallEbsThread(type, o,ebsCallUrl,dataImportLogMapper,period, effectiveDateFrom, effectiveDateTo);
                    Future future = executorService.submit(callEbs);
                    list.add(future);
                } catch (RejectedExecutionException rje) {
                    // 这里可以考虑简化 没想好还
                    DataImportLog log = new DataImportLog();
                    log.setId(idService.nextId());
                    log.setTaxpayerIdNum(o.getTaxPayerNumber());
                    log.setCompanyCode(o.getCode());
                    log.setCompanyName(o.getName());
                    log.setPeriodYear(periodInt / 100);
                    log.setPeriodMonth(periodInt % 100);
                    log.setType(EnumImportType.ExtractFinancialData.getCode());
                    log.setFileType(EnumEbsExtractType.getNameByCode(type));
                    log.setImportResult(false);
                    log.setPeriodStatus("EBS抽取");
                    log.setOperator("test");
                    log.setErrorMsg(EnumApiCodeMsg.BUSY.getMsg());
                    dataImportLogMapper.insertSelective(log);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            });
        });
        int res = 0;

        try{
            for(Future future : list){
                res += (Integer)list.get(0).get();
            }
            System.out.println(res);
        }catch (Exception e){
            e.printStackTrace();
        }

        return OperationResultDto.success();
    }

    @Test
    public void testOrgApi(){
        Map<String, String> headers = new HashMap<>();
        headers.put("token", token);
        headers.put("Content-Type", "application/x-www-form-urlencoded");
        headers.put("Idap", "eddie.wu_v");
        int pageSize = 1;
        List<String> failedCnList = Lists.newArrayList();
        List<String> failedList = Lists.newArrayList();
        List<OrgSyncData> failedObjList = Lists.newArrayList();
        try {
            for (int i = 1; i <= pageSize; i++) {
                String response = HttpUtil.get(orgSyncUrl+"?currentPage="+i, headers);
                System.out.println(response);
                DDSyncOrgInfo ddSyncOrgInfo = JSONObject.parseObject(response, DDSyncOrgInfo.class);
                pageSize = ddSyncOrgInfo.getTotalPage();
                List<OrgSyncData> orgSyncDatas = ddSyncOrgInfo.getData();
                orgSyncDatas.forEach(osd -> {
                    OrganizationExample example = new OrganizationExample();
                    example.createCriteria().andNameEqualTo(osd.getNameCN());
                    Organization o = new Organization();
                    o.setClientCode(osd.getCode());
                    o.setCode(osd.getCode());
                    o.setEnterpriseAccountCode(String.valueOf(osd.getSobId()));
                    o.setEnterpriseAccountName(osd.getSobName());
                    o.setCurrencyCode(osd.getCurrencyCode());
                    o.setLegalEntity(osd.getLegalEntity());
                    o.setLegalPersonName(osd.getLegalRepresentative());
                    o.setAddress(osd.getAddress());
                    o.setCreateTime(osd.getGmtCreate());
                    o.setUpdateTime(osd.getGmtModified());
                    o.setPsCode(osd.getPsCode());
                    RegionExample regionExample = new RegionExample();
                    regionExample.createCriteria().andShortNameEqualTo(osd.getCompanyLocation());
                    List<Region> regions = regionMapper.selectByExample(regionExample);
                    if (regions.size() > 0) {
                        o.setRegionId(regions.get(0).getId());
                    }
                    if(organizationMapper.countByExample(example)<1){
                        failedCnList.add(osd.getNameCN());
                        failedList.add(osd.getOrgName());
                        failedObjList.add(osd);
                    }
                    organizationMapper.updateByExampleSelective(o, example);
                });
            }
            System.out.println(failedCnList);
            System.out.println(failedList);
            System.out.println(JSON.toJSONString(failedObjList));
        } catch (Exception e) {
            System.out.println(String.format("机构信息同步异常:[%s]", e.getMessage()));
        }
    }

    @Test
    public  void testApi(){
        queryRemoteServerThenUpdateJE();
        queryRemoteServerThenUpdateTB();
        queryRemoteServerThenUpdateCF();
        queryRemoteServerThenUpdateBS();
        queryRemoteServerThenUpdateBSprc();
        queryRemoteServerThenUpdatePL();
        queryRemoteServerThenUpdatePLprc();
        queryRemoteServerThenUpdateOAR();
    }
    @Test
    public void queryRemoteServerThenUpdateOrg(){
        List<OrganizationQueryDto> items = new ArrayList();
        for(int i=0;i<2;i++){
            OrganizationQueryDto dto = new OrganizationQueryDto();
            dto.setTaxPayerNumber("sadsafsdf3232");
            dto.setCode("ceshi0009");
            dto.setName("测试成功");
            dto.setEnterpriseAccountCode("test");
            dto.setLegalPersonName("周杰伦");
            dto.setRegistrationCapital("100万");
            dto.setPaidInCapital("200万");
            dto.setLegalPersonPhoneNumber("1239512312");
            dto.setLegalPersonLandlineNum("1234678");
            dto.setLegalPersonEmailAddress("didi@qq.com");
            dto.setRegFinancialOfficerName("周杰伦");
            dto.setRegFinancialOfficerPhoneNum("123141231");
            dto.setRegFinancialOfficerLandlineNum("123213124");
            dto.setRegFinancialOfficerEmailAddress("滴滴大厦");
            dto.setCurrencyCode("CNY");
            dto.setRegisterAddress("滴滴大厦");
            dto.setParentName("滴滴测试");
            dto.setStaffSize(12345);
            items.add(dto);
        }
        String res=JSON.toJSONString(items);
        System.out.println(res);
//        ebsApiService.queryRemoteServerThenUpdateOrg(items);
    }

    @Test
    public  void queryRemoteServerThenUpdateJE(){
        List<JournalEntryQueryDto> items=new ArrayList<>();
        for(int i=0;i<10;i++){
            JournalEntryQueryDto result =new JournalEntryQueryDto();
            result.setSource("来源");
            result.setLedgerId("账套ID");
            result.setLedgerName("账套名称");
            result.setCurrencyCode("账套币种");
            result.setHeaderId("日记账头ID");
            result.setLineNum("日记账行号");
            result.setApprovalStatus("审批状态");
            result.setPostedStatus("过账");
            result.setPeriod("2018-11");
            result.setJournalSource("日记账来源");
            result.setCategory("日记账类别");
            result.setName("日记账名称");
            result.setVoucherNum("凭证编号");
            result.setDescription("摘要");
            result.setSegment1("主体代码");
            result.setSegment2("成本中心");
            result.setSegment3("科目代码");
            result.setSegment4("辅助科目");
            result.setSegment5("利润中心");
            result.setSegment6("产品");
            result.setSegment7("项目");
            result.setSegment8("公司间");
            result.setSegment9("备用1");
            result.setSegment10("备用2");
            result.setSegment1Name("主体说明");
            result.setSegment2Name("成本中心说明");
            result.setSegment3Name("科目说明");
            result.setSegment4Name("辅助科目说明");
            result.setSegment5Name("利润中心说明");
            result.setSegment6Name("产品说明");
            result.setSegment7Name("项目说明");
            result.setSegment8Name("公司间说明");
            result.setSegment9Name("备用1说明");
            result.setSegment10Name("备用2说明");
            result.setJournalCurrencyCode("币种");
            result.setSobCurrencyCode("本位币币种");
            result.setAccountedDr(new BigDecimal("123"));
            result.setAccountedCr(new BigDecimal("120.2"));
            result.setEnteredDr(new BigDecimal("1231.0231"));
            result.setEnteredCr(new BigDecimal("120.122"));
            result.setCfItem("现金流量表项");
            result.setAttribute1("城市");
            result.setAttribute3("对方银行账号");
            result.setAttribute4("银行流水号");
            result.setAttribute5("供应商编号");
            result.setAttribute6("交易单号");
            result.setAttribute7("供应商名称");
            result.setAttribute8("接收编码");
            result.setAttribute9("制单人");
            result.setAttribute10("审核人");
            result.setAttribute11("成本中心部门描述1");
            result.setAttribute12("成本中心部门描述2");
            result.setAttribute13("成本中心部门描述3");
            result.setAttribute14("成本中心部门描述4");
            result.setAttribute15("成本中心部门描述5");
            result.setAttribute16("成本中心部门描述6");
            result.setCreatedBy("pwcCreat");
            result.setLateUpdatedBy("pwcUpate");
            items.add(result);
        }
        String a=JSON.toJSONString(items);
        System.out.println(a);
        //ebsApiService.queryRemoteServerThenUpdateJE(items);
    }

    @Test
    public void queryRemoteServerThenUpdateTB() {
        List<TrialBalanceQueryDto> items=new ArrayList<>();
        for(int i=0;i<2;i++) {
            TrialBalanceQueryDto result = new TrialBalanceQueryDto();
//            result.setDate("2019-03-04 19:14:17");
//            result.setStatus(false);
//            result.setSource("来源");
//            result.setPeriod(201803);
            result.setLedgerId("账套ID");
            result.setLedgerName("账套名称");
            result.setCurrencyCode("账套币种");
            result.setCategory("主体性质");
            result.setAccountCategory("科目性质");
            result.setAcctCode1("一级科目代码");
            result.setAcctName1("一级科目说明");
            result.setAcctName2("二级科目说明");
            result.setAcctName3("三级科目说明");
            result.setSegment1("主体代码");
            result.setSegment2("成本中心代码");
            result.setSegment3("科目代码");
            result.setSegment4("辅助科目代码");
            result.setSegment5("利润中心代码");
            result.setSegment6("产品代码");
            result.setSegment7("项目代码");
            result.setSegment8("公司间代码");
            result.setSegment9("备用1代码");
            result.setSegment10("备用2代码");
            result.setSegment1Name("主体说明");
            result.setSegment2Name("成本中心说明");
            result.setSegment3Name("科目说明");
            result.setSegment4Name("辅助科目说明");
            result.setSegment5Name("利润中心说明");
            result.setSegment6Name("产品说明");
            result.setSegment7Name("项目说明");
            result.setSegment8Name("公司间说明");
            result.setSegment9Name("备用1说明");
            result.setSegment10Name("备用2说明");
            result.setBegBal(new BigDecimal("110"));
            result.setPeriodDr(new BigDecimal("110"));
            result.setPeriodCr(new BigDecimal("110"));
            result.setEndBal(new BigDecimal("110"));
            result.setQtdDr(new BigDecimal("110"));
            result.setQtdCr(new BigDecimal("110"));
            result.setYtdDr(new BigDecimal("110"));
            result.setYtdCr(new BigDecimal("110"));
            result.setBegBalBeq(new BigDecimal("110"));
            result.setPeriodDrBeq(new BigDecimal("110"));
            result.setPeriodCrBeq(new BigDecimal("110"));
            result.setEndBalBeq(new BigDecimal("110"));
            result.setQtdDrBeq(new BigDecimal("110"));
            result.setQtdCrBeq(new BigDecimal("110"));
            result.setYtdDrBeq(new BigDecimal("110"));
            result.setYtdCrBeq(new BigDecimal("110"));
            items.add(result);
        }
        String a=JSON.toJSONString(items);
        System.out.println(a);
    }

    @Test
    public void queryRemoteServerThenUpdateCF() {

        List<CashFlowQueryDto> items=new ArrayList<>();
        for(int i=0;i<2;i++) {
            CashFlowQueryDto result = new CashFlowQueryDto();
//            result.setDate("2019-03-04 19:14:17");
//            result.setSource("来源");
//            result.setPeriod(201803);
//            result.setStatus(true);
            result.setLedgerId("账套ID");
            result.setLedgerName("账套名称");
            result.setLedgerCurrencyCode("账套币种");
            result.setEntityCode("机构编码");
            result.setEntityName("机构名称");
            result.setCategory("主体性质");
            result.setFrequency("频度");
            result.setItemName("项目中文名称");
            result.setItemName2("项目英文名称");
            result.setPeriodAmt(new BigDecimal("1000.0454"));
            result.setYtdAmt(new BigDecimal("1000.0454"));
            items.add(result);
        }
        String a=JSON.toJSONString(items);
        System.out.println(a);

    }

    @Test
    public void queryRemoteServerThenUpdateBS() {
        List<BalanceSheetQueryDto> items=new ArrayList<>();
        for(int i=0;i<2;i++) {
            BalanceSheetQueryDto result = new BalanceSheetQueryDto();
//            result.setDate("2019-03-04 19:14:17");
//            result.setPeriod(201812);
//            result.setStatus(false);
            result.setLedgerCurrencyCode("账套币种");
            result.setEntityCode("机构编码");
            result.setEntityName("机构名称");
            result.setItemName("项目名称");
            result.setEndBal(new BigDecimal("1220.2980"));
            result.setBegBal(new BigDecimal("1220.2980"));
            result.setSource("来源");
            result.setLedgerId("账套ID");
            result.setLedgerName("账套名称");
            result.setCategory("主体性质");
            result.setFrequency("频度");
            items.add(result);
        }
            String a=JSON.toJSONString(items);
            System.out.println(a);

    }

    @Test
    public void queryRemoteServerThenUpdateBSprc() {
        List<BalanceSheetPrcQueryDto> items=new ArrayList<>();
        for(int i=0;i<2;i++) {
            BalanceSheetPrcQueryDto result = new BalanceSheetPrcQueryDto();
//            result.setDate("2019-03-04 19:14:17");
//            result.setStatus(false);
//            result.setSource("来源");
//            result.setPeriod(201804);
            result.setLedgerId("账套ID");
            result.setLedgerName("账套名称");
            result.setLedgerCurrencyCode("账套币种");
            result.setEntityCode("机构编码");
            result.setEntityName("机构名称");
            result.setCategory("主体性质");
            result.setFrequency("频度");
            result.setItemName("项目名称");
            result.setEndBal(new BigDecimal("1220.2980"));
            result.setBegBal(new BigDecimal("1220.2980"));
            items.add(result);
        }
        String a=JSON.toJSONString(items);
        System.out.println(a);
    }

    @Test
    public void queryRemoteServerThenUpdatePL() {
        List<ProfitLossStatementQueryDto> items=new ArrayList<>();
        for(int i=0;i<2;i++) {
            ProfitLossStatementQueryDto result = new ProfitLossStatementQueryDto();
//            result.setDate("2019-03-04 19:14:17");
//            result.setStatus(false);
//            result.setSource("来源");
//            result.setPeriod(201904);
            result.setLedgerId("账套ID");
            result.setLedgerName("账套名称");
            result.setLedgerCurrencyCode("账套币种");
            result.setEntityCode("机构编码");
            result.setEntityName("机构名称");
            result.setCategory("主体性质");
            result.setFrequency("频度");
            result.setItemName("项目名称");
            result.setPeriodAmt(new BigDecimal("1220.2980"));
            result.setYtdAmt(new BigDecimal("1220.2980"));
            items.add(result);
        }
        String a=JSON.toJSONString(items);
        System.out.println(a);
    }

    @Test
    public void queryRemoteServerThenUpdatePLprc() {
        List<ProfitLossStatementPrcQueryDto> items=new ArrayList<>();
        for(int i=0;i<2;i++) {
            ProfitLossStatementPrcQueryDto result = new ProfitLossStatementPrcQueryDto();
//            result.setDate("2019-03-04 19:14:17");
//            result.setStatus(false);
//            result.setSource("来源");
//            result.setPeriod(201904);
            result.setLedgerId("账套ID");
            result.setLedgerName("账套名称");
            result.setLedgerCurrencyCode("账套币种");
            result.setEntityCode("机构编码");
            result.setEntityName("机构名称");
            result.setCategory("主体性质");
            result.setFrequency("频度");
            result.setItemName("项目名称");
            result.setPeriodAmt(new BigDecimal("1220.2980"));
            result.setYtdAmt(new BigDecimal("1220.2980"));
            items.add(result);
        }
        String a=JSON.toJSONString(items);
        System.out.println(a);
    }

    @Test
    public void queryRemoteServerThenUpdateOAR() {
            List<OrganizationAccountingRateQueryDto> items=new ArrayList<>();
            for(int i=0;i<2;i++) {
                OrganizationAccountingRateQueryDto result = new OrganizationAccountingRateQueryDto();
//                result.setPeriod(2018);
                result.setConvertionType("汇率类型名称");
                result.setCurrencyFrom("原币种");
                result.setCurrencyTo("目标币种");
                items.add(result);
            }
        String a=JSON.toJSONString(items);
        System.out.println(a);
    }

    @Test
    public void testConvertPeriod(){
        System.out.println(convertPeriod(201813));
        System.out.println(convertPeriod(201814));
        System.out.println(convertPeriod(201812));
        System.out.println(convertPeriod(201810));
        System.out.println(convertPeriod(201811));

    }
    private Integer convertPeriod(Integer period){
        if(period%100==13){
            period--;
        }
        return period;
    }

    private static class CallEbsThread implements Callable {

        protected final Logger logger = LoggerFactory.getLogger(this.getClass());

        private int type;

        private Organization org;

        private String period;

        private String ebsCallUrl;

        private String effectiveDateFrom;

        private String effectiveDateTo;

        private DataImportLogMapper dataImportLogMapper;

        @Autowired
        private AuthUserHelper authUserHelper;

        @Autowired
        private DistributedIdService idService;

        CallEbsThread(int type, Organization org,String ebsCallUrl,DataImportLogMapper dataImportLogMapper,
                      String period, String effectiveDateFrom, String effectiveDateTo) {
            this.type = type;
            this.org = org;
            this.ebsCallUrl = ebsCallUrl;
            this.dataImportLogMapper = dataImportLogMapper;
            this.period = period;
            this.effectiveDateFrom = effectiveDateFrom;
            this.effectiveDateTo = effectiveDateTo;
        }

        @Override
        public Object call(){
            String response = null;
            String ledgerId = org.getEnterpriseAccountCode();
            String code = org.getCode();
            Integer periodInt = DateUtils.strToPeriod(period);
            int year = periodInt / 100;
            int month = periodInt % 100;
            DataImportLog log = new DataImportLog();
            log.setTaxpayerIdNum(org.getTaxPayerNumber());
            log.setOrganizationId(org.getId());
            log.setCompanyCode(code);
            log.setCompanyName(org.getName());
            log.setPeriodYear(year);
            log.setPeriodMonth(month);
            log.setType(EnumImportType.ExtractFinancialData.getCode());
            log.setFileType(EnumEbsExtractType.getNameByCode(type));
            log.setPeriodStatus("EBS抽取");
            log.setOperator("admin");
            try {
                switch (type) {
                    case EbsExtractTypeConstant.TB:
                        response = HttpUtil.post(ebsCallUrl + "/glMonthlyBal?ledgerId="+ledgerId+"&companyCode="+ code + "&period=" + period,
                                "","application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.JE:
                        // 这里BA反馈可按期间获取当月日记账即可
                        String effecDateFrom = DateUtils.getFirstDayOfMonth(year,month);
                        String effecDateTo = DateUtils.getLastDayOfMonth(year,month);
                        response = HttpUtil.post(ebsCallUrl + "/glJeLines"+"?ledgerId=" + ledgerId + "&companyCode=" + code + "&effectiveDateFrom=" + effecDateFrom+"&effectiveDateTo="+effecDateTo,
                                "","application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.BSPRC:
                        response = HttpUtil.post(ebsCallUrl + "/fsgAsset"+"?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period+"&prcFlag=Y",
                                "",
                                "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.PLPRC:
                        response = HttpUtil.post(ebsCallUrl + "/fsgProfit"+"?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period+"&prcFlag=Y",
                                "","application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.BS:
                        response = HttpUtil.post(ebsCallUrl + "/fsgAsset"+"?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period+"&prcFlag=N",
                                "","application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.PL:
                        response = HttpUtil.post(ebsCallUrl + "/fsgProfit"+"?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period+"&prcFlag=N",
                                "","application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.CF:
                        response = HttpUtil.post(ebsCallUrl + "/fsgCash"+"?ledgerId=" + ledgerId + "&companyCode=" + code + "&period=" + period,
                                "","application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    case EbsExtractTypeConstant.OCTB:
                        break;
                    case EbsExtractTypeConstant.RATE:
                        response = HttpUtil.post(ebsCallUrl + "/fsgCash"+"?dailyRates=" + period,
                                "",
                                "application/json;charset=utf-8", "UTF-8", 10000, 10000);
                        break;
                    default:
                        break;
                }
                System.out.println("call"+EnumEbsExtractType.getNameByCode(type)+":"+response);
                if (StringUtils.isNotEmpty(response)) {
                    if(EbsExtractTypeConstant.RATE==type){
                        EbsCallRateResp resp = JSON.parseObject(response, EbsCallRateResp.class);
                        log.setId(idService.nextId());
                        log.setImportResult(true);
                        log.setErrorMsg(String.format(EnumApiCodeMsg.CALLSUCCESS.getMsg(),resp.getRequestStatus()));
                    }else{
                        EbsCallResp resp = JSON.parseObject(response, EbsCallResp.class);
                        long taskId = Long.valueOf(resp.getTaskId());
                        if (taskId == -1) {
                            log.setId(idService.nextId());
                            log.setImportResult(false);
                            log.setErrorMsg(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), resp.getTaskDesc()));
                        } else {
                            log.setId(taskId);
                            log.setImportResult(true);
                            log.setErrorMsg(String.format(EnumApiCodeMsg.CALLSUCCESS.getMsg(), resp.getTaskDesc()));
                        }
                    }
                    dataImportLogMapper.insertSelective(log);
                }
            } catch (Exception e) {
                log.setId(idService.nextId());
                log.setImportResult(false);
                log.setErrorMsg(String.format(EnumApiCodeMsg.CALLFAILED.getMsg(), e.getMessage()));
                dataImportLogMapper.insertSelective(log);
                e.printStackTrace();
            }
            return new Integer(1);
        }
    }


    public String encryptInput(){
        byte[] encodeData;
        try {
            byte[] publicKey = org.apache.commons.codec.binary.Base64.decodeBase64("MFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBAKUfMPRKV6I5num1dDWcxTrgTjXf5LctsVj0CpbwHE83mmjUO5CAlvA0Fwy30ajCX5sLmsyi+Eu/4uNmM6GQF3kCAwEAAQ==");
            String nonce = generateNonce();
            String sDate = generateDate();
            long rNum = generateRandomNumber();
            String inputStr1 = sDate + "@@" + "DTS" + "@@" + nonce+rNum;
            byte[] data1 = inputStr1.getBytes();
            logger.debug("原文：" + inputStr1);
            encodeData = encryptByPublicKey(data1, publicKey);
        } catch (Exception ex) {
            throw new ServiceException("cus" + ex);
        }

        return org.apache.commons.codec.binary.Base64.encodeBase64String(encodeData);
    }



    private String generateNonce() throws Exception {
        String dateTimeString = Long.toString(System.currentTimeMillis());
        byte[] nonceByte = dateTimeString.getBytes();
        return Base64.getEncoder().encodeToString(nonceByte);
    }

    private String generateDate() throws Exception {
        DateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
        dateFormatter.setTimeZone(TimeZone.getTimeZone("UTC"));
        Date today = Calendar.getInstance().getTime();
        String created = dateFormatter.format(today);

        return created;
    }


    private long generateRandomNumber() throws Exception {
        final double d = Math.random();
        final long num = (int)(d*100000000000000L);
        return num;
    }



    private static byte[] encryptByPublicKey(byte[] data, byte[] key) throws Exception {
        X509EncodedKeySpec x509KeySpec = new X509EncodedKeySpec(key);
        KeyFactory keyFactory = KeyFactory.getInstance("RSA");
        PublicKey publicKey = keyFactory.generatePublic(x509KeySpec);
        Cipher cipher = Cipher.getInstance(keyFactory.getAlgorithm());
        cipher.init(Cipher.ENCRYPT_MODE, publicKey);
        return cipher.doFinal(data);
    }

    public static void main(String[] args) {
        System.out.println("--------------");
        System.out.println(new EbsApiServiceImplTest().encryptInput());
    }
}