package pwc.taxtech.atms.web.controller;

import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.nutz.lang.Times;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.UUID;

@Component
public class JwtUtil {
    private static final Logger logger = LoggerFactory.getLogger(JwtUtil.class);

    @Value("${jwt.expireSecond}")
    private Integer jwtExpireSecond;

    @Value("${jwt.base64Secret}")
    private String jwtBase64Secret;

    /***
     * @param username
     *            登录名,大小写不限，可以是全大写或全小写,如:admin, ADMIN
     * @param databaseUsername
     *            数据库用户名, 比如Admin
     * @param userid
     *            用户Id
     * @return
     */
    public String generateToken(String username, String databaseUsername, String userid) {
        Date now = new Date();
        // 过期时间设置为2天
        int expireSecond = jwtExpireSecond;
        Date expiration = Times.nextSecond(now, expireSecond);
        JwtBuilder jwtBuilder = Jwts.builder();
        // 设置Subject为登录用户名
        jwtBuilder.setSubject(username);
        jwtBuilder.setExpiration(expiration);
        jwtBuilder.setIssuedAt(now);
        // 设置时钟误差偏移量，即10分钟
        Date notBefore = Times.nextSecond(now, -600);
        jwtBuilder.setNotBefore(notBefore);
        jwtBuilder.setId(getUUID());
        jwtBuilder.claim("username", username);
        jwtBuilder.claim("databaseUsername", databaseUsername);
        jwtBuilder.claim("userid", userid);
        // 设置body.username为数据库用户名
        jwtBuilder.signWith(SignatureAlgorithm.HS512, jwtBase64Secret);
        return jwtBuilder.compact();
    }

    public static String getUUID() {
        return UUID.randomUUID().toString().toUpperCase();
    }

}