﻿vatModule.controller('citPreviewJournalMergeController', ['$rootScope','$scope', '$log', '$filter','$translate', '$timeout', 'SweetAlert', '$q', 'uiGridConstants', '$interval', 'citPreviewService', 'browserService', 'vatSessionService', 'region', 'enums', 'vatExportService',
    function ($rootScope,$scope, $log,$filter, $translate, $timeout, SweetAlert, $q, uiGridConstants, $interval, citPreviewService, browserService, vatSessionService, region, enums, vatExportService) {
        'use strict';

        $scope.startDate = new Date(vatSessionService.project.year, 0, 1);
        $scope.endDate = new Date(vatSessionService.project.year, 11, 31);
        $scope.dateFormat = $translate.instant('dateFormat4YearMonthDay');
        $scope.startMonth = 1;
        $scope.endMonth = 12;
        $scope.totalMoneyAmount = 0;
        $scope.totalTaxAmount = 0;

        var minDate = [1, vatSessionService.project.year];
        // var minDate = moment().startOf('month').subtract(0, 'months');
        var maxDate = [12, vatSessionService.project.year];
        var setDate = [
            [1, vatSessionService.project.year],
            [12, vatSessionService.project.year]];

        $scope.monthList = [$translate.instant('Month01'),
            $translate.instant('Month02'),
            $translate.instant('Month03'),
            $translate.instant('Month04'),
            $translate.instant('Month05'),
            $translate.instant('Month06'),
            $translate.instant('Month07'),
            $translate.instant('Month08'),
            $translate.instant('Month09'),
            $translate.instant('Month10'),
            $translate.instant('Month11'),
            $translate.instant('Month12')
        ];

        //初始化期间
            var initPeriods = function () {
            var curMonth = new Date().getMonth() + 1;

            $scope.queryParams = {
                pageInfo: {},
                period:  vatSessionService.year,
                periodStart: '',
                periodEnd: '',
                subjectCode: null,
                subjectName: null,
                description: null,
                orgCode: null,
                orgName: null,
                documentDate: null,
                projectId: vatSessionService.project.id
            };
        };

        //从数据库中load数据
        var loadJournalEntryDataFromDB = function () {
            //初始化查询信息
            $scope.queryParams.pageInfo = {
                totalCount: $scope.pagingOptions.totalItems,
                pageIndex: $scope.pagingOptions.pageIndex,
                pageSize: $scope.pagingOptions.pageSize,
                totalPage: 0
            };
            $scope.getDataFromDatabase($scope.queryParams);
        };

        $scope.getDataFromDatabase = function (queryParams){
            citPreviewService.getJournalMerges(queryParams).success(function (data) {
                if (data) {
                    // minDate = data.
                    var index = 1;
                    data.list.forEach(function (v) {
                        v.index = index++;
                    });
                    $scope.gridDataSource = data.list;

                    $scope.pagingOptions.totalItems = data.total;
                }
            });
        };

        //将选择了的查询条件显示在grid上方
        var doDataFilter = function (removeData) {
            if ($scope.queryParams.periodStart > $scope.queryParams.periodEnd) {
                $scope.queryParams.periodEnd = $scope.queryParams.periodStart;
            }

            loadJournalEntryDataFromDB();
            $('.filter-button').popover("hide");
        };

        //去掉所有的查询条件，重新load数据
        var doDataFilterReset = function () {
            $scope.queryParams = {
                pageInfo: {},
                period:  vatSessionService.year,
                periodStart: '',
                periodEnd: '',
                subjectCode: null,
                subjectName: null,
                description: null,
                orgCode: null,
                orgName: null,
                documentDate: null,
                projectId: vatSessionService.project.id
            };
            $scope.queryParams.periodStart = vatSessionService.year * 100 + 1;
            $scope.queryParams.periodEnd = vatSessionService.year * 100 + 12;
            loadJournalEntryDataFromDB();
            $('.filter-button').popover("hide");
        };

        var prepareSummary = function () {
            // do something before show popover
        };

        //在popover打开时执行事件
        var showPopover = function () {
            $timeout(function () {
                initDatePickers();
            }, 500);
        };

        //初始化时间控件
        var initDatePickers = function () {
            //认证开始时间
            var ele1 = $("#certificationDateStart");
            ele1.datepicker({
                startDate: $scope.startDate,
                endDate: $scope.endDate,
                language: region,
                autoclose: true,//选中之后自动隐藏日期选择框
                clearBtn: true,//清除按钮
                todayBtn: false,//今日按钮
                format: $scope.dateFormat//日期格式，详见 http://bootstrap-datepicker.readthedocs.org/en/release/options.html#format
            });
            ele1.datepicker("setDate", $scope.queryParams.certificationDateStart);
            //认证结束时间
            var ele2 = $("#certificationDateEnd");
            ele2.datepicker({
                startDate: $scope.startDate,
                endDate: $scope.endDate,
                language: region,
                autoclose: true,//选中之后自动隐藏日期选择框
                clearBtn: true,//清除按钮
                todayBtn: false,//今日按钮
                format: $scope.dateFormat//日期格式，详见 http://bootstrap-datepicker.readthedocs.org/en/release/options.html#format
            })
            ele2.datepicker("setDate", $scope.queryParams.certificationDateEnd);
        };

        var downloadJE = function () {
            citPreviewService.initExportJMData($scope.queryParams).success(function (data, status, headers) {
                if(status===204){
                    SweetAlert.warning("没有数据可以下载");
                    return;
                }
                vatExportService.exportToExcel(data, status, headers, '日记账信息.xlsx');
            }).error(function () {
                SweetAlert.error($translate.instant('PleaseContactAdministrator'));
            });
        };

        var handleJournal = function () {
            citPreviewService.generateTb(vatSessionService.project.id).success(function (resp) {
                if (resp.result) {
                    SweetAlert.success($translate.instant('ProcessSuccess'));
                }else {

                    if (resp.resultMsg && resp.resultMsg.length > 0) {
                        SweetAlert.warning(resp.resultMsg);
                    }else{
                        SweetAlert.error($translate.instant('ProcessFail'));
                    }
                }
            }).error(function () {
                SweetAlert.error($translate.instant('PleaseContactAdministrator'));
            });
        };
        $scope.refreshGrid = function () {
            $log.debug("refreshGrid");
            loadJournalEntryDataFromDB();
        };

        (function initialize() {
            $log.debug('VatPreviewInputInvoiceController.ctor()...');

            //分页的设置
            $scope.pagingOptions = {
                pageIndex: 1,  //当前页码
                totalItems: 0,  //总数据
                pageSize: 50,  //每页多少条数据
                pageSizeString:"50"
            };

            $('#input-invoice-period-picker').focus(function () {
                $('.filter-button').popover("hide");
            });
                //初始化month-picker
            $('#input-invoice-period-picker').rangePicker({
                minDate: minDate,
                maxDate: maxDate,
                setDate: setDate,
                months: $scope.monthList,//['1月', '2月', '3月', '4月', '5月', '6月', '7月', '8月', '9月', '10月', '11月', '12月'],
                ConfirmBtnText: $translate.instant('Confirm'),
                CancelBtnText: $translate.instant('ButtonCancel')
            })
                .on('datePicker.done', function (e, result) {
                    //开始月份
                    var startMonth = result[0][1] * 100 + result[0][0];
                    //结束月份
                    var endMonth = result[1][1] * 100 + result[1][0];
                    $scope.startMonth = startMonth;
                    $scope.endMonth = endMonth;

                    $scope.queryParams.periodStart = startMonth;
                    $scope.queryParams.periodEnd = endMonth;
                    loadJournalEntryDataFromDB();
                });

            $scope.gridOptions = {
                bindingOptions: {
                    dataSource: 'gridDataSource'
                },
                columns: [
                    {caption: $translate.instant('ApprovalStatus'),dataField: "approvalStatus",width: 120},
                    {caption: $translate.instant('Posting'), dataField: "postedStatus",width: 120},
                    {caption: $translate.instant('AccountingPeriod'), dataField: "accountPeriod",width: 80},
                    {caption: $translate.instant('DocumentDate'), dataField: "accountingDate", dataType: "date", format: "yyyy-MM-dd",width: 100},
                    {caption: $translate.instant('JournalSource'), dataField: "journalSource",width: 200},
                    {caption: $translate.instant('JournalCategory'), dataField: "category",width: 200},
                    {caption: $translate.instant('JournalName'), dataField: "name",width: 200},
                    {caption: $translate.instant('DocumentNo'), dataField: "voucherNum",width: 200},
                    {caption: $translate.instant('Summary'), dataField: "description",width: 200},
                    {caption: $translate.instant('MainBodyCode'), dataField: "orgCode",width: 200},
                    {caption: $translate.instant('costCenter'), dataField: "segment2",width: 200},
                    {caption: $translate.instant('AccountCode'), dataField: "subjectCode",width: 200},
                    {caption: $translate.instant('AuxiliarySubject'), dataField: "segment4",width: 200},
                    {caption: $translate.instant('ProfitCenter'), dataField: "segment5",width: 200},
                    {caption: $translate.instant('Product'), dataField: "segment6",width: 200},
                    {caption: $translate.instant('ProjectName'), dataField: "segment7",width: 200},
                    {caption: $translate.instant('BetweenCompanies'), dataField: "segment8",width: 200},
                    {caption: $translate.instant('Alternate1'), dataField: "segment9",width: 200},
                    {caption: $translate.instant('Alternate2'), dataField: "segment10",width: 200},

                    {caption: $translate.instant('MainBodyDescription'), dataField: "orgName",width: 200},
                    {caption: $translate.instant('CostCenterDescription'), dataField: "segment2Name",width: 200},
                    {caption: $translate.instant('SubjectDescription'), dataField: "subjectName",width: 200},
                    {caption: $translate.instant('AuxiliaryAccountDescription'), dataField: "segment4Name",width: 200},
                    {caption: $translate.instant('ProfitCenterDescription'), dataField: "segment5Name",width: 200},
                    {caption: $translate.instant('ProductManual'), dataField: "segment6Name",width: 200},
                    {caption: $translate.instant('ProjectInstruction'), dataField: "segment7Name",width: 200},
                    {caption: $translate.instant('InterCompanyDescription'), dataField: "segment8Name",width: 200},
                    {caption: $translate.instant('Alternate1Description'), dataField: "segment9Name",width: 200},
                    {caption: $translate.instant('Alternate2Description'), dataField: "segment10Name",width: 200},
                    {caption: $translate.instant('currency'), dataField: "journalCurrencyCode",width: 200},
                    {caption: $translate.instant('LocalCurrency'), dataField: "sobCurrencyCode",width: 200},
                    {
                        caption: $translate.instant('JournalDebitAmount'),
                        dataField: "accountedDr",
                        format: {type: 'fixedPoint', precision: 2},
                        width: 100
                    },
                    {
                        caption: $translate.instant('JournalCreditAmount'),
                        dataField: "accountedCr",
                        format: {type: 'fixedPoint', precision: 2},
                        width: 100
                    },
                    {
                        caption: $translate.instant('LocalCurrencyDebitAmount'),
                        dataField: "enteredDr",
                        format: {type: 'fixedPoint', precision: 2},
                        width: 100
                    },
                    {
                        caption: $translate.instant('LocalCurrencyCreditAmount'),
                        dataField: "enteredCr",
                        format: {type: 'fixedPoint', precision: 2},
                        width: 100
                    },
                    {caption: $translate.instant('CashFlowEntry'), dataField: "cfItem",width: 200},
                    {caption: $translate.instant('City'), dataField: "attribute1",width: 200},
                    {
                        caption: $translate.instant('TransactionDate'),
                        dataField: "attribute2",
                        dataType: "date",
                        format: "yyyy-MM-dd",
                        width: 100
                    },
                    {caption: $translate.instant('BankAccountNumber'), dataField: "attribute3",width: 200},
                    {caption: $translate.instant('BankSerialNumber'), dataField: "attribute4",width: 200},
                    {caption: $translate.instant('SupplierCode'), dataField: "attribute5",width: 200},
                    {caption: $translate.instant('TransactionOrderNumber'), dataField: "attribute6",width: 200},
                    {caption: $translate.instant('SupplierName'), dataField: "attribute7",width: 200},
                    {caption: $translate.instant('ReceiveCode'), dataField: "attribute8",width: 200},
                    {caption: $translate.instant('PreparedBy'), dataField: "attribute9",width: 200},
                    {caption: $translate.instant('Reviewer'), dataField: "attribute10",width: 120},
                    {caption: $translate.instant('CostCenterDepartmentDescription1'), dataField: "attribute11",width: 120},
                    {caption: $translate.instant('CostCenterDepartmentDescription2'), dataField: "attribute12",width: 120},
                    {caption: $translate.instant('CostCenterDepartmentDescription3'), dataField: "attribute13",width: 120},
                    {caption: $translate.instant('CostCenterDepartmentDescription4'), dataField: "attribute14",width: 120},
                    {caption: $translate.instant('CostCenterDepartmentDescription5'), dataField: "attribute15",width: 120},
                    {caption: $translate.instant('CostCenterDepartmentDescription6'), dataField: "attribute16",width: 120},
                    {caption: $translate.instant('importWay'), dataField: "source",width: 120}

                ],
                // onContentReady: function (e) {
                //     $scope.extractFinancialInstance = e.component;
                //     var totalCount = e.component.totalCount();
                //     if (totalCount > 0) {
                //         $scope.totalCount = totalCount;
                //     }
                // },
                loadPanel: {
                    enabled: true
                },
                selection: {
                    mode: "single"
                },
                paging: {
                    pageSize: 100,
                },
                showBorders: true,
                hoverStateEnabled: true,
                height: '99%',
                noDataText: $translate.instant('NoDataText'),
            };

            $scope.doDataFilter = doDataFilter;
            $scope.doDataFilterReset = doDataFilterReset;
            $scope.prepareSummary = prepareSummary;
            $scope.showPopover = showPopover;
            $scope.downloadJE = downloadJE;
            $scope.handleJournal = handleJournal;

            initPeriods();
            //初始化查询条件-期间范围
            $scope.queryParams.period = vatSessionService.year;
            $scope.queryParams.periodStart = vatSessionService.year * 100 + 1;
            $scope.queryParams.periodEnd = vatSessionService.year * 100 + 12;
            $scope.queryParams.organizationId = vatSessionService.project.organizationID;
            // if($rootScope.currentLanguage === 'en-us'){
            //     $('.periodInput')[0].style.left = "280px";
            // }else{
            //     $('.periodInput')[0].style.left = "250px";
            // }
            loadJournalEntryDataFromDB();
        })();
    }
]);




