﻿commonModule.
controller('editOrganizationStructureModalController', ['$scope', '$log', 'SweetAlert', '$translate', 'organizationStructureService', 'uiGridConstants', 'Upload', 'apiInterceptor', '$interval', '$q',
    function ($scope, $log, SweetAlert, $translate, organizationStructureService, uiGridConstants, Upload, apiInterceptor, $interval, $q) {

        var editModalSelector = "#org-structure-modal";
        $scope.editingObject = { id: "", name: "", IsActive: true };
        //显示指定消息编码对应的翻译文本信息
        var showError = function (messageCode) {
            var errMsg = $translate.instant(messageCode);
            SweetAlert.warning(errMsg);
        };

        var resetError = function () {
            var elms = $(editModalSelector).find("form:first div[data-for]");
            elms.html("");
            elms.attr("class", "validate-success");

            $scope.editingObject = { ID: "", name: "", IsActive: true };
        };

        //获取机构层级数据
        var getOrgStructureList = function () {
            var deferred = $q.defer();
            organizationStructureService.getOrganizationStructureList().success(function (data) {
         
                $scope.organizationStructureList = data;
                deferred.resolve();
            });
            return deferred.promise;
        };

        var editRow = function ()
        {
            getOrgStructureList().then(function () {
                $scope.isEdit = true;
                if ($scope.selectedId) {

                    var data = _.find($scope.organizationStructureList, function (num) {
                        return num.id === $scope.selectedId;
                    });
                    if (data) {
                        $scope.editingObject = { id: data.id, name: data.name, IsActive: data.isActive };
                    }
                }
                $(editModalSelector).modal('show');
            });
        };

        var addRow = function () {
            getOrgStructureList().then(function () {
                $scope.isEdit = false;
                $scope.editingObject = { id: "", name: "", IsActive: true };
                $(editModalSelector).modal('show');
            });
        };
        $scope.$watch('operateType', function (newValue, oldValue) {
            if (newValue) {
                resetError();
                if (newValue == constant.Operation.Add) {
                    addRow();
            
                } else if (newValue == constant.Operation.Edit) {
                    editRow();
                }
            }
        });

        $(editModalSelector).on("hidden.bs.modal", function () {
            $scope.operateType = null;
        });
        $scope.closeModal = function () {
            $scope.operateType = null;
        };

        //保存当前正在编辑数据行,经过界面初步校验后才会进入本功能
        $scope.save = function () {
            if ($scope.editingObject == null) {
                showError("OrganizationStructureNoSelected");
                return;
            }

            if ($scope.editingObject && $scope.editingObject.name.trim().length == 0) {
                showError("OrganizationStructureEmptyNode");
                return;
            }

            if (!($(editModalSelector).find("#edit-form").valid())) {
                return;
            }

            var osArray = [];
            osArray.push($scope.editingObject);

            var successedFun = function (rst) {

                if ($scope.isEdit) {
                    SweetAlert.success($translate.instant("OrganizationStructureEditSuccess"));
                }
                else {
                    SweetAlert.success($translate.instant("OrganizationStructureAddSuccess"));
                }
                $scope.isUpdate = true;
                $scope.operateType = null;
                $scope.refreshData();

                $(editModalSelector).modal('hide');
            }

            if ($scope.isEdit) {
                organizationStructureService.updateOrganizationStructure(osArray).success(successedFun);
            } else {
                organizationStructureService.addOrganizationStructure(osArray).success(successedFun);
            }

        };


        var intiValidate = function () {

            $.validator.addMethod("OrgStructureRepeated", function (value, element, param) {
                if ($scope.organizationStructureList && $scope.organizationStructureList.length > 0) {
                    var objects = $.grep($scope.organizationStructureList, function (n, i) {
                        return $scope.editingObject.id != n.id && $.trim(n.name) == $.trim(value);
                    });

                    return objects.length < 1;
                }

                return true;

            });


            formValidator = $(editModalSelector).find("#edit-form").validate({
                debug: true,

                rules: {
                    name: {
                        required: true,
                        minlength: 2,
                        maxlength: 50,
                        OrgStructureRepeated: true
                    },
                    IsActive: {
                        required: "input[name='IsActive']:checked"
                    }
                },
                messages: {
                    name: {
                        required: $translate.instant('OrganizationStructureEmptyNode'),
                        minlength: $translate.instant('OrganizationStructureEmptyNode'),
                        maxlength: $translate.instant('OrganizationStructureOutOfLengthNode'),
                        OrgStructureRepeated: $translate.instant('OrganizationStructureDuplicateNode')
                    },
                    IsActive: $translate.instant('OrganizationStructureStatusUnsureness')
                },
                errorPlacement: function (error, element) {
                    var elm = $(element).parents("form:first");
                    elm = elm.find("div[data-for='" + element.attr("name") + "']");
                    elm.html(error);
                    elm.attr("class", "validate-fail");
                }
            });
        };


        (function initialize() {
            $log.debug('editOrganizationStructureModalController.ctor()...');
            $scope.refreshData = getOrgStructureList;
            intiValidate();
        })();
    }
]);
