﻿commonModule.controller('vatReportUploadDetailController', ['$scope', '$log', '$translate', '$timeout', 'SweetAlert', '$q',
    '$interval', 'dxDataGridService', '$http', 'apiConfig', 'vatSessionService', 'Upload', 'apiInterceptor',
    function ($scope, $log, $translate, $timeout, SweetAlert, $q, $interval, dxDataGridService, $http, apiConfig, vatSessionService, Upload, apiInterceptor) {
        'use strict';
        $log.debug('vatReportUploadDetailController.ctor()...');

        //表格配置
        $scope.reportOneGridOptions = {
            selection: {
                mode: "single"
            },
            hoverStateEnabled: true,
            showBorders: true,
            columns: [
                {
                    dataField: 'serialNo',
                    caption: $translate.instant('RepUploadDtlColSerialNo'),
                    fixed: true,
                    allowHeaderFiltering: true, cellTemplate: function (container, options) {
                        try {
                            $("<span>" + (options.rowIndex + 1) + "</span>")
                                .appendTo(container);
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
                {
                    dataField: 'createTime',
                    caption: $translate.instant('RepUploadDtlColCreateTime'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    dataType: "date",
                    format: "yyyy-MM-dd HH:mm:ss",
                },
                {
                    dataField: 'creator',
                    caption: $translate.instant('RepUploadDtlColCreator'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'reportName',
                    caption: $translate.instant('RepUploadDtlColFileName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'fileUrl',
                    caption: $translate.instant('RepUploadDtlColDownload'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    cellTemplate: function (container, options) {
                        try {
                            $("<a href='" + options.data.fileUrl + "'>下载</a>")
                                .appendTo(container);
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                }
            ],
            bindingOptions: {
                dataSource: 'reportOneDataSource'
            }
        };


        $scope.reportTwoGridOptions = {
            selection: {
                mode: "single"
            },
            hoverStateEnabled: true,
            showBorders: true,
            columns: [
                {
                    caption: $translate.instant('RepUploadDtlColSerialNo'),
                    fixed: true,
                    width: '5%',
                    allowHeaderFiltering: true, cellTemplate: function (container, options) {
                        try {
                            $("<span>" + (options.rowIndex + 1) + "</span>")
                                .appendTo(container);
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
                {
                    dataField: 'reportTypeStr',
                    width: '20%',
                    caption: $translate.instant('RepUploadDtlColReportType'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'createTime',
                    width: '10%',
                    caption: $translate.instant('RepUploadDtlColCreateTime'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    dataType: "date",
                    format: "yyyy-MM-dd HH:mm:ss",
                },
                {
                    dataField: 'creator',
                    width: '10%',
                    caption: $translate.instant('RepUploadDtlColCreator'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'reportFileName',
                    width: '30%',
                    caption: $translate.instant('RepUploadDtlColFileName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    caption: $translate.instant('RepUploadDtlColUpload'),
                    fixed: true,
                    width: '8%',
                    allowHeaderFiltering: true,
                    cellTemplate: function (container, options) {
                        try {
                            if (!options.data.sourceType || "VAT" == options.data.sourceType) {
                                $("<a href='javascript:void(0)'>上传</a>").on('click', function () {
                                    $('#uploadDiv' + options.data.reportType).click(function(event) {
                                        event.stopPropagation();
                                    });
                                    $('#uploadBut' + options.data.reportType).click();
                                }).appendTo(container);
                                $('<div id="uploadDiv' + options.data.reportType + '"><input type="file"  style="display:none;"  id="uploadBut' + options.data.reportType + '" />').on('change', function () {
                                    uploadFile($('#uploadBut' + options.data.reportType)[0].files[0], options.data.reportType);
                                }).appendTo(container);
                            }
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
                {
                    dataField: 'fileUrl',
                    width: '8%',
                    caption: $translate.instant('RepUploadDtlColDownload'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    cellTemplate: function (container, options) {
                        if (options.data.fileUrl && "" != options.data.fileUrl) {
                            try {
                                $("<a href='" + options.data.fileUrl + "'>下载</a>")
                                    .appendTo(container);
                            } catch (e) {
                                $log.error(e);
                            }
                        }
                    }
                },
                {
                    dataField: 'sourceTypeStr',
                    width: '8%',
                    caption: $translate.instant('RepUploadDtlColSourceType'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
            ],
            bindingOptions: {
                dataSource: 'reportTwoDataSource'
            }
        };


        $scope.reportThreeGridOptions =  {
            selection: {
                mode: "single"
            },
            hoverStateEnabled: true,
            showBorders: true,
            columns: [
                {
                    caption: $translate.instant('RepUploadDtlColSerialNo'),
                    fixed: true,
                    width: '5%',
                    allowHeaderFiltering: true, cellTemplate: function (container, options) {
                        try {
                            $("<span>" + (options.rowIndex + 1) + "</span>")
                                .appendTo(container);
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
                {
                    dataField: 'reportTypeStr',
                    width: '20%',
                    caption: $translate.instant('RepUploadDtlColTaxType'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'createTime',
                    width: '10%',
                    caption: $translate.instant('RepUploadDtlColCreateTime'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    dataType: "date",
                    format: "yyyy-MM-dd HH:mm:ss",
                },
                {
                    dataField: 'creator',
                    width: '10%',
                    caption: $translate.instant('RepUploadDtlColCreator'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'reportFileName',
                    width: '30%',
                    caption: $translate.instant('RepUploadDtlColFileName'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    caption: $translate.instant('RepUploadDtlColUpload'),
                    fixed: true,
                    width: '8%',
                    allowHeaderFiltering: true,
                    cellTemplate: function (container, options) {
                        try {
                            if (!options.data.sourceType || "VAT" == options.data.sourceType) {
                                $("<a href='javascript:void(0)'>上传</a>").on('click', function () {
                                    $('#uploadDiv' + options.data.reportType).click(function(event) {
                                        event.stopPropagation();
                                    });
                                    $('#uploadBut' + options.data.reportType).click();
                                }).appendTo(container);
                                $('<div id="uploadDiv' + options.data.reportType + '"><input type="file"  style="display:none;"  id="uploadBut' + options.data.reportType + '" />').on('change', function () {
                                    uploadFile($('#uploadBut' + options.data.reportType)[0].files[0], options.data.reportType);
                                }).appendTo(container);
                            }
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
                {
                    dataField: 'fileUrl',
                    width: '8%',
                    caption: $translate.instant('RepUploadDtlColDownload'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    cellTemplate: function (container, options) {
                        if (options.data.fileUrl && "" != options.data.fileUrl) {
                            try {
                                $("<a href='" + options.data.fileUrl + "'>下载</a>")
                                    .appendTo(container);
                            } catch (e) {
                                $log.error(e);
                            }
                        }
                    }
                },
                {
                    dataField: 'sourceTypeStr',
                    width: '8%',
                    caption: $translate.instant('RepUploadDtlColSourceType'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
            ],
            bindingOptions: {
                dataSource: 'reportThreeDataSource'
            }
        };

        var uploadFile = function (file, reportType) {
            var month = vatSessionService.month < 10 ? ("0" + vatSessionService.month) : vatSessionService.month;
            Upload.upload({
                url: apiInterceptor.webApiHostUrl + '/reportUploadDetail/saveFileUpload',
                data: {
                    projectId: vatSessionService.project.id,
                    period: "" + vatSessionService.year + month,
                    reportType: reportType,
                    sourceType: "VAT"
                },
                file: file,
                headers: {
                    'Access-Control-Allow-Origin': '*',
                    Authorization: apiInterceptor.tokenType + ' ' + apiInterceptor.apiToken(),
                    withCredentials: true
                },
                withCredentials: true
            }).then(function (resp) {
                $scope.getReportFileUpload();
            }, function (resp) {

            }, function (evt) {

            });
        }
        //刷新页面
        $scope.refreshGrid = function () {
            $scope.searchParam.projectId = vatSessionService.project.id;
            var month = vatSessionService.month < 10 ? ("0" + vatSessionService.month) : vatSessionService.month;
            $scope.searchParam.period = "" + vatSessionService.year + month;
            $scope.searchParam.pageInfo = {
                pageIndex: $scope.pagingOptions.pageIndex,
                totalItems: $scope.pagingOptions.totalItems,
                pageSize: $scope.pagingOptions.pageSize
            };
            $http.post('/reportUploadDetail/queryPage', $scope.searchParam, apiConfig.createVat())
                .success(function (res) {
                    if (res && res.list) {
                        $scope.reportOneDataSource = res.list;
                        $scope.pagingOptions.totalItems = res.pageInfo.totalCount;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };

        //获取申报表或税票上传数据
        $scope.getReportFileUpload = function () {
            var month = vatSessionService.month < 10 ? ("0" + vatSessionService.month) : vatSessionService.month;
            $http.post('/reportUploadDetail/queryFileUpload', {
                projectId: vatSessionService.project.id,
                period: "" + vatSessionService.year + month
            }, apiConfig.createVat())
                .success(function (res) {
                    if (res) {
                        $scope.reportTwoDataSource = [];
                        $scope.reportThreeDataSource = [];
                        angular.forEach(res, function (data) {
                            if ("增值税纳税申报表" === data.reportType || "增值税更正申报表" === data.reportType) {
                                $scope.reportTwoDataSource.push(data);
                            }
                            if ("增值税纳税税票" === data.reportType || "增值税更正税票" === data.reportType) {
                                $scope.reportThreeDataSource.push(data);
                            }
                        });
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };


        (function initialize() {

            //分页的设置
            $scope.pagingOptions = {
                pageIndex: 1,  //当前页码
                totalItems: 0,  //总数据
                pageSize: 10,  //每页多少条数据
                pagingSelection: [{id: 5, value: 5}, {id: 10, value: 10}],
                pageSizeString: '10',
            };

            $scope.searchParam = {};

            function init() {
                $scope.refreshGrid();
                $scope.getReportFileUpload();

            }

            init();

        })();
    }
]);