﻿frameworkModule.directive("dashboardTaxCashFlowAnalysis", ['$translate', '$filter', 'region', function ($translate, $filter, region) {
    'use strict';

    return {
        restrict: "A",
        templateUrl: "/app/dashboard/tax-cash-flow-analysis/tax-cash-flow-analysis.html",
        replace: true,
        scope: {
            options: "=dashboardTaxCashFlowAnalysis"
        },
        controller: "dashboardTaxCashFlowAnalysisController",
        link: function ($scope, $element, $attr) {
            var colourArray = ['#a32020', '#eb8c00', '#db536a', '#dc6900', '#e0301e', '#602320', '#3f3f40', '#968c6d'];

            var chartsContainer = $($element).find(".charts-container");
            var cashFlowChart = echarts.init(chartsContainer[0], 'vintage');

            //用于使echart自适应高度和宽度,通过窗体高宽计算容器高宽
            var resizeEchart = function (width) {
                width = width || $element.width();

                // 如果渲染页面时该控件处于隐藏状态，则$element.width()会为0，此时不去重新计算
                if (width > 0) {
                    $element.find('.charts-container').width(width + 'px');
                    $element.find('.charts-container').show();
                }
                else {
                    $element.find('.charts-container').hide();
                }
            };

            var resizeCashFlowChart = function () {
                //重置容器高宽
                resizeEchart();
                cashFlowChart.resize();
            };

            // display
            //  1 资产构成
            //  2 负债构成
            if ($scope.display === 1) {
                chartsContainer.css("height", 350);
                cashFlowChart.setOption({
                    title: {
                        subtext: '单位(千元)'
                    },
                    dataZoom: {
                        show: true,
                        realtime: true,
                        start: 0,
                        end: 34,
                        //zoomLock: false
                    },
                    grid: {
                        x2: 50,
                        y: 50
                    },
                    tooltip: {
                        trigger: 'axis',
                        formatter: function (params) {
                            return params[0].name + '<br/>'
                                + params[0].seriesName + ' : ' + params[0].value.toString().replace(/(\d{1,2})(?=(\d{3})+\.)/g, '$1,') + ' 千元<br/>'
                                + params[1].seriesName + ' : ' + params[1].value.toString().replace(/(\d{1,2})(?=(\d{3})+\.)/g, '$1,') + ' 千元<br/>'
                        }
                    },
                    color: colourArray,
                    legend: {
                        data: ['预测税额', '实交税额']
                    },
                    xAxis: [
                        {
                            type: 'category',
                            data: $scope.categoryAxisData
                        }
                    ],
                    yAxis: [
                        {
                            type: 'value',
                            name: '金额',
                            axisLabel: {
                                formatter: '{value}'
                            }
                        }

                    ],
                    series: [
                        {
                            name: '预测税额',
                            type: 'bar',
                            barWidth: 20,
                            yAxisIndex: 0,
                            label: {
                                normal: {
                                    show: true,
                                    position: 'top'
                                }
                            },
                            data: $scope.valueAxisData[0]
                        },
                        {
                            name: '实交税额',
                            type: 'bar',
                            barWidth: 20,
                            yAxisIndex: 0,
                            label: {
                                normal: {
                                    show: true,
                                    position: 'top'
                                }
                            },
                            data: $scope.valueAxisData[1]
                        }
                    ]
                });
            }

            if ($scope.display === 2) {
                chartsContainer.css("height", 300);
                cashFlowChart.setOption({
                    tooltip: {
                        trigger: 'axis',
                        formatter: function (params) {
                            return params[0].name + '<br/>'
                                + params[0].seriesName + ' : ' + params[0].value + ' %<br/>'
                                + params[1].seriesName + ' : ' + params[1].value + ' %<br/>'
                        }
                    },
                    legend: {
                        data: [$translate.instant('ReturnRatioMunicipalLevel'), $translate.instant('ReturnRatioDistrictLevel')]
                    },
                    color: colourArray,
                    toolbox: {
                        show: false,
                        feature: {
                            mark: {show: true},
                            dataView: {show: true, readOnly: false},
                            magicType: {show: true, type: ['line', 'bar', 'stack', 'tiled']},
                            restore: {show: true},
                            saveAsImage: {show: true}
                        }
                    },
                    grid: [{
                        left: region === 'zh-CN' ? '10%': '205px'
                    }],
                    calculable: true,
                    xAxis: [
                        {
                            type: 'value'
                        }
                    ],
                    yAxis: [
                        {
                            type: 'category',
                            data: [$translate.instant('OtherTaxes'), $translate.instant('LocalEducationAttach'), $translate.instant('EducationAttach'),
                                $translate.instant('ConstructionTax'), $translate.instant('CIT'), $translate.instant('VAT')]
                        }
                    ],
                    series: [
                        {
                            name: $translate.instant('ReturnRatioMunicipalLevel'),
                            type: 'bar',
                            stack: '总量',
                            itemStyle: {
                                normal: {
                                    label: {
                                        show: true, position: 'insideRight', formatter: function (x) {
                                            if (x.data == 0) {
                                                return '';
                                            }

                                            return x.data + ' %';
                                        }
                                    }
                                }
                            },
                            data: [0.00, 0.00, 0.00, 5.25, 3.75, 5.10]
                        },
                        {
                            name: $translate.instant('ReturnRatioDistrictLevel'),
                            type: 'bar',
                            stack: '总量',
                            itemStyle: {normal: {label: {show: true, position: 'insideRight', formatter: '{c} %'}}},
                            data: [60.00, 40.00, 20.00, 64.75, 50.00, 5.10]
                        }
                    ]
                });
            }

            resizeCashFlowChart();

            //用于使echart自适应宽度
            $(window).bind('resize', resizeCashFlowChart);

            $scope.$on('$destroy', function () {
                $(window).unbind('resize', resizeCashFlowChart);
            });
        }
    };
}]);