﻿frameworkModule.controller('dashboardTaxRiskAlertController', ["$scope", "$compile", "$timeout", "$translate", function ($scope, $compile, $timeout, $translate) {
    'use strict';
    /***************************************** DEMO CODE BEGIN *****************************************/
    var orgList = [
        { "ID": "root", "name": $translate.instant('HeaderCompany'), "expanded": true, "status": "未开始"},
        { "ID": "1", "categoryId": "root", "name": $translate.instant('CompanyA1'), "expanded": true, "status": "已提交"},
        { "ID": "2", "categoryId": 'root', "name": $translate.instant('CompanyA2'), "expanded": true, "status": "已提交" },
        { "ID": "3", "categoryId": 'root', "name": $translate.instant('CompanyA3'), "expanded": true, "status": "未开始" },
        { "ID": "4", "categoryId": 'root', "name": $translate.instant('CompanyA4'), "expanded": true, "status": "未开始" },
        { "ID": "5", "categoryId": 'root', "name": $translate.instant('CompanyA5'), "expanded": true, "status": "未开始" },
        { "ID": "6", "categoryId": "root", "name": $translate.instant('CompanyA6'), "expanded": true, "status": "未开始" }
    ];
    /***************************************** DEMO CODE BEGIN *****************************************/

    var riskLevelColors = [
        {
            name: "Low",
            color: "#feb100"
        },
        {
            name: "Mid",
            color: "#f06000"
        },
        {
            name: "High",
            color: "#b70707"
        }
    ];

    $scope.riskDesc = {
        dataSource: null,
        dxDataGridInstance: null,
        dxDataGridOptions: {
            bindingOptions: {
                dataSource: "riskDesc.dataSource"
            },
            allowColumnResizing: true,
            columns: [
                {
                    alignment: "left",
                    dataField: "taxName",
                    dataType: "string",
                    caption: $translate.instant('TaxType'),
                    width: "11em"
                },
                {
                    alignment: "left",
                    dataField: "riskLevel",
                    dataType: "string",
                    caption: $translate.instant('RiskLevel'),
                    width: "7em"
                },
                {
                    alignment: "left",
                    dataField: "desc",
                    dataType: "string",
                    caption: $translate.instant('RiskDescription'),
                    width: "auto"
                }
            ],
            editing: {
                allowAdding: false,
                allowDeleting: false,
                allowUpdating: false
            },
            focusStateEnabled: true,
            hoverStateEnabled: true,
            keyExpr: null,
            loadPanel: {
                enabled: true
            },
            noDataText: $translate.instant('NoDataText'),
            paging: {
                enabled: false
            },
            rowAlternationEnabled: false, //单双行颜色
            scrolling: {
                mode: "infinite"
            },
            selection: {
                allowSelectAll: false,
                mode: "single"
            },
            showBorders: true,
            showColumnLines: true,
            showRowLines: true,
            sorting: {
                mode: "single"
            },
            onInitialized: function (e) {
                $scope.riskDesc.dxDataGridInstance = e.component;
            },
            onCellPrepared: function (e) {
                if (!(e.rowType === "data" && e.column.dataField === "riskLevel"))
                    return;

                var i = $("<i class='fa fa-exclamation-circle' style='font-size: 1.1em;'></i>");
                i.css("margin-right", "3px");

                var color = _.find(riskLevelColors, function (item) {
                    return item.name == e.data["riskLevel"];
                });
                if (color)
                    i.css("color", color.color);

                $(e.cellElement).empty();
                $(e.cellElement).text($translate.instant(e.data["riskLevel"]));
                $(e.cellElement).prepend(i);
            }
        }
    };

    $scope.riskDesc.dataSource = [
        {
            riskLevel: 'Mid',
            taxName: $translate.instant('VATType'),
            desc: $translate.instant('VATTypeDesc')
        },
        {
            riskLevel: 'High',
            taxName: $translate.instant('CITType'),
            desc: $translate.instant('CITTypeDesc')
        },
        {
            riskLevel: 'Mid',
            taxName: $translate.instant('IITType'),
            desc: $translate.instant('IITTypeDesc')
        },
        {
            riskLevel: 'Mid',
            taxName: $translate.instant('LVATType'),
            desc: $translate.instant('LVATTypeDesc')
        },
        {
            riskLevel: 'Mid',
            taxName: $translate.instant('PropertyTax'),
            desc: $translate.instant('PropertyTaxTypeDesc')
        },
        {
            riskLevel: 'Low',
            taxName: $translate.instant('SpecialTaxType'),
            desc: $translate.instant('SpecialTaxTypeDesc')
        }
    ];

    $scope.selectedOrgIds = ["root"];
    $scope.selectOrgBoxOptions = {
        bindingOptions: {
            value: "selectedOrgIds"
        },
        dataSource: orgList,
        displayExpr: "name",
        valueExpr: "ID",
        placeholder: '. . .',
        showClearButton: false,
        width: '15em',
        treeView: {
            dataSource: orgList,
            dataStructure: "plain",
            focusStateEnabled: false,
            keyExpr: "ID",
            parentIdExpr: "categoryId",
            displayExpr: "name",
            selectByClick: true,
            selectNodesRecursive: false,
            showCheckBoxesMode: "none",
            selectionMode: 'single',
            treeViewInstance: null,
            onContentReady: function (e) {
                e.component.unselectAll();
                if ($scope.selectedOrgIds && $scope.selectedOrgIds.length > 0)
                    e.component.selectItem($scope.selectedOrgIds[0]);
            },
            onItemSelectionChanged: function (args) {
                var value = args.component.getSelectedNodesKeys();
                $scope.selectedOrgIds = value;
            }
        }
    };

    $scope.period = new Date();

    $scope.periodSelectorOption = {
        bindingOptions: {
            value: "period"
        },
        acceptCustomValue: false,
        displayFormat: $translate.instant('dateFormat4YearMonthForDX'),
        maxZoomLevel: "year",
        minZoomLevel: "year",
        min: "2010/1/1",
        max: "2030/12/31",
        placeholder: "",
        width: '15em',
    };

    var checkListConfigs = {
        "businessUnit": {
            name: "businessUnit",
            data: ["事业部1", "事业部2", "事业部3", "事业部4"],
            width: "15em",
            selectedItemKeys: [],
            dropDownBoxInstance: null
        },
        "district": {
            name: "district",
            data: ["区域1", "区域2", "区域3", "区域4"],
            width: "15em",
            selectedItemKeys: [],
            dropDownBoxInstance: null
        },
        "taxCategory": {
            name: "taxType",
            data: [$translate.instant('VATType'), $translate.instant('ConsumptionTax'), '流转税附加税费', '印花税', $translate.instant('PropertyTax'),
                '土地使用税', '车船税', $translate.instant('LVATType'), '契税', '车辆购置税', '关税', $translate.instant('IITType'), $translate.instant('CITType')],
            width: "15em",
            selectedItemKeys: [],
            dropDownBoxInstance: null
        }
    };

    $scope.riskSectorChartOption = {
        data: [
            {
                name: $translate.instant('VATType'),
                value: 50
            },
            {
                name: $translate.instant('CITType'),
                value: 80
            },
            {
                name: $translate.instant('IITType'),
                value: 60
            },
            {
                name: $translate.instant('LVATType'),
                value: 55
            },
            {
                name: $translate.instant('PropertyTax'),
                value: 40
            },
            {
                name: $translate.instant('SpecialTaxType'),
                value: 20
            }
        ]
    };

    $scope.getDropDownBoxOptions = function (controllerName) {
        var checkList = checkListConfigs[controllerName];
        checkList.selectedItemKeys = checkList.data;

        var option = {
            width: checkList.width,
            value: checkList.selectedItemKeys,
            fieldTemplate: "field",
            contentTemplate: "content",
            dataSource: checkList.data,
            onInitialized: function (e) {
                checkList.dropDownBoxInstance = e.component;
            }
        };

        return option;
    };

    $scope.getDropDownListOptions = function (controllerName) {
        var checkList = checkListConfigs[controllerName];

        var option = {
            selectionMode: "multiple",
            dataSource: checkList.data,
            selectedItemKeys: checkList.selectedItemKeys,
            showSelectionControls: true,
            onSelectionChanged: function (e) {
                var selectedItemKeys = e.component.option("selectedItemKeys");
                checkList.selectedItemKeys = selectedItemKeys;
                checkList.dropDownBoxInstance.option("value", selectedItemKeys);
            }
        };

        return option;
    };

    $scope.getDropDownTextBoxOptions = function (controllerName) {
        var checkList = checkListConfigs[controllerName];

        var option = {
            value: checkList.data,
            readOnly: true,
            onValueChanged: function (e) {
                var displayText = "";
                if (checkList.selectedItemKeys.length == checkList.data.length) {
                    switch (checkList.name) {
                        case 'businessUnit':
                            displayText = $translate.instant('AllBusinessUnit');
                            break;
                        case 'district':
                            displayText = $translate.instant('AllArea');
                            break;
                        case 'taxType':
                            displayText = $translate.instant('AllTax');
                            break;
                        default:
                            displayText = $translate.instant("All") + checkList.name;
                            break;
                    }
                }

                if (checkList.selectedItemKeys.length > 0 && checkList.selectedItemKeys.length < checkList.data.length)
                    displayText = checkList.selectedItemKeys.join(",");

                e.component.option("value", displayText);
            },
            onInitialized: function (e) {
                e.component.option("onValueChanged")(e);
            }
        };

        return option;
    };
}]);