﻿citModule.controller('importLogController', ['$scope', '$log', '$translate', '$timeout', '$q', '$interval'
    , 'apiInterceptor', 'Upload', 'vatImportService', 'SweetAlert', 'uiGridConstants', '$uibModal'
    , 'vatSessionService', 'enums',
    function ($scope, $log, $translate, $timeout, $q, $interval
        , apiInterceptor, Upload, vatImportService, SweetAlert, uiGridConstants, $uibModal
        , vatSessionService, enums) {
        'use strict';

        $scope.pleaseSelect = $translate.instant('PleaseSelect');
        $scope.startDate = new Date(new Date().getFullYear() - 20, 1, 1);
        $scope.endDate = new Date(new Date().getFullYear() + 20, 1, 1);
        $scope.viewMode = 'days';

        $scope.period = $scope.periodId;
        $scope.moduleid = enums.vatModuleEnum.Import_TrialBalance;
        $scope.chunkSize = 100000;
        $scope.projectID = vatSessionService.project.id;
        $scope.startRowNum = 2;
        var date = new Date();
        var year = date.getFullYear();
        var month = date.getMonth()+1;
        var day = date.getDate();

        $scope.dateFormat = $translate.instant('dateFormat4YearMonth');

        //初始化ack-pagination
        $scope.pagingOptions = {
            pageIndex: 1, //当前页码
            totalItems: 0, //总数据
            totalPages: 10, //总页数
            maxSize: 5, //分页数字的限制。
            pageSize: constant.page.pageSizeArrary[3], //每页多少条数据,100条
            pageSizeString: constant.page.pageSizeArrary[3].toString(),
            firstPage: $translate.instant('PagingFirstPage'),
            previousPage: $translate.instant('PagingPreviousPage'),
            nextPage: $translate.instant('PagingNextPage'),
            lastPage: $translate.instant('PagingLastPage')
        };

        //初始化查询参数
        $scope.queryParams = {
            pageInfo: {
                pageIndex: 1, //当前页码
                totalItems: 0, //总数据
                totalPages: 10, //总页数
                maxSize: 5, //分页数字的限制。
                pageSize: constant.page.pageSizeArrary[3], //每页多少条数据,100条
                pageSizeString: constant.page.pageSizeArrary[3].toString(),
                firstPage: $translate.instant('PagingFirstPage'),
                previousPage: $translate.instant('PagingPreviousPage'),
                nextPage: $translate.instant('PagingNextPage'),
                lastPage: $translate.instant('PagingLastPage')
            },
            type : constant.importFileType.undefined,
            startDate: year+'/'+month+'/'+day,
            endDate : year+'/'+month+'/'+day
        };

        var loadImportLogInfoDatagrid = function () {
            $scope.importLogGridOptions = {
                bindingOptions: {
                    dataSource: 'ImportLogGridSource'
                },
                showBorders: true,
                columns: [{
                    dataField: "id",
                    visible: false,
                    allowHeaderFiltering: false,
                    caption: $translate.instant('id')
                }, {
                    dataField: "taxpayerIdNum",
                    width: '15%',
                    allowHeaderFiltering: false,
                    caption: $translate.instant('TaxpayerIdNum')
                }, {
                    dataField: "companyCode",
                    allowHeaderFiltering: false,
                    caption: $translate.instant('CompanyCode'),
                    width: '15%'
                }, {
                    dataField: "companyName",
                    allowHeaderFiltering: false,
                    width: '27%',
                    caption: $translate.instant('CompanyName')
                }, {
                    dataField: "periodYear",
                    allowHeaderFiltering: false,
                    width: '5%',
                    caption: $translate.instant('YearPeriod')
                }, {
                    dataField: "periodMonth",
                    allowHeaderFiltering: false,
                    width: '10%',
                    caption: $translate.instant('TMSPeriod')
                }, {
                    dataField: "periodMonth",
                    allowHeaderFiltering: false,
                    width: '5%',
                    caption: $translate.instant('Period')
                }, {
                    dataField: "recordSize",
                    allowHeaderFiltering: false,
                    width: '8%',
                    caption: $translate.instant('RecordSize')
                }, {
                    dataField: "errorMsg",
                    allowHeaderFiltering: false,
                    width: '15%',
                    caption: $translate.instant('Status')
                }
                ],
                onContentReady: function (e) {
                    $scope.accountingRateListInstance = e.component;
                    var totalCount = e.component.totalCount();
                    if (totalCount > 0) {
                        $scope.totalCount = totalCount;
                    }
                },
                loadPanel: {
                    enabled: false
                },
                selection: {
                    mode: "single"
                },
                grouping: {
                    autoExpandAll: false
                },
                allowColumnResizing: true,
                columnAutoWidth: true,
                showRowLines: true,
                showColumnLines: true,
                rowAlternationEnabled: true,  //单双行颜色
                noDataText: $translate.instant('NoDataText'),
                selectAllText: $translate.instant('SelectAll'),
                searchPanel: {
                    placeholder: $translate.instant('Search'),
                    width: 518,
                    visible: true
                },
                headerFilter: {
                    visible: false,
                    texts: {
                        cancel: $translate.instant('Cancel'),
                        ok: $translate.instant('Confirm'),
                        emptyValue: $translate.instant('Empty'),
                        selectAllText: $translate.instant('SelectAll')
                    }
                }
            };
            getImportLog();
        };

        var getImportLog = function () {
            vatImportService.displayImportLogAll($scope.queryParams).success(function (data) {
                if (data && data.list) {
                    $scope.ImportLogGridSource = data.list;
                    $scope.pagingOptions.totalItems = data.pageInfo.totalCount;
                }else {
                    SweetAlert.error($translate.instant('SystemError'));
                }
            });
        };

        var initPagingControl = function (totalItemsCount) {
            $scope.pagingOptions.totalItems = totalItemsCount;
        };

        // 检查用户机构权限
        var checkUserOrganizationPermissionList = function () {
            var list = [];
            var userManageTemp = constant.vatPermission.dataImport.balanceSheet;
            list.push(userManageTemp.importCode);
            $scope.hasImportPermission = false;
            $scope.$root.checkUserOrganizationPermissionList(list).success(function (data) {
                $scope.hasImportPermission = data[userManageTemp.importCode];
            });
        };


        //控制明细信息中删除权限
        var getUserPermission = function () {
            var list = [];
            var code = constant.vatPermission.dataImport.balanceSheet.importCode;
            list.push(code);

            $scope.$root.checkUserOrganizationPermissionList(list).success(function (data) {
                $scope.hasEditPermission = data[code];
            });
        };


        var setButtonWrapStyle = function () {
            if ($scope.fileName) {
                return { width: "100%" };
            }
        };

        var setGridStyle = function () {
            if ($scope.showTotalSecondRow) {
                return { 'margin-top': '60px' }
            }
            else {
                return { 'margin-top': '55px' }
            }
        };

        var initDatePicker = function () {
            var ele = $(".startDatepicker");
            ele.datepicker({
                startDate: $scope.startDate,
                endDate: $scope.endDate,
                viewMode: $scope.viewMode,
                minViewMode: $scope.viewMode,
                autoclose: true, //选中之后自动隐藏日期选择框
                clearBtn: true, //清除按钮
                todayBtn: false, //今日按钮
                format: 'yyyy/mm/dd' //日期格式，详见 http://bootstrap-datepicker.readthedocs.org/en/release/options.html#format
            });
            ele.datepicker("setDate", new Date().formatDateTime('yyyy/mm/dd'));
            var ele2 = $(".endDatepicker");
            ele2.datepicker({
                startDate: $scope.startDate,
                endDate: $scope.endDate,
                viewMode: $scope.viewMode,
                minViewMode: $scope.viewMode,
                autoclose: true, //选中之后自动隐藏日期选择框
                clearBtn: true, //清除按钮
                todayBtn: false, //今日按钮
                format: 'yyyy/mm/dd' //日期格式，详见 http://bootstrap-datepicker.readthedocs.org/en/release/options.html#format
            });
            ele2.datepicker("setDate", new Date().formatDateTime('yyyy/mm/dd'));
        };

        var initParam = function () {
            $scope.initPagingControl = initPagingControl;
            $scope.setButtonWrapStyle = setButtonWrapStyle;
            $scope.setGridStyle = setGridStyle;
            $scope.$watch('queryParams.startDate', function (newValue, oldValue) {
                getImportLog();
            });
            $scope.$watch('queryParams.endDate', function (newValue, oldValue) {
                getImportLog();
            });
        };

        //开始
        (function initialize() {
            $log.debug('ImportLogController.ctor()...');
            initParam();
            checkUserOrganizationPermissionList();
            getUserPermission();
            loadImportLogInfoDatagrid();
            initDatePicker();

            $timeout(function () {
                $scope.isLoadComplete = true;
            }, 500);
        })();
    }
]);