﻿vatModule.controller('vatRevenueDetailController', ['$scope', '$log', '$translate', '$timeout', 'SweetAlert', '$q',
    '$interval', 'dxDataGridService', '$http', 'apiConfig', 'vatSessionService',
    function ($scope, $log, $translate, $timeout, SweetAlert, $q, $interval, dxDataGridService, $http, apiConfig, vatSessionService) {
        'use strict';
        $log.debug('vatRevenueDetailController.ctor()...');

        //表格配置
        $scope.revenueGridOptions = $.extend(true, {}, dxDataGridService.BASIC_GRID_OPTIONS, {
            columns: [

                {
                    dataField: 'serialNo',
                    caption: $translate.instant('RevDetailColSerialNo'),
                    fixed: true,
                    allowHeaderFiltering: true, cellTemplate: function (container, options) {
                        try {
                            $("<span>" + (options.rowIndex + 1) + "</span>")
                                .appendTo(container);
                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
                {
                    dataField: 'subject',
                    caption: $translate.instant('RevDetailColSubject'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'account',
                    caption: $translate.instant('RevDetailColAccount'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'profitCenter',
                    caption: $translate.instant('RevDetailColProfitCenter'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'product',
                    caption: $translate.instant('RevDetailProduct'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'subjectExplain',
                    caption: $translate.instant('RevDetailColSubjectExp'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'accountExplain',
                    caption: $translate.instant('RevDetailColAccountExp'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'profitCenterExplain',
                    caption: $translate.instant('RevDetailColProfitCenterExp'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'productExplain',
                    caption: $translate.instant('RevDetailProductExp'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'amount',
                    caption: $translate.instant('RevDetailAmount'),
                    fixed: true,
                    allowHeaderFiltering: true
                },
                {
                    dataField: 'type',
                    caption: $translate.instant('RevDetailType'),
                    fixed: true,
                    allowHeaderFiltering: true, cellTemplate: function (container, options) {
                        try {
                            if (options.data.emptyCode == 1 || options.data.type == null) {
                                $('<i class="fa fa-exclamation-circle fa-1x" style="color: yellow" aria-hidden="true "></i>&nbsp;&nbsp;')
                                    .appendTo(container);
                            } else {
                                $("<span>" + options.data.type + "</span>")
                                    .appendTo(container);

                            }

                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
                {
                    dataField: 'category',
                    caption: $translate.instant('RevDetailCategory'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    cellTemplate: function (container, options) {
                        try {
                            if (options.data.emptyCode == 1 || options.data.category == null) {
                                $('<i class="fa fa-exclamation-circle fa-1x" style="color: yellow" aria-hidden="true "></i>&nbsp;&nbsp;')
                                    .appendTo(container);
                            } else {
                                switch (options.data.category) {
                                    case "0":
                                        $("<span>货物及加工修理修配劳务</span>")
                                            .appendTo(container);
                                        break;
                                    case  "1":
                                        $("<span>服务、不动产和无形资产</span>")
                                            .appendTo(container);
                                        break;
                                }
                            }

                        } catch (e) {
                            $log.error(e);
                        }
                    }
                },
                {
                    dataField: 'taxOn',
                    caption: $translate.instant('RevDetailTaxOn'),
                    fixed: true,
                    allowHeaderFiltering: true,
                    cellTemplate: function (container, options) {
                        try {
                            if (options.data.emptyCode == 1 || options.data.taxOn == null) {
                                $('<i class="fa fa-exclamation-circle fa-1x" style="color: yellow" aria-hidden="true "></i>&nbsp;&nbsp;')
                                    .appendTo(container);
                            } else {
                                switch (options.data.taxOn) {
                                    case "0":
                                        $("<span>一般计税</span>")
                                            .appendTo(container);
                                        break;
                                    case  "1":
                                        $("<span>简易计税</span>")
                                            .appendTo(container);
                                        break;
                                    case  "2":
                                        $("<span>免抵退税</span>")
                                            .appendTo(container);
                                        break;
                                    case  "3":
                                        $("<span>免税</span>")
                                            .appendTo(container);
                                        break;
                                }
                            }

                        } catch (e) {
                            $log.error(e);
                        }
                    }
                }
            ],
            bindingOptions: {
                dataSource: 'pageConfDataSource'
            }
        });


        //刷新页面
        $scope.refreshGrid = function () {
            $scope.searchParam.projectId = vatSessionService.project.id;
            var month = vatSessionService.month < 10 ? ("0" + vatSessionService.month) : vatSessionService.month;
            $scope.searchParam.period = "" + vatSessionService.year + month;
            $scope.searchParam.queryDate = vatSessionService.year + "-" + month;
            $scope.searchParam.pageInfo = {
                pageIndex: $scope.pagingOptions.pageIndex,
                totalItems: $scope.pagingOptions.totalItems,
                pageSize: $scope.pagingOptions.pageSize
            };
            console.log('searchParam:' + JSON.stringify($scope.searchParam));
            $http.post('/revenueDetail/queryPage', $scope.searchParam, apiConfig.createVat())
                .success(function (res) {
                    if (res && res.list) {
                        $scope.pageConfDataSource = res.list;
                        $scope.pagingOptions.totalItems = res.pageInfo.totalCount;
                    } else {
                        SweetAlert.error($translate.instant('SystemError'));
                    }
                })
        };
        //搜索框的处理
        $scope.searchboxService = {

            showOrHideSearchBox: function () {
                $scope.hasShowMoreSearchBox = !$scope.hasShowMoreSearchBox;
            },
            search: function () {
                $scope.pagingOptions.pageIndex = 1;
                $scope.refreshGrid();
            },
            resetBox: function () {
                $scope.searchParam = {};
            }
        };
        $scope.revenueType = [];
        $scope.getRevenueType = function () {
            var month = vatSessionService.month < 10 ? ("0" + vatSessionService.month) : vatSessionService.month;
            $http.post('/revenueConf/queryAll', {
                startDate: vatSessionService.year + "-" + month,
                endDate: vatSessionService.year + "-" + month,
                orgId: vatSessionService.project.organizationID
            }, apiConfig.createVat())
                .success(function (res) {
                    if (res) {
                        angular.forEach(res, function (d) {
                            $scope.revenueType.push({'key': d.name, 'val': d.name});
                        })
                    } else {
                        SweetAlert.error($translate.instant('GetRevenueTypeError'));
                    }
                })
        };
        (function initialize() {

            //分页的设置
            $scope.pagingOptions = {
                pageIndex: 1,  //当前页码
                totalItems: 0,  //总数据
                pageSize: 20  //每页多少条数据
            };

            $scope.hasShowMoreSearchBox = true;

            $scope.searchParam = {};
            $scope.searchOptions = {
                categoryOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.category'
                    },
                    dataSource: [
                        {'key': '货物及加工修理修配劳务', 'val': 0},
                        {'key': '服务、不动产和无形资产', 'val': 1},
                    ]
                },
                taxOnOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.taxOn'
                    },
                    dataSource: [
                        {'key': '一般计税', 'val': 0},
                        {'key': '简易计税', 'val': 1},
                        {'key': '免抵退税', 'val': 2},
                        {'key': '免税', 'val': 3},
                    ]
                },
                typeOptions: {
                    displayExpr: 'key',
                    valueExpr: 'val',
                    bindingOptions: {
                        value: 'searchParam.type'
                    },
                    dataSource: $scope.revenueType
                }
            };

            $scope.revenueConfAddDiv = ".vat-revenue-config #revenueConfAddDiv";

            function init() {
                $scope.getRevenueType();
                $scope.refreshGrid();
            }

            init()

        })();
    }
]);